#ifndef __SYS_CONF__
#define __SYS_CONF__
/*	Copyright (c) 1984 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"$Revision: 3.11 $"
/*
 * Copyright 1989 Silicon Graphics, Inc.  All rights reserved.
 */
#include "sys/sema.h"

/*
 * Declaration of block device switch. Each entry (row) is
 * the only link between the main unix code and the driver.
 * The initialization of the device switches is in the file conf.c.
 */
struct bdevsw {
	uint	d_flags;
	int	d_cpulock;
	int	(*d_open)();
	int	(*d_close)();
	int	(*d_strategy)();
	int	(*d_print)();
	int	(*d_map)();
	int	(*d_unmap)();
	int	(*d_dump)();
	int	(*d_size)();
};

/*
 * Character device switch.
 */
struct cdevsw {
	uint	d_flags;
	int	d_cpulock;
	int	(*d_open)();
	int	(*d_close)();
	int	(*d_read)();
	int	(*d_write)();
	int	(*d_ioctl)();
	int	(*d_map)();
	int	(*d_unmap)();
	int	(*d_poll)();
	struct	tty *d_ttys;
	struct	streamtab *d_str;
};

#define	DC_OPEN		1
#define DC_CLOSE	2
#define	DC_READ		3
#define DC_WRITE	4
#define DC_IOCTL	5
#define DC_PRINT	6
#define DC_STRAT	7
#define DC_DUMP		8
#define DC_SIZE		9
#define DC_MAP		10
#define DC_UNMAP	11
#define DC_POLL		12

#define bdopen(maj,a1,a2,a3)		bdrv(maj, DC_OPEN, a1, a2, a3)
#define bdstrat(maj,a1)			bdrv(maj, DC_STRAT, a1)
#define bdprint(maj,a1,a2)		bdrv(maj, DC_PRINT, a1, a2)
#define bdclose(maj,a1,a2,a3)		bdrv(maj, DC_CLOSE, a1, a2, a3)
#define bddump(maj,a1,a2,a3,a4,a5)	bdrv(maj, DC_DUMP, a1, a2, a3, a4, a5)
#define bdsize(maj,a1)			bdrv(maj, DC_SIZE, a1)
#define bdmap(maj,a1,a2,a3,a4)		bdrv(maj, DC_MAP, a1, a2, a3, a4)
#define bdunmap(maj,a1)			bdrv(maj, DC_UNMAP, a1)
#define cdopen(maj,dev,a2,a3)		cdrv(maj, DC_OPEN, dev, a2, a3)
#define cdclose(maj,dev,a2,a3)		cdrv(maj, DC_CLOSE, dev, a2, a3)
#define cdread(maj,dev)			cdrv(maj, DC_READ, dev)
#define cdwrite(maj,dev)		cdrv(maj, DC_WRITE, dev)
#define cdioctl(maj,dev,a2,a3,a4)	cdrv(maj, DC_IOCTL, dev, a2, a3, a4)
#define cdmap(maj,dev,a2,a3,a4,a5)	cdrv(maj, DC_MAP, dev, a2, a3, a4, a5)
#define cdunmap(maj,dev,a2)		cdrv(maj, DC_UNMAP, dev, a2)
#define cdpoll(maj,dev,a2)		cdrv(maj, DC_POLL, dev, a2)

/*
 * driver semaphoring types
 * and cdevsw/bdevsw flags
 */
#define DLOCK_MASK	0x7
#define	DLOCK_NONE	0x1	/* driver does own semaphoring */
#define	DLOCK_PROCESSOR	0x2	/* driver executed on master processor */
/* #define DLOCK_xx	0x4	unused */
#define DEVSW_WBACK	0x8	/* write back cache before strategy routine */

#define	FMNAMESZ	8

struct fmodsw {
	char	f_name[FMNAMESZ+1];
	struct	streamtab *f_str;
};


/*file system switch structure */
struct fstypsw {
/* 0*/	int		(*fs_init)();
/* 1*/	int		(*fs_iput)();
/* 2*/	struct inode	*(*fs_iread)();
/* 3*/	int		(*fs_filler)();
/* 4*/	int		(*fs_iupdat)();
/* 5*/	int		(*fs_readi)();
/* 6*/	int		(*fs_writei)();
/* 7*/	int		(*fs_itrunc)();
/* 8*/	int		(*fs_statf)();
/* 9*/	int		(*fs_namei)();
/*10*/	int		(*fs_mount)();
/*11*/	int		(*fs_umount)();
/*12*/	struct inode	*(*fs_getinode)();
/*13*/	int		(*fs_openi)();		/* open inode */
/*14*/	int		(*fs_closei)();		/* close inode */
/*15*/	int		(*fs_update)();		/* update */
/*16*/	int		(*fs_statfs)();		/* statfs and ustat */
/*17*/	int		(*fs_access)();
/*18*/	int		(*fs_getdents)();
/*19*/	int		(*fs_allocmap)();	/* Let the fs decide if */
						/* if can build a map so */
						/* this fs can be used for */
						/* paging */
/*20*/	int		*(*fs_freemap)();	/* free block list */
/*21*/	int		(*fs_readmap)();	/* read a page from the fs */
						/* using the block list */
/*22*/	int		(*fs_setattr)();	/* set attributes */
/*23*/	int		(*fs_notify)();		/* notify fs of action */
/*24*/	int		(*fs_fcntl)();		/* fcntl */
/*25*/	int		(*fs_fsinfo)();		/* additional info */
/*26*/	int		(*fs_ioctl)();		/* ioctl */

/*
 * Extensions.  These extensions are only used if the filesystem
 * shares code with the "com" filesystem.
 */
/*27*/	int		(*fs_dirlookup)();	/* lookup name in directory */
/*28*/	int		(*fs_direnter)();	/* enter name,inum in dir */
/*29*/	int		(*fs_dirremove)();	/* remove a dir entry */
/*30*/	int		(*fs_dirinit)();	/* initialize a new dir */
/*31*/	int		(*fs_dirisempty)();	/* dir empty predicate */
/*32*/	int		(*fs_bmap)();		/* bmap */
/*33*/	struct inode	*(*fs_ialloc)();	/* allocate an inode */
/*34*/	int		(*fs_idestroy)();	/* free inode representation */
/*35*/	int		(*fs_ifree)();		/* free inode */
/*36*/	int		(*fs_setsize)();	/* set file size */

/*
 * Other extensions.
 */
/*37*/	int		(*fs_rmount)();		/* mount root */
/*38*/	int		(*fs_rumount)();	/* unmount root */
/*39*/	int		(*fs_poll)();		/* poll for i/o conditions */
};


/* FS specific data */
struct fsinfo {
	long		fs_flags;	/* flags - see below */
	struct mount	*fs_pipe;	/* The mount point to be used */
					/* as the pipe device for */
					/* this fstyp */
	char		*fs_name; 	/* Pointer to fstyp name */
					/* See above */
	long		fs_notify;	/* Flags for fs_notify */
					/* e.g., NO_CHDIR, NO_CHROOT */
					/* see nami.h */
};

#ifdef _KERNEL

extern struct bdevsw bdevsw[];
extern struct cdevsw cdevsw[];
extern struct fmodsw fmodsw[];
extern struct fstypsw fstypsw[];
extern struct fsinfo fsinfo[];

extern int	bdevcnt;
extern int	cdevcnt;
extern int	fmodcnt;
extern short	nfstyp;

#endif /* _KERNEL */
#endif /* __SYS_CONF__ */
