#ifndef __SYS_FILE_H__
#define __SYS_FILE_H__

#ifdef __cplusplus
extern "C" {
#endif

/**************************************************************************
 *									  *
 * 		 Copyright (C) 1990, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/
/*	Copyright (c) 1984 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"$Revision: 3.23 $"

#include <sys/fcntl.h>

#ifdef _KERNEL
/*
 * One file structure is allocated for each open/creat/pipe call.
 * Main use is to hold the read/write pointer associated with
 * each open file.
 *
 * Locking:
 *	All accesses to the system file table must be protected with
 *	file_tlock.
 *
 *	All share group changes to per process open file tables must be
 *		be protected with shdflock().
 *	Note that accesses to open files (u.u_ofile[i]->f_inode) are NOT
 *	protected, thus f_inode may ONLY be changed BEFORE the file descriptor
 *	becomes public to the share group.
 */

typedef struct file
{
	char	f_flag;
	char	f_msgcount;		/* BSD msg count */
	cnt_t	f_count;		/* reference count */
	union {
		struct inode *f_uinode;	/* pointer to inode structure */
		struct file  *f_unext;	/* pointer to next entry in freelist */
	} f_up;
	off_t	f_offset;		/* read/write character pointer */
} file_t;

#define f_inode		f_up.f_uinode
#define f_next		f_up.f_unext

extern struct file	file[];		/* The file table itself */
extern struct file	*ffreelist;	/* Head of freelist pool */
extern lock_t		file_tlock;	/* lock on file table */

extern struct file	*getf(unsigned);
extern void		closef(struct file *);
extern int		ufalloc(int);
extern void		unfalloc(struct file *);
extern struct file	*falloc(struct inode *, int);
extern int		findfd(dev_t);
extern void		dupit(struct file *, int);
extern void 		forceclose(dev_t);
extern void		addofile(file_t *, int, char);
extern void		delofile(file_t *, int);
extern void		chgofile(file_t *, int, char);
extern void 		dofsync(void);
extern void		dodsync(void);
#endif /* _KERNEL */

/* flags */
#define	FOPEN	0xffffffff
#define	FREAD	0x01
#define	FWRITE	0x02
#define	FMARK	0x20
#define	FDEFER	0x40
#define	FMASK	0xff
#if u3b2
#define FNET	0x80		/* needed by 3bnet */
#endif

/* The following are extensions for 4.3BSD compatibility. */

/*
 * Flock(3) call.
 */
#define	LOCK_SH		1	/* shared lock */
#define	LOCK_EX		2	/* exclusive lock */
#define	LOCK_NB		4	/* don't block when locking */
#define	LOCK_UN		8	/* unlock */

/*
 * Access call.		XXX Also defined in <unistd.h>
 */
#ifndef F_OK
#define	F_OK		0	/* does file exist */
#define	X_OK		1	/* is it executable by caller */
#define	W_OK		2	/* writable by caller */
#define	R_OK		4	/* readable by caller */
#endif

/*
 * Lseek call.	See POSIX equivalents in <unistd.h>
 */
#ifndef L_SET
#define	L_SET		0	/* absolute offset */
#define	L_INCR		1	/* relative to current offset */
#define	L_XTND		2	/* relative to end of file */
#endif

#ifndef _KERNEL
#if defined(_BSD_COMPAT)
extern int flock(int, int);
#endif /* _BSD_COMPAT */
#endif /* _KERNEL */

#ifdef __cplusplus
}
#endif

#endif /* __SYS_FILE_H__ */

