#ifndef __COM_PNCC_H__
#define	__COM_PNCC_H__
/*
 * Copyright 1989 Silicon Graphics, Inc.  All rights reserved.
 *
 * Pathname component cache.
 *
 * The cache key is (parent directory, component name).  Cache value is
 * (child i-number, entry offset, capabilities).  Upon a cache hit, the user
 * may call iget(parent mount, child i-number).  If i-numbers have volatile
 * semantics for a filesystem, that filesystem may use the capabilities
 * to further qualify hits and misses.
 *
 * $Revision: 3.11 $
 */
#include "sys/fstyp.h"			/* for struct dirlookupres */

typedef unsigned short	ncap_t;		/* capability type */

/*
 * Public name cache value structure, an extension of the directory lookup
 * result object.
 */
struct ncvalue {
	struct dirlookupres ncv_dlres;	/* directory lookup result */
	ncap_t		ncv_pcap;	/* parent inumber capability */
	ncap_t		ncv_cap;	/* entry inumber capability */
};
#define	ncv_inum	ncv_dlres.dlr_inum
#define	ncv_offset	ncv_dlres.dlr_offset

struct ncmeter {
	unsigned long
		hits,		/* successful lookups */
		misses,		/* unsuccessful short-name lookups */
		steps,		/* hash chain search loop iterations */
		longlooks,	/* long name lookups (not counted as misses) */
		enters,		/* number of entry creates done */
		longents,	/* enters for long names (failed) */
		removes,	/* number of component names removed */
		devpurges,	/* number of purges by dev */
		purges;		/* number of total purges */
};

struct nclinks {			/* cache block base class */
	struct ncblock	*ncl_forw;	/* next and prev hash links */
	struct ncblock	*ncl_back;
	struct ncblock	*ncl_lruforw;	/* next and prev lru links */
	struct ncblock	*ncl_lruback;
};

/*
 * Each block represents one directory entry.  For the sake of both
 * simplicity and space efficiency, only names shorter than or as long as
 * NC_NAMLEN are cached.  An available block not on any hash chain is
 * indicated by a zero-length name.
 */
#define	NC_NAMLEN	14

struct ncblock {			/* cache linkage must come first */
	struct nclinks	nc_links;		/* hash and LRU links */
	ino_t		nc_pinum;		/* i-number of parent dir */
	dev_t		nc_pdev;		/* mount dev of parent dir */
	char		nc_name[NC_NAMLEN];	/* the entry name */
	struct ncvalue	nc_val;			/* cache value */
};
#define	nc_forw		nc_links.ncl_forw
#define	nc_back		nc_links.ncl_back
#define	nc_lruforw	nc_links.ncl_lruforw
#define	nc_lruback	nc_links.ncl_lruback
#define	nc_pcap		nc_val.ncv_pcap
#define	nc_cap		nc_val.ncv_cap
#define	nc_inum		nc_val.ncv_inum
#define	nc_offset	nc_val.ncv_offset

#ifdef _KERNEL
/*
 * The result parameter filled in by a name cache lookup.
 */
struct ncentry {			/* public entry state */
	struct ncvalue	nce_val;	/* the cache lookup output */
	char		*nce_hash;	/* opaque hashing info */
};
#define	nce_pcap	nce_val.ncv_pcap
#define	nce_cap		nce_val.ncv_cap
#define	nce_dlres	nce_val.ncv_dlres
#define	nce_inum	nce_val.ncv_inum
#define	nce_offset	nce_val.ncv_offset

/*
 * Name cache operations.
 */
void	pncc_init(void);

enum ncstat { NC_MISS, NC_HIT, NC_NOTCACHED };

/*
 * ncstat = pncc_lookup(dp, name, namlen, &nce)
 *	Look for (dp,name) in the cache, returning its value in nce if found,
 *	NULL otherwise.  Unless NULL, the entry yields information to be used
 *	with pncc_enter() or pncc_remove().
 */
enum ncstat pncc_lookup(struct inode *, char *, int, struct ncentry *);

/*
 * ncstat = pncc_enter(dp, name, namlen, &nce);
 *	Make a name cache entry for (dp,name)->(entry->nce_val).  The nce
 *	structure must have been filled in by pncc_lookup().
 */
enum ncstat pncc_enter(struct inode *, char *, int, struct ncentry *);

/*
 * Remove the given entry from the name cache if and only if the capabilities
 * in entry match those of a cache entry.
 */
void	pncc_remove(struct ncentry *);

/*
 * Flush all entries refering to dev from the name cache.  If dev is NODEV,
 * flush the entire cache.  The device name used should be that of a mount
 * structure (not necessarily the same as the inode structure's device).
 */
void	pncc_purgedev(dev_t);

#define	pncc_purge()	pncc_purgedev(NODEV)

#endif	/* _KERNEL */
#endif	/* __COM_PNCC_H__ */
