#ifndef __SYS_GR1_DEV_H__
#define __SYS_GR1_DEV_H__

/*
 * $Revision: 1.91 $
 */

#define GFX_NAME_GR1		"GR1"	/* Unique name for GR1 type boards */

struct gr1_info {
	struct gfx_info gfx_info; 	/* device independent information */
	unsigned char BoardType;	/* see GR1 variants below */
	unsigned char REversion; 	/* 1 or 2 */
	unsigned char Bitplanes; 	/* 8 or 24 */
 	unsigned char Auxplanes;	/* 2 or 4 */
	unsigned char Widplanes;	/* 2 or 4 */
	unsigned char Zplanes;		/* 24 if installed */
	unsigned char Turbo;		/* true if turbo option installed */
	unsigned char SmallMonitor;	/* true if small monitor attached */
	unsigned char Cursorplanes; 	/* 1 or 2 */
	unsigned char VRAM1Meg;		/* 1 if have 1 megabit VRAMs */
	signed char picrev;
};

#define GR1_SYNC_VALUE 2

/* XXX goes in GR1 or CPU? specific header file */
#define GR1_BASE_PHYS	0x1F000000	/* private bus GR1 board family base 
					 * physical addr */

#define GR1_GFX_SIZE	0x3000		/* size of region to map into graphics
					 * process's address space.
					 * VGR regs don't need to be mapped */

/* GR1 variants; -1 means probe failed */
#define GR1_TYPE_NOT_THERE -1		/* probe failed */
#define GR1_TYPE_GR1	0		/* standard Personal Iris GR1 */
#define GR1_TYPE_VGR_PB	1		/* 9U-sized on private bus */
#define GR1_TYPE_UNK	0		/* ie, not initialized */
#define GR1_TYPE_PGR	3		/* 6U-sized (on private bus) */

/* placeholders for boards not currently supported under IRIX */
#define GR1_TYPE_NOT_SUPPORTED \
			GR1_TYPE_NOT_THERE
#define GR1_TYPE_VGR_VME \
			GR1_TYPE_NOT_SUPPORTED	/* 9U-sized on VME bus 
						 * XXX fix HAS_VGR_REGS() when
						 * this is supported */
#define GR1_TYPE_MGR	GR1_TYPE_NOT_SUPPORTED	/* Microchannel version */
#define GR1_TYPE_AGR	GR1_TYPE_NOT_SUPPORTED	/* AT bus version */

/* TRUE if board has the registers in struct gr1_hw prefixed with "vgr" */
#define HAS_VGR_REGS(type) ((type) == GR1_TYPE_VGR_PB || (type) == GR1_TYPE_PGR)

/*
 * The max length of user DMA transfer (might be broken into several DMA
 * transfers in kernel) is entire screen's worth of pixel data.
 */
#define GR1_MAXUDMA	(1280 * 1024 * sizeof(long))

/* Microcode word most sig 8 bits bit definitions */
/*XXX Maybe put these in some other header file? */
#define CX_INCMEM	0x08
#define CX_INCRE	0x20
#define CX_FIELD2	0x10
#define CX_DBSRC	0xC0
#define CX_RDFIFO	0x40


#define SPLRETR()	spl7()
#define SPLGR1()	spl5()


#ifdef _KERNEL
/*
 * Defines used in sggr1_dma()
 */
#define DMAMAXXLEN 	2048	/* Max x length for pixel DMA */
#define DMAMAXYLEN 	2048	/* Max y length for pixel DMA */

/* alloc context dma desc per page of ctxt going in + out */
#define CXDMA_DASIZE	(2 * (CXBIG_NBYTES / NBPP + 1))

/*
 * Per board graphics data.
 * This is only known to the graphics board driver.
 */
struct gr1_download {
	long int	downloading;	/* Boolean: in the middle of */
	long int	codeend;	/* End of code section of file */
	long int	memp;		/* Where we are in HW memory */
	long int	filep;		/* Where we are in file */
};

struct gr1_data {
	struct gfx_data		gfx_data;	/* common gfx driver above */
	struct gr1_info 	*info;		/* gr1 hardware info */
	struct gr1_hw		*base;		/* k1 seg board base address */
	long int		resetdone;	/* boolean: was board reset? */
	sema_t 			dmalock;	/* semaphore for locking gfx DMA */
	unsigned int 		attaches;
	struct gr1_download	download;	/* XXX dynamically allocate this */
	void *			Retrace;
	unsigned long 		loadedmodes[XMAP_NMODEREGS];
	gdmada_t		*cx_da;		/* ctxt dma desc array ptr */
	void *			dmadap;		/* space for DMA descriptor array */
		/* cursor stuff */
	int 			mon_curs_xoff;	/* monitor dep magic offset */
	int			mon_curs_yoff;	
	int			mon_xmaxscreen;	/* monitor dep screen size */
	int			mon_ymaxscreen;
	int			cursor_xoffset;	/* hotspot + magic offset */
	int			cursor_yoffset;
	int 			cursor_x;
	int			cursor_y;
};

struct gr1_cyclemap
{
	unsigned long duration;
	unsigned long nextmap;
};

/*
 * Device DEPENDENT rendering node structure
 */
struct gr1_rnode {
	/* XXX Unfortunately we probably need this */
	long finish0;		/* Saved FINISH0 flag from context switch */

        /* user return buffer to copy out data at interrupt time */
        char *rtnbufu;		/* user virtual address of buffer */
        char *rtnbufk;		/* kernel virtual address of buffer */
        long rtnbuflen;		/* buffer size (bytes) */
        long rtnbufcount;	/* number bytes in buffer */
        long rtnbufoverflow;	/* buffer overflowed (not a flag because we
				   change this at interrupt time, which could
				   get flags out of sync) */

	long swapinterval_wid;	/* needed by cyclemap & swapinterval */
	long swapinterval_buffer;
	
	struct gr1_cyclemap *cyclemap; /* malloced array of 16 when used */	
	unsigned long cyclemap_curmap;
	unsigned long cyclemap_nexttime;

	/* XXX validateclip makes us depend on rrm.h - this should be moved
	 * to the device independent rnode struct and the clip routines made
	 * more device independent
	 */
#ifndef STANDALONE
	struct RRM_ValidateClip validateclip;	/* squirreled-away data for loading during cxt sw (if rnid!=0) */
#endif

	struct rrm_rnode *nextintervalrnp;
};

#endif /* _KERNEL */

/* XXX
 * You might ask "Why the hell these are here?"  Because this is what the
 * GTX and G do and it's a couple of weeks before Cypress MR so I don't
 * have time to fix it.  These should be in a CG3 header file!  Maybe use
 * cg2vme.h.
 */
/* defines for the CG register */
#define CG_MODE_REG		0
#define CG_CONTROL_REG		1
#define CG_HORPHASE_REG		2
#define CG_SUBPHASE_REG		3
#define CG_LASTREG		CG_SUBPHASE_REG

/*
 * flag telling swapbuffers to swap 2 WIDs because we are doing
 * the fast Z clear which uses the LSB of the WID
 * XXX this mask MUST leave the wid positive 
 */
#define GR1_ZCLEAR_USING2WIDS	0x40000000

/*
 *   GR1 specific (private) ioctls
 */
#define GR1_BASE		10000
#define GR1_SETCURSOR		(GR1_BASE+1)
#define GR1_SETCURSORCOLOR	(GR1_BASE+2)
#define GR1_DISPLAYCURSOR       (GR1_BASE+3)
#define GR1_SETCOLOR		(GR1_BASE+4)
#define GR1_SETOVERLAYCOLOR	(GR1_BASE+5)
#define GR1_SETGAMMARAMP	(GR1_BASE+6)
#define GR1_SETDISPLAYMODE	(GR1_BASE+7)
#define GR1_GETCMAP		(GR1_BASE+8)
#define GR1_GETOVERLAYCMAP	(GR1_BASE+9)
#define GR1_GETMODES		(GR1_BASE+10)
#define GR1_PIXELDMA		(GR1_BASE+11)
#define GR1_RTNBUFBEGIN		(GR1_BASE+12)
#define GR1_RTNBUFEND		(GR1_BASE+13)
#define GR1_BLANKSCREEN		(GR1_BASE+14)
#define GR1_CYCLEMAP		(GR1_BASE+15)
#define GR1_BLINK		(GR1_BASE+16)
#define GR1_SETMONITOR		(GR1_BASE+17)
#define GR1_GETMONITOR		(GR1_BASE+18)
#define GR1_RCGREG		(GR1_BASE+19) /* XXX Should be in CG3 driver */
#define GR1_WCGREG		(GR1_BASE+20) /* XXX Should be in CG3 driver */

/*
 *   GR1 ioctl args
 */
struct gr1_colorentry {
	unsigned short index;
	unsigned char red;
	unsigned char green;
	unsigned char blue;
};

struct gr1_setcursor_args {
	int cursorplane ;
	short int xhot ;
	short int yhot ;
	unsigned char glyph[(64*64)/8] ;
};

struct gr1_setcursorcolor_args {
	short numberofcells;
	struct gr1_colorentry colors[1];
};

struct gr1_displaycursor_args {
	int display;
};

struct gr1_setcolor_args {
	short numberofcells;
	struct gr1_colorentry colors[1];    
};

struct gr1_setoverlaycolor_args {
	short numberofcells;
	struct gr1_colorentry colors[1];    
};

struct gr1_setdisplaymode_args {
   	long int wid;
	unsigned long int displaymode;
};

struct gr1_getcmap_args {
	unsigned int start;    
	unsigned int size;    
	unsigned char *buffer;
};

struct gr1_getoverlaycmap_args {
	unsigned int size;    
	unsigned char *buffer;
};

struct gr1_getmodes_args {
	unsigned int size;    
	unsigned int *buffer;
};

/* Flags for graphics DMA */
#define GR1_READ        0x01            /* DMA direction: gfx to host */
#define GR1_WRITE       0x02            /* DMA direction: host to gfx */
#define GR1_LNBYLN      0x04            /* Line by line DMA: 1 line at a time */
#define GR1_SINGLE      0x08            /* Single line DMA (special case) */
#define GR1_LASTDMA	0x10		/* Last DMA chunk */
#define GR1_STRIDE      0x20            /* Stride DMA */

struct gr1_pixeldma_args {
	long token;			/* Token to send down pipe */
	long x;				/* Rectangular coordinate in pixels */
	long xlen;
	long y;
	long ylen;
	unsigned short flags;		/* Bit flags defined below */
	char pixsize;			/* Pixel size (1, 2, 4 bytes) */
	char pmoffset;			/* Number of bits of the first CPU
					   word of each scanline that are
					   to be ignored */
	long pmstride;			/* Number of 32-bit CPU words per
					   scanline */
	char *buf;			/* Data buffer */
	long nbytes;			/* Total bytes of DMA */
	long index;			/* Index to row offsets; used only by
					   rectwrite DMA */
};

struct gr1_rtnbufbegin_args {
        char *buffer;           /* user buffer virtual address */
        long buflen;            /* length of buffer */
};

	/* arbitrary size chosen so we don't lock down too much mem */
#define GR1_MAX_RTNBUF		128 * NBPC	/* Return buf max size (bytes)*/

struct gr1_rtnbufend_args {
        long token;             /* token kernel sends down pipe */
        long overflow;          /* buffer overflow flag */
};

struct gr1_blankscreen_args {
   	int blank;		/* 0 on, 1 off, 2 query */
};

struct gr1_cyclemap_args {
   	unsigned int duration;	/* time between this cycle */
   	unsigned int map;	/* switch from this map */
   	unsigned int nextmap;	/* to this one */
};

struct gr1_setmonitor_args {
	unsigned char monitortype;
	unsigned char flag;
	long xmaxscreen;
	long ymaxscreen;
	long xcursmagic;
	long ycursmagic;
	long topscan;
};

struct gr1_getmonitor_args {
	unsigned char monitortype;
	unsigned char flag;
};

struct gr1_cgreg_args {		/* GR1_RCGREG / GR1_WCGREG arguments */
	unsigned int reg;	/* register number */
	unsigned char value;	/* value */
};

/* Display control flags: */
#define GR1_SM_SYNCGRN	0x01		/* Composite sync on green */
#define GR1_SM_BLANK	0x02		/* Screen is currently off */
#define GR1_SM_STEREO	0x04		/* Stereo optic display enabled */
#define GR1_SM_EXTCLK	0x08		/* Timing locked to external clock
					   (ie. in genlock mode) */

#define GR1_SM_OVERRIDE_TOPSCAN 0x10	/* Use the topscan value supplied with
					   struct gr1_setmonitor_args instead of
					   determining it from monitortype;
					   never set by Gr1GetMonitor */


struct gr1_setgammaramp_args {
	unsigned char red[256];
	unsigned char green[256];
	unsigned char blue[256];
};

#endif /* __SYS_GR1_DEV_H__ */
