/**************************************************************************
 *									  *
 * 		 Copyright (C) 1989, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************
 *
 * This file contains definitions for the kernel portion of the GL
 * input subsystem.
 *
 *
 * $Revision: 1.22 $
 * $Date: 1990/12/11 16:21:00 $
 *
 */

#ifndef NULL
#define NULL	0
#endif 

#define MAX_RETRACE_EVENTS 	80 /* maximum number of timers */

typedef struct retrevent {
	short 	count;	/* clicks left 'til this event is called */
	short	repeat;	/* clicks between events of this type */
	struct inputchan *ic;	/* input channel owning this event */
	short	dnum;
} retrevent;

typedef struct {
	struct inputchan *reserved;
	char 		state;		/* 1 = down; 0 = up */
} buttondata;

typedef struct valuatordata_s {
	struct valuatordata_s *next;
	short	dev;
	short 	value;
	short 	minval;
	short	maxval;
	short	absdevice;
	long 	offset;
	long	raw;
	long	halfmaxraw;
} valuatordata;

typedef struct {
	u_short type;
	short	value;
} queueentry;

typedef struct {
	u_short tiedevice1;	/* valuator device number (or zero) */
	u_short tiedevice2;	/* valuator device number (or zero) */
} procbuttondata;

typedef struct {
	short 	oldvalue;
	short 	noise;
} procvaluatordata;

typedef struct {
	short 	noise;
} proctimerdata;

/*
 * Device hash table
 */

/* table element */
typedef struct {
	u_short	he_dev;
	union	{
		struct	{
			short	doqueue:1;
			union	{
				procvaluatordata vd;
				procbuttondata bd;
				proctimerdata td;
				} d;
			} c;
		valuatordata *gv;
		buttondata gb;
		} he_v;
	} ich_elem_t;

#define he_doqueue	he_v.c.doqueue
#define he_vd		he_v.c.d.vd
#define he_bd		he_v.c.d.bd
#define he_td		he_v.c.d.td

#define he_gv		he_v.gv
#define he_gb		he_v.gb

struct devhash {
	u_short		ich_shift;	/* log2(hash table size) */
	u_short		ich_freecnt;	/* count of free entries */
	ich_elem_t	*ich_base;	/* dynamically allocated hash table */
	ich_elem_t	*ich_limit;	/* end of hash table */
#ifdef OS_METER
	struct ichmeters {
		u_long	im_lookups;	/* hash table lookups */
		u_long	im_probes;	/* linear probes in hash table */
		u_long	im_misses;	/* keys not found */
		u_long	im_adds;	/* entries added */
		u_long	im_grows;	/* table expansions */
		u_long	im_shrinks;	/* table compressions */
		u_long	im_dels;	/* entries deleted */
	} ich_meters;
#define	ich_lookups	ich_meters.im_lookups
#define	ich_probes	ich_meters.im_probes
#define	ich_misses	ich_meters.im_misses
#define	ich_adds	ich_meters.im_adds
#define	ich_grows	ich_meters.im_grows
#define ich_shrinks	ich_meters.im_shrinks
#define	ich_dels	ich_meters.im_dels
#endif	/* OS_METER */
};

#define ACTIVE(x) ((x)->he_dev)

typedef struct inputchan {
	int			ic_pid;
	caddr_t			ic_procp;
	short			ic_state;
	short			ic_num;	/* (minor-1) this inchan's number */
	queueentry 		*ic_queuein,
				*ic_queueout;
	sema_t			ic_queuesema;
	struct devhash		ic_hash;
	short			*ic_siglist;
	int			ic_queue_size;
	queueentry 		ic_queue[1];
} inputchan;

/* states */
#define	ICS_FREE	0x0000		/* cx is unused */
#define	ICS_BUSY	0x0001		/* cx is in use */
#define	ICS_RESERVED	0x0002		/* reserved for allocation */
#define	ICS_OVERFLOW   	0x0004		/* queue events have been lost */
#define ICS_SELECTING	0x0008		/* waiting on select */

#ifdef _KERNEL

/* externals of exported variables */
extern short 		gri_maxinchans;		/* from master.d/gri_dev */
extern short		gri_qsize;		/* from master.d/gri_dev */
extern int		gri_valrate;		/* from master.d/gri_dev */

extern inputchan	 *gl_curric;
extern inputchan 	**gri_inputchannels;

extern lock_t		grilock;
extern lock_t		gridevlock;
extern int		spltty(void);
extern int		splhi(void);

#define IC_USELOCK()	int ic_spl
#define IC_LOCK()	ic_spl = splock(grilock)
#define IC_UNLOCK()	spunlock(grilock,ic_spl)
#define IC_LOCKHI()	ic_spl = splockspl(grilock,spltty)
#define IC_UNLOCKLO()	spunlockspl(grilock,ic_spl)

#define IG_USELOCK()	int ig_spl
#define IG_LOCK()	ig_spl = splock(gridevlock)
#define IG_UNLOCK()	spunlock(gridevlock,ig_spl)
#define IG_LOCKHI()	ig_spl = splockspl(gridevlock,splhi)
#define IG_UNLOCKLO()	spunlockspl(gridevlock,ig_spl)

#define IG_ASSERTLOCKED() SPAPSERT(ownlock(gridevlock))
#define IG_ASSERTUNLOCKED() SPAPSERT(!ownlock(gridevlock))
#define IC_ASSERTLOCKED() SPAPSERT(ownlock(grilock))
#define IC_ASSERTUNLOCKED() SPAPSERT(!ownlock(grilock))
#define ASSERTNOTFROMINTR() ASSERT(private.p_kstackflag != PDA_CURINTSTK)

#include "sys/pda.h"

/* prototypes for exported functions */

void ic_reset(inputchan *);
caddr_t ic_close_l(inputchan *);
void ic_qdevice(inputchan *, u_short);
void ic_unqdevice(inputchan *, u_short);
int  ic_isqueued_l(inputchan *, u_short);
void ic_noise(inputchan *, u_short, short);
void ic_tie(inputchan *, u_short, u_short, u_short);
int  ic_qcheck_l(inputchan *,int);
void ic_qenter_l(inputchan *, u_short, short);
void ic_qreset_l(inputchan *);
int  ic_qtest_l(inputchan *);
long ic_qread(inputchan *, short *);
void ig_init(void);
void ig_open(void);
void ig_adddevice(u_short,u_short);
long ig_getbutton(u_short);
long ig_getvaluator(u_short);
void ig_setvaluator(u_short, short, short, short);
void ig_attachcursor(u_short, u_short);
void ig_detachcursor(void);
void ig_getdev(int, short *);
void ig_changedevice(u_short, short);
void ig_changevaluatorcooked(u_short, long);
void ig_changevaluator_l(u_short, long);
void gl_setcursoroffset(int, int);
void ig_reservebutton(inputchan *, short, u_short);
void ig_unreservebuttons();
void ig_changebutton(u_short, short);
void gl_scrtimeout(long);
void kb_setrepeat(short,short);
void kb_getrepeat(short*,short*);
void mousemoved();
void save_queue(inputchan *, unsigned char *);
void restore_queue(inputchan *, unsigned char *);

void gl_mousewarp(short, short);
void gl_getmousewarp(short *, short *);
#endif 	/* _KERNEL */
