/* GIO bus FDDI board
 *
 * Copyright 1992 Silicon Graphics, Inc.  All rights reserved.
 *
 * This contents of this file must match the corresponding definitions
 *	in xpi/if_xpi_s.h.
 */

#ifndef __IF_XPI_H__
#define __IF_XPI_H__

#ident "$Revision: 1.9 $"


#define XPI_MAXBD   2			/* allow this many boards */

#define XPI_BD_MHZ 16			/* approximate CPU speed */

#define XPI_MAX_OUTQ 64			/* max output queue LENGTH */


/* shape of the board GIO bus as seen by the host
 */

#define	XPI_GIO_RESET	    0x0100	/* set to reset board (latch) */
#define	    XPI_GIO_RESET_DELAY 200	/*	usec delay for reset */
#define	XPI_GIO_FLAG0	    0x0200
#define	XPI_GIO_FLAG1	    0x0400
#define	XPI_GIO_FLAG2	    0x0800
#define	XPI_GIO_FLAG3	    0x1000
#define XPI_GIO_FLAG_MASK   0x1e00
#define	XPI_GIO_H2B_INTR    0x2000	/* write to create intr on board */

#define	XPI_GIO_B2H_CLR	    XPI_GIO_FLAG3   /* clear board->host intr */

/* initial 32 bits of message specifying the host communications area */
#define XPI_GIO_DL_PREAMBLE    0x7ffffffe


/* shape of the board memory as far as the host knows
 */
#define XPI_EPROM	0
#define XPI_SRAM	0x400000
#define XPI_EPROM_SIZE	(128*1024)

/* use SRAM starting here for downloading firmware */
#define	XPI_DOWN	(XPI_SRAM + XPI_EPROM_SIZE)

#define	XPI_FIRM	0
#define	XPI_FIRM_SIZE	(16*1024)

/* the MAC address lives here */
#define	XPI_MAC_SIZE,	16
#define	XPI_MAC_ADDR	(XPI_EPROM_SIZE - XPI_MAC_SIZE)

/* the host driver is stored here */
#define	XPI_DRV		(XPI_FIRM + XPI_FIRM_SIZE)
#define	XPI_DRV_SIZE	(XPI_EPROM_SIZE - XPI_FIRM_SIZE - XPI_MAC_SIZE)

/* the EEPROM programming code is put here */
#define XPI_PROG_SIZE	(1024*2)
#define XPI_PROG	(XPI_DOWN-XPI_PROG_SIZE)


/* Motorola FDDI chip definitions.
 *	These should be moved to another file if there is every any
 *	code which uses them.
 */

/* FSI CAM */
#define FSI_CAM_LEN	32		/* 32 entries */
#define	FSI_CAM_CONST	0xb900		/* 48-bit */
#define FSI_CAM_VALID	0x0080		/* valid entry */
struct fsi_cam {
	ushort	fsi_id;
	ushort	fsi_entry[3];
};

/* MAC packet request header */
#define MAC_DATA_HDR0	0x30		/* any token */
#define MAC_DATA_HDR1	0x38		/* append CRC, same token type */
#define MAC_DATA_HDR2	0x00

#define MAC_BEACON_HDR0	0x04		/* IMMED_MODE */
#define MAC_BEACON_HDR1	0x20		/* append CRC, send no token */
#define MAC_BEACON_HDR2	0x00



/* The host and board communicate mostly with a common area in host memory.
 * The board reads the first part and writes the second part while the host
 * does the opposite.
 *
 */


/* host-to-board commands
 *	values for xpi_hc.cmd
 */
enum xpi_cmd {
	XCMD_NOP=0,			/* 00	do nothing */
	XCMD_INIT,			/* 01	set parameters */
	XCMD_STO,			/* 02	download to the board */
	XCMD_EXEC,			/* 03	execute downloaded code */
	XCMD_FET,			/* 04	upload from the board */

	XCMD_REP,			/* 05	repeat states & disarm PHY */
	XCMD_SETLS0,			/* 06	ELM0 transmitted line state */
	XCMD_SETLS1,			/* 07 */
	XCMD_PC_OFF0,			/* 08	clear ELM PCM */
	XCMD_PC_OFF1,			/* 09 */
	XCMD_PC_START0,			/* 0a	start ELM PCM */
	XCMD_PC_START1,			/* 0b */
	XCMD_SETVEC0,			/* 0c	signal some bits */
	XCMD_SETVEC1,			/* 0d */
	XCMD_LCT0,			/* 0e	start LCT */
	XCMD_LCT1,			/* 0f */
	XCMD_PC_TRACE0,			/* 10	start PC_TRACE */
	XCMD_PC_TRACE1,			/* 11 */
	XCMD_JOIN0,			/* 12	finish signaling */
	XCMD_JOIN1,			/* 13 */
	XCMD_WRAPA,			/* 14 */
	XCMD_WRAPB,			/* 15 */
	XCMD_THRU,			/* 16 */

	XCMD_BYPASS_INS,		/* 17	"insert" the bypass */
	XCMD_BYPASS_REM,		/* 18	turn off the bypass */

	XCMD_MAC_MODE,			/* 19	set MAC mode */
	XCMD_MAC_PARMS,			/* 1a	set MAC parameters */
	XCMD_MAC_OFF,			/* 1b	turn off the MAC */
	XCMD_D_BEACON,			/* 1c	start directed beacon */

	XCMD_PINT_ARM,			/* 1d	arm PHY interrupts */
	XCMD_WAKEUP,			/* 1e	wake up C2B DMA */

	XCMD_FET_LS0,			/* 1f	fetch line ELM0 state */
	XCMD_FET_LS1,			/* 20 */
	XCMD_FET_CT			/* 21	fetch counters */
};


struct xpi_counts {			/* XCMD_FET_CT */
	u_long	ticks;			/* free running usec timer */

	u_long	rngop;			/* I	non-RingOP to RingOP */
	u_long	rngbroke;		/* I	RingOP to non-RingOP */
	u_long	multda;			/*	our DA with A bit set */
	u_long	misfrm;
	u_long	xmtabt;
	u_long	tkerr;			/*	duplicate token */
	u_long	clm;			/* I */
	u_long	bec;			/* I */
	u_long	tvxexp;			/* I */
	u_long	trtexp;			/* I */
	u_long	tkiss;			/*	tokens issued */
	u_long	myclm;
	u_long	loclm;
	u_long	hiclm;
	u_long	mybec;
	u_long	otrbec;

	u_long	frame_ct;		/*	total frames seen */
	u_long	tok_ct;			/*	token count */
	u_long	err_ct;			/*	locatable errors */
	u_long	lost_ct;		/*	format errors */

	u_long	lem0_ct;		/*	LEM "events" from ELM0 */
	u_long	lem1_ct;

	u_long	vio0;			/*	violation symbols from ELM0 */
	u_long	vio1;
	u_long	eovf0;			/*	elasticity buffer overflows */
	u_long	eovf1;
	u_long	elm0st;			/*	ELM status bits */
	u_long	elm1st;

	u_long	error;			/*	error detected by local MAC */
	u_long	e;			/*	received E bit */
	u_long	a;			/*	received A bit */
	u_long	c;			/*	received C bit */
	u_long	rx_ovf;			/*	receive FIFO overrun */
	u_long	tx_under;		/*	transmit FIFO underrun */

	u_long	noise0;			/* I	noise events from ELM0 */
	u_long	noise1;			/* I */

	u_long	tot_junk;		/*	discarded frames */
	u_long	shorterr;		/*	frame too short */
	u_long	longerr;		/*	frame too long */

	enum mac_states mac_state;	/*	current MAC state */

	u_long	dup_mac;		/*	likely duplicate address */

	u_long	rnglatency;		/*	measured ring latency */
	u_long	t_neg;			/*	recent T_Neg */

	u_long	mac_bug;		/* I	firmware bugs */
	u_long	elm_bug;		/* I */
	u_long	fsi_bug;		/* I */
	u_long	mac_cnt_ovf;		/*	too much interrupt latency */

/* I: interesting event which causes an interrupt.
 *	When an interrupt is required by a change in one of these counters,
 *	the interrupt will occur the next time the board interrupt is enabled,
 *	regardless of whether the host has since asked that the counters
 *	be updated.
 */
};



typedef struct xpi_hc {
    /* host-written/board-read portion of the communcations area
     */
    u_long	cmd;			/* one of the xpi_cmd enum */
    u_long	cmd_id;			/* ID used by board to ack cmd */

    union {
	char	cmd_data[64];		/* minimize future version problems */

	struct xpi_dwn {		/* XCMD_STO, XCMD_EXEC, XCMD_FET */
	    u_long  addr;		/* board address */
	    u_long  cnt;		/* number of values */
#define	    XPI_DWN_LEN 8
	    union {			/* values */
		u_long  l[XPI_DWN_LEN];
		u_char	c[XPI_DWN_LEN*4];
	    } val;
	} dwn;

	u_long	    exec;		/* goto downloaded code */

	struct {			/* initialization parameters */
	    u_long  flags;
#		define	XPI_IN_CKSUM 1	/* board IP cksums */
#		define	XPI_IN_CKSUM_SIZE 4096
#		define	XPI_LAT_VOID 2	/* enable latency measures */
	    u_long  b2h_buf;		/* B2H buffer */
	    u_long  b2h_len;		/* B2H_LEN */
	    u_long  max_bufs;		/* maximum total mbufs */
	    u_long  d2b_buf;
	    u_long  d2b_buf_lim;
	    u_long  c2b_buf;
	    u_long  c2b_dma_delay;
	    u_long  int_delay;
	    u_long  max_host_work;
	    u_long  c2b_sleep;
	    u_long  camp;		/* pointer to FSI CAM entries */
	    u_long  ctptr;		/* counters for XCMD_FET_CT */
	} init;

	u_long	    ls;			/* XCMD_SETLS* */

	u_long	    vec_n;		/* XCMD_SETVEC* */

	struct {			/* XCMD_MAC_MODE */
	    u_long  mode;
#		define MMODE_PROM   0x1	/*	promiscuous */
#		define MMODE_AMULT  0x2	/*	all multicasts for routing */
#		define MMODE_HMULT  0x4	/*	CAM is full or want all */
#		define MAX_MULTI_CAM 32	/*	max before all-multi */
#		define MMODE_HLEN   (256/8)
	    u_char  hash[MMODE_HLEN];	/* multicast hash bits */
	} mac;

	struct {			/* XCMD_MAC_PARMS */
	    u_char  addr[6];		/*	long address */
	    u_long  t_req;
	    u_long  r15;		/*	TVX and T_Max */
	} mac_parms;

	struct {			/* XCMD_D_BEACON */
	    struct  lmac_frame msg;
	    u_char  una[2];
	    u_char  pad[2];
	} frame;
    } arg;



    /* host-read/board-written part of the communications area
     */

    volatile unsigned long sign;	/* board signature */
#define XPI_SIGN	0xfdd1f00d
    volatile unsigned long vers;	/* board/eprom version */
    /*			((((yy-92)*13 + m)*32 +  d)*24 + hr)*60 + minute */
#define XPI_MIN_VERS	((((92-92)*13 + 7)*32 + 22)*24 + 0)*60 + 0
#define XPI_VERS_MASK	0x00ffffff
#define XPI_CKSUM_VERS	0x80000000	/*	bad checksum */
#define XPI_BYPASS_VERS	0x40000000	/* have an optical bypass */
#define XPI_MAC2_VERS	0x20000000	/* DAS-DM */
#define XPI_PHY2_VERS	0x10000000	/* DAS */

    volatile unsigned long inum;	/* ++ on board-to-host interrupt */

    volatile unsigned long cmd_ack;	/* ID of last completed command */

    volatile union {
	char	    cmd_res[64];	/* minimize future version problems */

	u_long	    ls;			/* XCMD_FET_LS* */

	struct xpi_dwn dwn;		/* XCMD_FET */
    } res;
};


#define XPI_MAC_SUM(cksum,macp) { int macn;		\
    (cksum) = 0;					\
	for (macn = 0; macn < XPI_MAC_SIZE/4; macn++)	\
		(cksum) += ((u_long*)(macp))[macn];	\
}

/* board-to-host DMA
 */
struct xpi_b2h {
	u_char	b2h_sn;
	u_char	b2h_op;
	ushort	b2h_val;
};
#define XPI_B2H_SLEEP	1		/* board has run out of work */
#define XPI_B2H_ODONE	2		/* output DMA commands finished */
#define XPI_B2H_IN	3		/* input DMA */
#define XPI_B2H_IN_DONE	4
#define XPI_B2H_PHY	5		/* PHY interrupt */

/* received line states, a superset of pcm_ls
 */
enum xi_pcm_ls {
    RCV_TRACE_PROP=8,
    RCV_SELF_TEST =9,
    RCV_BREAK	  =10
};

#define XPI_MAX_DMA_ELEM 7		/* max segments/data output DMA */
#define XPI_PHY_INT_NUM	12

#define XPI_B2H_LEN	((XPI_MAX_SML + XPI_MAX_MED + XPI_MAX_BIG    \
			  + 1 + XPI_MAX_OUTQ*2+1 + XPI_PHY_INT_NUM + 2) | 1)

#define	XPI_PAD_IN_DMA	24		/* input padding added by board */


#define XPI_MAX_BIG	32		/* clusters in pool */
#define XPI_MAX_MED	64		/* medium sized mbufs in pool */
#define XPI_MAX_SML	64		/* little mbufs in pool */
#define XPI_BIG_SIZE	MCLBYTES	/* size of big mbufs */
#define XPI_MED_SIZE (((1514+XPI_PAD_IN_DMA+MSIZE-1)/MSIZE)*MSIZE-32)
#define XPI_SML_SIZE	MLEN

/* host-control-to-board DMA
 */
struct xpi_c2b {
    u_long	c2b_op;
    u_long	c2b_addr;
};
#define XPI_C2B_EMPTY   (0*4)
#define XPI_C2B_SML	(1*4)		/* add little mbuf to pool */
#define XPI_C2B_MED	(2*4)		/* add medium mbuf to pool */
#define XPI_C2B_BIG	(3*4)		/* add cluster to pool */
#define XPI_C2B_PHY_ACK (4*4)		/* finished PHY interrupt */
#define	XPI_C2B_WRAP    (5*4)		/* back to start of buffer */
#define	XPI_C2B_IDELAY  (6*4)		/* delay interrupts */

/* big enough to never be short of space
 *	max output queue + max input queue + max buffer pool
 *		+ N*PHY int acks + sleep note + marker
 */
#define XPI_C2B_LEN	(XPI_MAX_SML + XPI_MAX_MED + XPI_MAX_BIG \
			 + XPI_PHY_INT_NUM + 1 + 1)

/* host-data-to-board DMA
 */
union xpi_d2b {
    struct {
	u_long	len;			/* length of chunk */
	u_long	addr;			/* host address */
    } sg;
    struct {
	u_short start;			/* start-of-request flag */
	u_short	totlen;			/* total frame length */
	u_short	sumstart;		/* start checksum here or 0x8000 */
	u_short sumoff;			/* put it here */
    } hd;
};
#define XPI_D2B_RDY	0x8000		/* in first hd.start of DMA string */
#define XPI_D2B_BAD	0xc000		/* start of un-ready DMA string */

#define XPI_D2B_LEN	(XPI_MAX_OUTQ*(XPI_MAX_DMA_ELEM+1))

#if XPI_MAX_OUTQ != 64
? error ? error ?			/* fix the firmware */
#endif


/* special IOCTLs
 */
#define SIOC_XPI_STO	_IOW('P', 1, struct xpi_dwn)
#define SIOC_XPI_EXEC	_IOW('P', 2, struct xpi_dwn)
#define SIOC_XPI_FET	_IOWR('P', 3, struct xpi_dwn)
#define SIOC_XPI_VERS	_IOR('P', 4, u_long)
#define SIOC_XPI_SIGNAL	_IOW('P', 5, u_long)
#define XPI_SIGNAL_RESET 0x8000

#endif /* __IF_XPI_H__ */
