#ifndef __SYS_IMMU_H__
#define __SYS_IMMU_H__

/**************************************************************************
 *									  *
 * 		 Copyright (C) 1989, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/
/*	Copyright (c) 1984 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"$Revision: 3.76 $"

#include "sys/sema.h"
#include "sys/var.h"

/*
 * Page Descriptor (Table) Entry Definitions for MIPS CPU
 * The Disk Block Descriptor is included in the structure,
 * so pde's are 2 words long. 
 */

typedef struct dbd {
	uint	dbd_watch :  1,	/* is page being watched */
		dbd_type  :  3,	/* The values for this field are given	*/
				/* below.				*/
		dbd_swpi  :  4,	/* The index into swaptab for the	*/
				/* device this page is on if dbd_type	*/
				/* is DBD_SWAP.				*/
		dbd_pgno  : 24;	/* The logical page index into the file.*/
} dbd_t;

/* For the R2000/R3000
 *      	   <- hard bits -> |<-  soft bits ->
 *  +---------------------+-+-+--+-+--+--+--+--+---+
 *  |        pfn          |n|m|vr|g|gs|sv|cw|  | nr|
 *  +---------------------+-+-+--+-+--+--+--+--+---+
 *             20          1 1  1 1  1  1  1  2   3
 *
 * For the R4000
 *       <-             hard bits                ->
 *  +---+--------------------------------+---+-+-+-+
 *  |   |               pfn              | cc|m|v|g|
 *  +---+--------------------------------+---+-+-+-+
 *    2                  24                3  1 1 1
 *
 * For the IP17, we overload the 2 unused bits and
 * the 4 top unused address bits for the software bits:
 *
 *  +--+--+--+---+-----------------------+---+-+-+-+
 *  |gs|sv|cw| nr|         pfn           | cc|m|v|g|
 *  +--+--+--+---+-----------------------+---+-+-+-+
 *    1  1  1  3	    20		   3  1 1 1
 */

/*
 *  pfn	- Page frame number
 *  n	- if set is non-cacheable
 *  m	- modified bit if clear a write will cause fault
 *  vr	- Valid and referenced bit.  If clear a fault will occur. Used as
 *	  reference bit also because there is no HW reference bit
 *  g	- Global so ignore pid in translations
 *  gs	- Global page is swappable.
 *  sv	- software valid bit set if page is valid
 *  cw  - software copy-on-write bit
 *  nr  - Reference count. Set to all ones on reference. Decremented
 *        (not below zero) when page is aged by ageregion.
 *        Pages with lower nr will be taken by getpages.
 *  cc  - (R4000) Cache coherency algorithm.
 */
#ifdef R3000
typedef struct pte {
#ifdef _MIPSEB
	uint	pte_pfn:20,	/* HW: core page frame number or 0 */
		pte_n:1,	/* HW: non-cacheable bit */
		pte_m:1,	/* HW: modified (dirty) bit */
		pte_vr:1,	/* HW: valid bit (& SW referenced)*/
		pte_g:1,	/* HW: ignore pid bit */
		pte_gs:1,	/* SW: global page swappable bit */
		pte_sv:1,	/* SW: page valid bit */
		pte_cw:1,	/* SW: copy on write bit */
		:2,		/* unused */
		pte_nr:3;	/* SW: page reference count */
#endif /* _MIPSEB */
#ifdef _MIPSEL
	uint	pte_nr:3,	/* SW: page reference count */
		:2,		/* unused */
		pte_cw:1,	/* SW: copy on write bit */
		pte_sv:1,	/* SW: page valid bit */
		pte_gs:1,	/* SW: global page swappable bit */
		pte_g:1,	/* HW: ignore pid bit */
		pte_vr:1,	/* HW: valid bit (& SW referenced)*/
		pte_m:1,	/* HW: modified (dirty) bit */
		pte_n:1,	/* HW: non-cacheable bit */
		pte_pfn:20;	/* HW: core page frame number or 0 */
#endif /* _MIPSEL */
} pte_t;
#endif
#ifdef R4000
typedef struct pte {
	uint	pte_gs:1,	/* SW: global page swappable bit */
		pte_sv:1,	/* SW: page valid bit */
		pte_cw:1,	/* SW: copy on write bit */
#if JUMP_WAR
		pte_eop:1,	/* SW: End-Of-Page bit */
		pte_nr:2,	/* SW: page reference count */
#else
		pte_nr:3,	/* SW: page reference count */
#endif
		pte_pfn:20,	/* HW: core page frame number or 0 */
		pte_cc:3,	/* HW: coherency algorithm for this page */
		pte_m:1,	/* HW: modified (dirty) bit */
		pte_vr:1,	/* HW: valid bit (& SW referenced)*/
		pte_g:1;	/* HW: ignore pid bit */
} pte_t;
#endif	/* R4000 */

#if !(defined(R3000) || defined(R4000))
typedef struct pte {
	uint	opaque_pte;
	} pte_t;
#endif

/*	The page table entries are followed by a list of disk block 
 *	descriptors which give the location on disk where a
 *	copy of the corresponding page is found.  If the page
 *	is on swap, it is always a single block.  However, if
 *	it is on a regular file, a single page may correspond
 *	to a number of non-consecutive disk blocks.
 */

typedef union pde {
	struct {
		pte_t	pte;		/* pte structure */
		dbd_t	dbd;		/* dbd structure */
	} pgm;
	struct {
		uint	pg_pde;		/* Full page descriptor (table) entry */
		uint	cnt;		/* generic counter for owner */
	} pgi;
	struct {
		caddr_t	ppt;		/* pointer to page table */
		caddr_t	dummy;
	} pgpt;
} pde_t;

#define pg_pfn	pte.pte_pfn
#define pg_m	pte.pte_m
#define pg_vr	pte.pte_vr
#define pg_g	pte.pte_g
#define pg_gs	pte.pte_gs
#define pg_sv	pte.pte_sv
#define pg_cw	pte.pte_cw
#define pg_nr	pte.pte_nr

#ifdef R3000
#define pg_n	pte.pte_n
#endif
#ifdef R4000
#define pg_cc	pte.pte_cc
#endif

#define pg_ppt	pgpt.ppt

/* For the R4000, a single tlb maps 2 consecutive virtual pages.
 * This structure is used in the upage, for wired entries.
 */
typedef struct {
	pde_t	pde_low;
	pde_t	pde_hi;
} tlbpde_t;

/* convert pointer to pde to pointer to dbd */
#define pdetodbd(x)	((dbd_t *)&((x)->pgm.dbd))

#define	DBD_NONE	0	/* There is no copy of this page on 	*/
				/* disk.				*/
#define	DBD_SWAP	1	/* A copy of this page is on block	*/
				/* number dbd_pgno of the swap file	*/
				/* swptbl[dbd_swpi].			*/
#define	DBD_FILE	2	/* A copy of this page is on the file	*/
				/* described by the inode r_iptr.  The	*/
				/* dbd_pgno field is the logical page	*/
				/* index into the file.			*/
#define	DBD_DZERO	4	/* This is a demand zero page.  No	*/
				/* space is allocated now.  When a 	*/
				/* fault occurs, allocate a page and	*/
				/* initialize it to all zeros.		*/
#define	DBD_DFILL	5	/* This is a demand fill page.  No	*/
				/* space is allocated now.  When a	*/
				/* fault occurs, allocate a page and	*/
				/* do not initialize it at all.  It	*/
				/* will be initialized by reading in 	*/
				/* data from disk.			*/
#define	DBD_MFILE	6	/* This page should be swapped out to	*/
				/* the file described by the inode	*/
				/* r_iptr.  The dbd_pgno field is the	*/
				/* logical page index into the file.	*/
#define DBD_AUTOGROW	7	/* Autogrows file and becomes DBD_FILE	*/
				/* if the page is referenced.		*/

/*
 * segment table
 * This is either an entire page - for a single indirect pointer
 * or a list of segment/page table pointer pairs (so use less space)
 */
typedef struct seg {
	short	s_segno;
	short	s_pad;
	caddr_t	s_ppt;
} seg_t;
#define MAXPSEGS	32		/* these are stored in upage */
#define SEGUNUSED	-1

/* Page descriptor (table) entry dependent constants */

#define NPGPT		512		/* Nbr of pages per page table (seg). */
#define PGPTMASK	0x1ff		/* mask for page table */
#define	NBPP		4096		/* Number of bytes per page */
#define	NBPPT		4096		/* Number of bytes per page table */
#define	BPTSHFT		12 		/* LOG2(NBPPT) if exact */
					/* A disk block is 512 bytes (sector) */
#define NDPP		8		/* Number of disk blocks per page */
#define DPPSHFT		3		/* Shift for disk blocks per page. */

#define PNUMSHFT	12		/* Shift for page number from addr. */
#define POFFMASK        0xFFF		/* Mask for offset into page. */
#define PGFNMASK	0xFFFFF		/* Mask page frame nbr after shift. */
#define SOFFMASK	0x1FFFFF	/* mask for offset into segment (2M) */
#define soff(x)		((uint)(x) & SOFFMASK)

/* Page descriptor (table) entry field masks */

#ifdef R3000
#define PG_ADDR		0xFFFFF000	/* physical page address */
#define PG_PFNUM	PG_ADDR		/* physical page address */
#define PG_N		0x00000800	/* non-cacheable bit */
#define PG_CACHMASK	PG_N		/* Use mask to determine cachability */
#define PG_M		0x00000400	/* modified bit	*/
#define PG_VR		0x00000200	/* valid and referenced  bit */
#define PG_G		0x00000100	/* global (ignore pid)  bit */
#define PG_NDREF	0x00000007	/* reference count (software) */
#define PG_GS		0x00000080	/* software global page swappable bit */
#define PG_SV		0x00000040	/* software valid bit */
#define PG_CW		0x00000020	/* software copy on write bit */

#define PTE_PFNSHFT	12		/* shift pfn to place in pte */
#define PG_NDREFNORM	0		/* normalize NDREF for arithmetic */
#endif
#ifdef R4000

#define	PG_CACHMASK	0x00000038	/* cache coherency algorithm */
#define	PG_CACHSHFT	3		/* position in pte_t */
#define PG_UNCACHED	0x00000010	/* non-cached */
#define PG_N		PG_UNCACHED
#if _RUN_UNCACHED
#define	PG_NONCOHRNT	0x00000010
#define	PG_COHRNT_EXLWR	0x00000010	/* Cacheable coherent exclusive on write */
#else
#define	PG_NONCOHRNT	0x00000018	/* Cacheable non-coherent */
#define	PG_COHRNT_EXLWR	0x00000028	/* Cacheable coherent exclusive on write */
#endif
#define PG_M		0x00000004	/* modified bit	*/
#define PG_VR		0x00000002	/* valid and referenced  bit */
#define PG_G		0x00000001	/* global (ignore pid)  bit */

#if JUMP_WAR
#define PG_EOP		0x10000000	/* end-of-page bit */
#define PG_NDREF	0x0c000000	/* reference count (software) */
#else
#define PG_NDREF	0x1c000000	/* reference count (software) */
#endif

#define PG_GS		0x80000000	/* software global page swappable bit */
#define PG_SV		0x40000000	/* software valid bit */
#define PG_CW		0x20000000	/* software copy on write bit */
#define PG_ADDR		0x03ffffc0	/* physical page address */
#define PG_PFNUM	PG_ADDR		/* physical page address */

#define PG_NDREFNORM	26		/* normalize NDREF for arithmetic */

#define PTE_PFNSHFT	6		/* shift pfn to place in pte */
#endif	/* def R4000 */

/* byte addr to virtual page */

#define pnum(X)   ((uint)(X) >> PNUMSHFT) 

/* page offset */

#define poff(X)   ((uint)(X) & POFFMASK)

/*	Disk blocks (sectors) and pages.
*/
#define	ptod(PP)	((PP) << DPPSHFT)		/* pages to sectors */
#define	dtop(DD)	(((DD) + NDPP - 1) >> DPPSHFT)  /* sectors to pages */
#define	dtopt(DD)	((DD) >> DPPSHFT)  		/* sectors to pages,*/
							/* truncated	    */
/*	Disk blocks (sectors) and bytes.
*/

#define	btod(BB)	(((BB) + NBPSCTR - 1) >> SCTRSHFT)

/*	Form page descriptor (table) entry from modes and page frame number.
*/

#define	mkpde(mode,pfn)	(mode | ((pfn) << PTE_PFNSHFT))

/*	The following macros are used to check the value
 *	of the bits in a page descriptor (table) entry 
 */

/*	Software bits:
*/
#define pg_isvalid(pde) 	((pde)->pgm.pg_sv)

/*	The following macros are used to set the value
 *	of the bits in a page descrptor (table) entry 
 */

/*	Hardware bits:
*/

/*	Set, clear hardware valid bit.
*/
#define pg_ishrdvalid(P)	((P)->pgi.pg_pde & PG_VR)
#define pg_sethrdvalid(P)	((P)->pgi.pg_pde |= PG_VR)
#define pg_clrhrdvalid(P)	((P)->pgi.pg_pde &= ~PG_VR)
#define pg_ismod(P)		((P)->pgm.pg_m)

/*	Set, clear modified (writeable) bit.
*/
#define pg_setmod(P)	((P)->pgi.pg_pde |= PG_M)
#define pg_clrmod(P)	((P)->pgi.pg_pde &= ~PG_M)

/*	Set, clear global (ignore tlb pid) bit.
*/
#define pg_setglob(P)	((P)->pgi.pg_pde |= PG_G)
#define pg_clrglob(P)	((P)->pgi.pg_pde &= ~PG_G)
#define	pg_isglob(P)	((P)->pgi.pg_pde)


/*	Test, set, clear, copy cache attributes.
*/
#ifdef R3000
#define pg_isnoncache(P)	((P)->pgi.pg_pde & PG_N)
#define pg_setnoncache(P)	((P)->pgi.pg_pde |= PG_N)
#define pg_clrnoncache(P)	((P)->pgi.pg_pde &= ~PG_N)
#define pg_setcachable(P)	pg_clrnoncache(P)
#define pg_copycachemode(T,S)	((T)->pgi.pg_pde = \
					((T)->pgi.pg_pde & ~PG_N) | \
					((S)->pgi.pg_pde & PG_N))
#endif
#ifdef R4000
#define	pg_cachemode(P)		((P)->pgi.pg_pde & PG_CACHMASK)
#define	pg_nomode(P)		((P)->pgi.pg_pde & ~PG_CACHMASK)
#define pg_isnoncache(P)	(pg_cachemode(P) == PG_UNCACHED)
#define pg_setnoncache(P)	((P)->pgi.pg_pde = \
					pg_nomode(P) | PG_UNCACHED)
#define pg_setnoncohrnt(P)	((P)->pgi.pg_pde = \
					pg_nomode(P) | PG_NONCOHRNT)
#define pg_copycachemode(T,S)	((T)->pgi.pg_pde = \
					pg_nomode(T) | pg_cachemode(S))
void pg_setcachable(pde_t *P);
#if JUMP_WAR
#define pg_seteop(P)	((P)->pgi.pg_pde |= PG_EOP)
#define pg_iseop(P)	((P)->pgi.pg_pde & PG_EOP)
#endif
#endif

/*	Software bits:
*/

/*	Set, clear copy-on-write bit.
**	Also, clear (set) the modify bit so
**	we'll (won't) get protection violations.
*/
#define pg_setcw(P)	((P)->pgi.pg_pde |= PG_CW); pg_clrmod(P) 
#define pg_clrcw(P)	((P)->pgi.pg_pde &= ~PG_CW); pg_setmod(P)
#define pg_iscw(P)	((P)->pgm.pg_cw)

/*	Note that the valid and refernce bits are the same.
*/
#define pg_setref(P)	pg_setvalid(P)	/* Set ref bit.	*/
#define pg_clrref(P)	pg_clrvalid(P)	/* Clear ref bit.	*/

/*	Set valid bit.
**	Note: due to lack of reference bits, "valid bit" is vr, nr, and sv.
*/
#define pg_setvalid(P)	((P)->pgi.pg_pde |= PG_VR|PG_NDREF|PG_SV)

/*	Clear valid bit.
*/
#define pg_clrvalid(P)	((P)->pgi.pg_pde &= ~(PG_VR|PG_NDREF|PG_SV))

/*	Set, clear soft valid bit.
*/
#define pg_setsftval(P)	((P)->pgi.pg_pde |= PG_SV)
#define pg_clrsftval(P)	((P)->pgi.pg_pde &= ~PG_SV)

/*	Set, clear reference bits.
*/
#define pg_setndref(P)	((P)->pgi.pg_pde |= PG_NDREF)
#define pg_clrndref(P)	((P)->pgi.pg_pde &= ~PG_NDREF)

/*	Query, set and clear ``global page swappable'' bit.
*/
#define pg_isgswappable(P)	((P)->pgi.pg_pde & PG_GS)
#define pg_setgswappable(P)	((P)->pgi.pg_pde |= PG_GS)
#define pg_clrgswappable(P)	((P)->pgi.pg_pde &= ~PG_GS)

/*	Set, clear lock bit.
*/
#define pg_setlock(P)	pfntopfdat((P)->pgm.pg_pfn)->pf_rawcnt++
#define pg_clrlock(P)	{ ASSERT(pfntopfdat((P)->pgm.pg_pfn)->pf_rawcnt > 0); \
			  pfntopfdat((P)->pgm.pg_pfn)->pf_rawcnt--; }

/*	Some 'bit' macros (rather than pde * macros) to
 *	manipulate cache attributes of pte's.
 */
#ifdef R3000
#define	pte_iscached(pte)	(((pte) & PG_N) == 0)
#define	pte_cachebits()		(0)
#define	pte_cached(pte)		((pte) & ~PG_N)
#define	pte_noncached(pte)	((pte) | PG_N)
#endif
#ifdef R4000
#define	pte_iscached(pte)	(((pte) & PG_CACHMASK) != PG_UNCACHED)
uint	pte_cachebits(void);
#define	pte_cached(pte)		(((pte) & ~PG_CACHMASK) | pte_cachebits())
#define	pte_noncached(pte)	(((pte) & ~PG_CACHMASK) | PG_UNCACHED)
#endif	/* def R4000 */


/*	Virtual addresses.
*/
#define REDZONEPAGES	1

/*	First 64K reserved for system - graphics, etc.
*/
#define PRDAADDR	stob(1)		/* process private data area */
#define LOWUSRATTACH	stob(2)		/* lowest addr allowed for USR attach */
#define USRDATA		0x10000000	/* location of user data (256 Mb) */
#if defined(VCE_WAR)
	/* Move the user's top of stack down - to avoid VCE's on
	 * the PRDA, which is mapped at 2 meg.  */
#define USERSTACK	0x7fff8000	/* location of user's stack */
#else
#define USERSTACK	0x7ffff000	/* location of user's stack */
#endif
#define TOPOFSTACK	0x80000000	/* segment top of users stack */
#define HIUSRATTACH	0x80000000	/* highest allowable user address */
#define UADDR		0xffffc000	/* location of U area */
#define KERNELSTACK	0xfffff000	/* location of kernel stack */
#define PDAPAGE		0xffffb000	/* private data area */

#define PRDASIZE	NBPP		/* private proc data area size */
#define EA_SIZE		32		/* EMULATE_AREA size */
#define UVPN		0x000ffffc	/* virtual page number of U */
#define KUSEG		0		/* kuseg user text/data */
#define K0SEG		0x80000000	/* k0seg kernel text/data cached */
#define K1SEG		0xa0000000	/* k1seg kernel text/data uncached */
#define K2SEG		0xc0000000	/* k2seg kernel mapped data */

#ifdef _KERNEL

/*	The following variables describe the memory managed by
**	the kernel.  This includes all memory above the kernel
**	itself.
*/
extern int	kpbase;		/* The address of the start of	*/
				/* the first physical page of	*/
				/* memory above the kernel.	*/
				/* Physical memory from here to	*/
				/* the end of physical memory	*/
				/* is represented in the pfdat.	*/
extern pde_t	*kptbl;		/* Kernel page table.		*/
extern int	maxmem;		/* Maximum available free	*/
				/* memory.			*/
extern int	mmem;		/* Number of resident mapped	*/
				/* file pages.			*/
				/* currently in core.		*/
extern int	mmemlim;	/* Maximum number of resident	*/
				/* mapped file pages.		*/
extern int	freemem;	/* Current free memory.		*/
extern int	usermem;	/* Memory available to users.	*/
extern int	availrmem;	/* Available resident (not	*/
				/* swappable) memory in pages.	*/
extern int	availsmem;	/* Available swappable memory	*/
				/* in pages.			*/
extern int availsmem_accounting;/* availsmem accounting on */
extern ulong	freeswap;	/* free swap space		*/
extern int	availcnt;	/* Waiters for resident memory.	*/
extern lock_t	avail_lock;	/* avail[rs]mem spinlock	*/
extern sema_t	availwait;	/* avail[rs]mem spinlock	*/

#define	availmemlock()		splockspl(avail_lock, splhi)
#define	availmemunlock(T)	spunlockspl(avail_lock, T)

#define	wakermem()			\
{					\
	while (availcnt) {		\
		vsema(&availwait);	\
		availcnt--;		\
	}				\
}

#endif /* _KERNEL */

/*	Conversion macros
*/

/*	convert system address to physical address */
/*	(works only for k0seg and k1seg) */

#define svirtophys(x)	((uint)(x) & ~0xE0000000)

/*	convert pde to physical address */

#define pdetophys(x)	((x)->pgm.pg_pfn<<PNUMSHFT)

/*	convert pde to kernel virtual address */

#define pdetokv(x)	((caddr_t)((x)->pgm.pg_pfn<<PNUMSHFT | 0x80000000))

/*	Get page number from system virtual address.  */

#define	svtop(X)	((uint)svirtophys(X) >> PNUMSHFT) 
#define svtopfn(x)	(svtop(x))

/*	Get page frame number from pde	*/
#define pdetopfn(x)	((x)->pgm.pg_pfn)
#define pdetopfdat(x)	pfntopfdat(pdetopfn(x))

/*	Get system virtual address from page number.  */
/*	(converts pfn to address in k0seg, cached)	*/

#define	ptosv(X)	((caddr_t)(((uint)(X) << PNUMSHFT) | 0x80000000))
#define	pfntosv(X)	((caddr_t)(((uint)(X) << PNUMSHFT) | 0x80000000))
#define phystokv(x)	((caddr_t)((x) | 0x80000000))

#ifdef	_KERNEL

/*	The following routines are involved with the pfdat
**	table described in pfdat.h
*/

#define	kvtopfdat(kv)	(&pfdat[svtop(kv)])
#define	pfntopfdat(pfn)	(&pfdat[pfn])
#define	pfdattopfn(pfd)	(pfd - pfdat)
#define	pfdattophys(pfd)	((pfd - pfdat) << PNUMSHFT)

/*	Test whether a virtual address is in the kernel dynamically
**	allocated area.
*/

#define	iskvir(va)	((caddr_t)(va) >= (caddr_t)K2SEG && \
	(caddr_t)(va) < (caddr_t)(K2SEG + ctob(v.v_syssegsz)))
#endif /* _KERNEL */

/*	Convert dynamically allocated kernel virtual address 
 *	(in k2seg) to address of kernel page table entry.
 *	(dbx needs this so its outside _KERNEL)
 */
#define kvtokptbl(X) (&kptbl[pnum((uint)(X) - (uint)K2SEG)])

/*	Test whether an address is (secondary) cache-aligned.
 *	On a machine with no secondary cache, this always returns
 *	"aligned".
 */
#define SCACHE_ALIGNED(x) (((long)(x) & scache_linemask) == 0)
#ifdef _KERNEL
extern int scache_linemask;
#endif /* _KERNEL */

/*	Flags used by various vm routines.
*/

#define VM_NOSLEEP	0x0001	/* do not wait (sleep) for pages. */
#define VM_TLBNOSET	0x0002	/* do not set wired entries to tlbpid */
#define VM_ONSWAP	0x0004	/* backing store already allocated */
#define VM_FLUSHWIRED	0x0008	/* flush wired pdes. */
#define VM_UNCACHED	0x0010	/* return uncached virtual address */
#define VM_TLBINVAL	0x0020	/* invalidate all tlbpids */
#define VM_TLBVALID	0x0040	/* TLB ids on other processors still valid */
#define VM_UNSEQ	0x0080	/* unsequential access */
#define VM_DIRECT	0x0100	/* direct map -- that is, k0 or k1 seg */
#define VM_PHYSCONTIG	0x0100	/* physically contiguous */
#define VM_ATTACH	0x0200	/* attach page */
#define VM_CACHEALIGN	0x0400	/* guarantee that memory is cache aligned */
#define	VM_STALE	0x4000	/* physical contents can be in other's icache */
#define	VM_LCK		0x10000	/* lock page */
#define VM_VACOLOR	0x20000	/* color virtual addresses */
#define VM_PACOLOR	0x40000	/* color physical addresses */

extern uint cachecolormask;
#if defined(R4000)
#define colorof(address) (((unsigned)(address) >> PNUMSHFT) & cachecolormask)
#endif

/*	Flags used by cache flushing routines.
 */
/* Which cache to flush */
#define	CACH_ICACHE		0x10000
#define	CACH_DCACHE		0x20000

/* cache ops and address types */
#define	CACH_OPMASK		0xff
#define	CACH_INVAL		0x1
#define	CACH_WBACK		0x2

#define	CACH_NOADDR		0x100
#define	CACH_VADDR		0x200

#ifdef _KERNEL

/* Function Prototypes */
extern void	*kvpalloc(uint, int, int);
extern void	*kvalloc(uint, int, int);
extern void	kvpfree(void *, uint);
extern void	kvfree(void *, uint);
extern char	*vmap(paddr_t, uint);
extern void	vunmap(char *);

extern paddr_t	kvtophys(void *);

/* Low level TLB management routines */
extern void unmaptlb(unsigned char, uint);
extern void invaltlb(int);
#ifdef R3000
extern void tlbwired(int, unsigned char, caddr_t, pte_t);
#endif
#ifdef R4000
extern void tlbwired(int, unsigned char, caddr_t, pte_t, pte_t);
#endif
/* Too many places to change
extern void tlbdropin(unsigned char, caddr_t, pte_t);
*/
extern void tlbdropin();
#ifdef R4000
extern void tlbdrop2in(unsigned char, caddr_t, pte_t, pte_t);
#endif
extern void flush_tlb(int);
extern void set_tlbpid(unsigned char);
extern unsigned char tlbgetpid(void);
extern int tlbgetrand(void);

#endif /* _KERNEL */

#endif /* __SYS_IMMU_H__ */
