/* $Revision: 1.9 $ */

/*
 * Key types.  The key type is used by the keyboard decoder to
 * decide what to do with the key.
 */
#define	KB_KEYTYPE_SHIFT	0		/* a shift key */
#define	KB_KEYTYPE_CONTROL	1		/* a control key */
#define	KB_KEYTYPE_ALT		2		/* an alt key */
#define	KB_KEYTYPE_META		3		/* a meta key */
#define	KB_KEYTYPE_CAPSLOCK	4		/* a caps lock key */
#define	KB_KEYTYPE_NUMLOCK	5		/* a num lock key */
#define	KB_KEYTYPE_LETTER	6		/* affected by capslock */
#define	KB_KEYTYPE_PAD		7		/* affected by numlock */
#define	KB_KEYTYPE_OTHER	8		/* other keys */
#define	KB_KEYTYPE_COMPOSE	9		/* compose key */
#define	KB_KEYTYPE_MAX		9

/*
 * Key binding override flag.  If a key has been rebound from its
 * vanilla state, this flag is set.
 */
#define	KB_KEYTYPE_REBOUND	0x80

/*
 * Key binding types.  There are four levels of key binding
 * available.  The binding levels have the following precedense:
 *	highest:	alt
 *			control
 *			shift/capslock/numlock
 *	lowest:		normal
 *
 * If caps lock is enabled, then the shift binding is used
 * for KB_KEYTYPE_LETTER keys.  If num lock is enabled,
 * then the shift binding is used for KB_KEYTYPE_PAD.
 *
 * The bindings do not concatenate, so there is no difference
 * between control-shift-a and control-a.
 *
 * Meta is handled in a special fashion.  After fully decoding the
 * key, if the key generates a single character result, and
 * the meta key is in force, then the upper bit of the key code
 * is set.
 *
 * For all keys, regardless of type, if a binding value exists
 * for the given binding level, it is used.  It is only when no
 * value exists for a binding level that special code comes into
 * play.  Thus, if the left shift key were given a base binding value,
 * and then it was pressed with no other modifier keys down, it would
 * generate that value, and NOT behave as a shift key.
 *
 * Some keys on the keyboard are handled special.  These special
 * rules only come into play as with the other keys - when the key
 * has no assigned value for the given modifier key combination.
 *	- On the 83 key keyboard, if the BREAKKEY is pressed with a base
 *	  binding (i.e., no shifts in force), then a break function is
 *	  generated.
 *
 *	- On the 101 key keyboard, if the PAUSEKEY is pressed with
 *	  a control binding (i.e. control-pause) then a break function
 *	  is generated, not the keys normal value.
 */
#define	KB_BINDING_BASE		0
#define	KB_BINDING_SHIFT	1
#define	KB_BINDING_CONTROL	2
#define	KB_BINDING_ALT		3
#define	KB_BINDING_MAX		3

#ifdef	CVT
typedef struct {
	unsigned char type;
	unsigned char keycode;
	unsigned char *data[KB_BINDING_MAX+1];
} keyboardTable;
#endif

#ifdef	STANDALONE
typedef struct {
	int code;			/* gl device code */
	int type;			/* type of key */
	int len[KB_BINDING_MAX+1];	/* length of binding */
	char *value[KB_BINDING_MAX+1];	/* value of binding */
} keyMap;
#else
typedef struct {
	unsigned short code;			/* gl device code */
	unsigned short type;			/* type of key */
	unsigned char len[KB_BINDING_MAX+1];	/* length of binding */
	unsigned char *value[KB_BINDING_MAX+1];	/* value of binding */
} keyMap;
#endif

typedef struct {
	unsigned char result;			/* result of composition */
	unsigned char c1, c2;			/* characters composed by */
} composeData;

/*
 * All the state that a keyboard has.
 */
typedef struct {
	unsigned char shadow0;		/* shadow state for leds */
	unsigned char shadow1;		/* shadow state for leds */
	unsigned char lightstate;	/* shadow state for leds */
	int	fd;			/* fd that kbd is open on */
	int	type;			/* type of keyboard */
	keyMap	*keyMap;		/* pointer to keyMap array */
	int	numKeys;		/* number of keys in keyMap */
	int	composeCount;
	char	composeStash[2];
	signed char modifier[KB_KEYTYPE_META+1];/* modifier states */
	short	state;			/* other state */
#define	OPENED		0x01
#define	CAPSLOCK	0x02
#define	NUMLOCK		0x04
#define	COMPOSE		0x08
} keyboardState;

/* data for 83 key iris keyboard */
#define KB_REPLY3000	0xAA
#define KB_SGI3000	0xff

/* keyboard types */
#define	KB_TYPE_AMERICAN	0
#define	KB_TYPE_GERMAN		1
#define	KB_TYPE_FRENCH		2
#define	KB_TYPE_ITALIAN		3
#define	KB_TYPE_DANISH		4
#define	KB_TYPE_SPANISH		5
#define	KB_TYPE_NORWEGIAN	6
#define	KB_TYPE_SWEDISH		7
#define	KB_TYPE_SWISS_FRENCH	8
#define	KB_TYPE_UNITED_KINGDOM	9
#define	KB_TYPE_BELGIUM		10
#define	KB_TYPE_SWISS_GERMAN	11
#define	KB_TYPE_DUTCH		12
#define	KB_TYPE_FINNISH		13
#define	KB_TYPE_PORTUGUESE	14
#define	KB_TYPE_GREEK		15

/* functions provided in library */
extern	unsigned char *	kbd_decode(unsigned char keyCode, int *len);
extern	int		kbd_start();
extern	void		kbd_bindkey(unsigned char code);
extern	void		kbd_unbindkey(unsigned char code);
extern	long		kbd_getstate();
