#ifndef __SYS_SIGINFO_H__
#define __SYS_SIGINFO_H__

#pragma once

/*
 * This header is based on preliminary information from the draft
 * MIPS ABI, and is subject to change.
 *
 * The siginfo structure is currently used only by /debug
 */

#define ILL_ILLOPC	1
#define ILL_ILLOPN	2
#define ILL_ILLADR	3
#define ILL_ILLTRP	4
#define ILL_PRVOPC	5
#define ILL_PRVREG	6
#define ILL_COPROC	7
#define ILL_BADSTK	8

#define FPE_INTDIV	1
#define FPE_INTOVF	2
#define FPC_FLTDIV	3
#define FPC_FLTOVF	4
#define FPC_FLTUND	5
#define FPC_FLTRES	6
#define FPC_FLTINV	7
#define FPC_FLTSUB	8

#define SEGV_MAPERR	1
#define SEGV_ACCERR	2

#define BUS_ADRALN	1
#define BUS_ADRERR	2
#define BUS_OBJERR	3

#define TRAP_BRKPT	1
#define TRAP_TRACE	2
#define TRAP_NORLD	10

#define CLD_EXITED	1
#define CLD_KILLED	2
#define CLD_DUMPED	3
#define CLD_TRAPPED	4
#define CLD_STOPPED	5
#define CLD_CONTINUED	6

#define POLL_IN		1
#define POLL_OUT	2
#define POLL_MSG	3
#define POLL_ERR	4
#define POLL_PRI	5
#define POLL_HUP	6

#define SI_MAXSZ	128
#define SI_PAD ((SI_MAXSZ/sizeof(int)) - 3)

typedef struct siginfo {
    int si_signo;
    int si_code;
    int si_errno;

    union {
	int _pad[SI_PAD];
	struct {
	    pid_t pid;
	    union {
		struct { uid_t uid; } _kill;
		struct {
		    clock_t _utime;
		    int     _status;
		    clock_t _stime;
		} _cld;
	    } _pdata;
	} _proc;
	struct { char *_addr; } _fault;
	struct {
	    int _fd;
	    long _band;
	} _file;
    } _data;
} siginfo_t;

#define si_pid		_data._proc._pid
#define si_uid		_data._proc._pdata._cld._uid
#define si_addr		_data._fault._addr
#define si_status	_data._proc._pdata._cld._status
#define si_band		_data._file._band
#endif
