/*
 *	$Revision: 1.16 $
 */

#ifndef __SV1_VIDEO_H_
#define __SV1_VIDEO_H_

enum boolean    {BOOL_TRUE = 1, BOOL_FALSE = 0};
enum direction	{TO_USER, TO_KERNEL};

#define	VIDEO_BOARDS		1

/*
 * These are the top-level ioctl values for use
 * with video.   You do not need to be board owner
 * (that is, have attached the board) to use these.
 */
#define	GFX_V_GETNUM_BOARDS	(GFX_BASE+20)
#define	GFX_V_GETBOARDINFO	(GFX_BASE+21)
#define	GFX_V_ATTACH_BOARD	(GFX_BASE+22)
#define	GFX_V_DETACH_BOARD	(GFX_BASE+23)
#define	GFX_V_IS_MANAGED	(GFX_BASE+24)

/*
 * These are the low-level ioctl values which allow
 * you to set up a particualr board and change the
 * hardware parameters.   You need to have gone 
 * through an attach sequence and become board owner
 * before you can use these.
 */
#define	GFX_V_INITIALIZE	(GFX_BASE+25)
#define	GFX_V_START		(GFX_BASE+26)
#define	GFX_V_DOWNLOAD		(GFX_BASE+27)
#define	GFX_V_MAPALL		(GFX_BASE+28)
#define	GFX_V_LABEL		(GFX_BASE+29)
#define	GFX_V_SETUP		(GFX_BASE+30)
#define	GFX_V_DIMENSIONS	(GFX_BASE+31)
#define	GFX_V_GET_SETUP		(GFX_BASE+32)
#define	GFX_V_LOAD_MAP		(GFX_BASE+33)
#define	GFX_V_GET_MAP		(GFX_BASE+34)
#define	GFX_V_SET_PARAMS	(GFX_BASE+35)
#define	GFX_V_GET_PARAMS	(GFX_BASE+36)
#define	GFX_V_GET_FRAME		(GFX_BASE+37)
#define	GFX_V_TUNE_BOARD	(GFX_BASE+38)
#define	GFX_V_FEATURES		(GFX_BASE+39)
#define	GFX_V_DMA		(GFX_BASE+40)
#define	GFX_V_PUT_FRAME		(GFX_BASE+41)
#define	GFX_V_SET_FORMAT	(GFX_BASE+42)
#define	GFX_V_GET_FORMAT	(GFX_BASE+43)
#define	GFX_V_GET_STATUS	(GFX_BASE+44)
#define GFX_V_USE_EXCLUSIVE	(GFX_BASE+45)

/*
 * XXX for debugging, remove later.
 */
#define GFX_V_DEBUGMAP		(GFX_BASE+110)
#define GFX_V_DUMMY		(GFX_BASE+111)

/*
 * Start of board private commands
 */
#define GFX_V_PRIVATE_BASE	11000
#define	GFX_V_PRIVATE_CEIL	12000

/*
 * Video board description.   This is the struct returned
 * by the GFX_V_GETBOARDINFO ioctl.
 */
#define	GFX_V_INFO_NAME_SIZE	16
#define	GFX_V_INFO_LABEL_SIZE	16

struct gfx_v_info {
	char name[GFX_V_INFO_NAME_SIZE];   /* unique name for this video */
	char label[GFX_V_INFO_LABEL_SIZE]; /* manager's name for this board */
	unsigned int length;		   /* size of a complete info struct */
					   /* for this board */
};
typedef	struct gfx_v_info	gfx_v_Info;

/*
 * The follow arguments struct is passed down with a
 * GFX_V_SETP ioctl.    Only non-zero fields are
 * acted upon, so that you can set or change the values
 * of selected parameters.   You can also read back
 * the current setup into the same struct, and then
 * change selected values.
 */
#define BCST_index	0x00
#define BCST_value	(1 << BCST_index)
#define COLOR_index	0x01
#define COLOR_value	(1 << COLOR_index)
#define BYPASS_index	0x02
#define BYPASS_value	(1 << BYPASS_index)
#define GRAB_index	0x03
#define GRAB_value	(1 << GRAB_index)
#define DROP_index	0x04
#define DROP_value	(1 << DROP_index)
#define DITHER_index	0x05
#define DITHER_value	(1 << DITHER_index)
#define FILTER_index	0x06
#define FILTER_value	(1 << FILTER_index)
#define SLAVE_index	0x07
#define SLAVE_value	(1 << SLAVE_index)
#define RATE_index	0x08
#define RATE_value	(1 << RATE_index)
#define INPUT_index	0x09
#define INPUT_value	(1 << INPUT_index)
#define MODE_index	0x0a
#define MODE_value	(1 << MODE_index)
#define IOMODE_index	0x0b
#define IOMODE_value	(1 << IOMODE_index)
#define	DENCSRC_index	0x0c
#define DENCSRC_value	(1 << DENCSRC_index)
#define	DENCLUTS_index	0x0d
#define DENCLUTS_value	(1 << DENCLUTS_index)
#define	DENCCCIR_index	0x0e
#define DENCCCIR_value	(1 << DENCCCIR_index)
#define	DENCSCBW_index	0x0f
#define DENCSCBW_value	(1 << DENCSCBW_index)
#define	DENCHPLL_index	0x10
#define DENCHPLL_value	(1 << DENCHPLL_index)
#define	DENCNINT_index	0x11
#define DENCNINT_value	(1 << DENCNINT_index)
#define	DENCSCEN_index	0x12
#define DENCSCEN_value	(1 << DENCSCEN_index)
#define	DENCSRSN_index	0x13
#define DENCSRSN_value	(1 << DENCSRSN_index)
#define	DENCGPSW_index	0x14
#define DENCGPSW_value	(1 << DENCGPSW_index)
#define	DENCCOKI_index	0x15
#define DENCCOKI_value	(1 << DENCCOKI_index)
#define	DENCCPR_index	0x16
#define DENCCPR_value	(1 << DENCCPR_index)
#define	DENCKEYE_index	0x17
#define DENCKEYE_value	(1 << DENCKEYE_index)
#define	DENCDD_index	0x18
#define DENCDD_value	(1 << DENCDD_index)
#define	DENCFMT_index	0x19
#define DENCFMT_value	(1 << DENCFMT_index)
#define	DENCSYSEL_index	0x1a
#define DENCSYSEL_value	(1 << DENCSYSEL_index)
#define	DENCSTD_index	0x1b
#define DENCSTD_value	(1 << DENCSTD_index)
#define	DENCVTCR_index	0x1c
#define DENCVTCR_value	(1 << DENCVTCR_index)
#define	GENLOCK3_index	0x1d
#define GENLOCK3_value	(1 << GENLOCK3_index)
#define SETUP_MAX_index	GENLOCK3_index
/*
 *
 * The invalid values in the following enums must be 0.
 */
enum	broadcast_std	{NO_STD = 0, NTSC_STD, PAL_STD};
enum	color_value	{NO_COLOR = 0, COLOR, MONO,
				COLOR_NO_INIT, MONO_NO_INIT};
enum	bypass_value	{NO_BYPASS = 0, BYPASS_ON, BYPASS_OFF};
enum	framegrab_value	{NO_FRAMEGRAB = 0, FRAMEGRAB_ON, FRAMEGRAB_OFF};
enum	fielddrop_value	{NO_FIELDDROP = 0, FIELDDROP_ON, FIELDDROP_OFF};
enum	dither_value	{NO_DITHER = 0, DITHER_ON, DITHER_OFF};
enum	filter_value	{NO_FILTER = 0, FILTER_ON, FILTER_OFF};
enum	slave_value	{NO_SLAVE = 0, SLAVE_ON, SLAVE_OFF};
enum	video_rate	{NO_RATE = 0, RATE_50, RATE_60};
enum	input_type	{NO_INPUT_MODE = 0, TV_MODE, VTR_MODE};
enum	video_mode	{NO_MODE = 0, COMP_MODE, SVIDEO_MODE};
enum	io_mode		{NO_IOMODE = 0, IOMODE_1, IOMODE_2,
				IOMODE_3, IOMODE_4, IOMODE_5a, IOMODE_5B};
enum	denc_src	{NO_DENCSRC = 0, DENCSRC_EXT, DENCSRC_TV2};
enum	denc_fmt	{NO_DENCFMT = 0, FMT_411DMSD2, FMT_411CUSTM,
				FMT_422DMSD2, FMT_422CUSTM, FMT_444YUV,
				FMT_444RGB, FMT_RESERVED, FMT_8BIT_INDEX};
enum	denc_sysel	{NO_DENCSYSEL = 0, SYSEL_CSYNC_LO, SYSEL_HSVS_LO,
				SYSEL_CSYNC_HI, SYSEL_HSVS_HI};
enum	denc_std	{NO_DENCSTD = 0, STD_NTSC4460_SQP, STD_NTSC4450_SQP,
				STD_PALBG50_SQP, STD_NTSC4460_CCIR,
				STD_NTSC4450_CCIR, STD_PALBG50_CCIR,
				STD_PALM60_SQP, STD_RES_1, STD_RES_2,
				STD_PALM60_CCIR, STD_PALM50_CCIR,
				STD_PALM50_SQP, STD_NTSCM60_SQP,
				STD_NTSCM60_CCIR};

struct video_setup_args {
	long			valid_mask; /* says which fields contain
						valid values */
	enum	broadcast_std	BroadcastStd;
	enum	color_value	ColorValue;
	enum	bypass_value	BypassValue;
	enum	framegrab_value	FramegrabValue;
	enum	fielddrop_value	FielddropValue;
	enum	dither_value	DitherValue;
	enum	filter_value	FilterValue;
	enum	slave_value	SlaveValue;
	enum	video_rate	VideoRate;
	enum	input_type	InputType;
	enum	video_mode	VideoMode;
	enum	io_mode		IoMode;
	enum	denc_src	DencSrc;
	enum	boolean		DencLuts;
	enum	boolean		DencCcir;
	enum	boolean		DencScbw;
	enum	boolean		DencHpll;
	enum	boolean		DencNint;
	enum	boolean		DencScen;
	enum	boolean		DencSrsn;
	enum	boolean		DencGpsw;
	enum	boolean		DencCoki;
	enum	boolean		DencCpr;
	enum	boolean		DencKeye;
	enum	boolean		DencDd;
	enum	denc_fmt	DencFmt;
	enum	denc_sysel	DencSysel;
	enum	denc_std	DencStd;
	enum	boolean		DencVtcr;
	enum	boolean		GenlockIn3;
};

/*
 * This struct is passed down with the GFX_V_DIMENSIONS ioctl.
 * It allows you to set the top-left and bottom-right of the
 * video window, and the decimation (which has not yet been
 * implemented).
 */
#define	X_IN_index	0
#define	X_IN_flag	(1 << X_IN_index)
#define	Y_IN_index	1
#define	Y_IN_flag	(1 << Y_IN_index)
#define	DECIMATION_index	2
#define	DECIMATION_flag	(1 << DECIMATION_index)
#define	SOURCE_index	3
#define	SOURCE_flag	(1 << SOURCE_index)
#define	X_OUT_index	4
#define	X_OUT_flag	(1 << X_OUT_index)
#define	Y_OUT_index	5
#define	Y_OUT_flag	(1 << Y_OUT_index)
enum	dim_direction	{NO_DIMENSION = 0, SET_DIMENSION, GET_DIMENSION};
struct video_dimensions_args {
	long			valid_mask; /* says which fields contain
						valid values */
	enum    dim_direction	dim_direction;
	int	x_start, x_end, y_start, y_end;
	int	decimation;
	int	Source;
	int	x_out_start, x_out_end, y_out_start, y_out_end;
};

/*
 * This struct is passed down with the GFX_V_ATTACH_BOARD
 * ioctl.   It allows you to become the owner of the named
 * video board.
 */
struct video_attach_board_args {
	unsigned long int board;
	unsigned long int vaddr;
};

/*
 * This struct is passed down with the GFX_V_TUNE_BOARD
 * ioctl.   It allows you to tune hardware dependent board
 * parameters.    These are not parameters to control the
 * normal operation of the board, but parameters the control
 * the internal operation of the board.  Example: the delay
 * when using the I2C bus.    You need to be the owner of 
 * the named video board before you can use this ioctl.
 */
#define	X_DEBUG		0
#define	X_INTERRUPT	1
#define	X_UNUSED1	2
#define	X_UNUSED2	3
struct video_tune_board_args {
	int		bus_delay;
	int		params[4];
};

#define	HSYb_index	0x00
#define	HSYb_flag	(1 << HSYb_index)
#define	HSYe_index	0x01
#define	HSYe_flag	(1 << HSYe_index)
#define	HCLb_index	0x02
#define	HCLb_flag	(1 << HCLb_index)
#define	HCLe_index	0x03
#define	HCLe_flag	(1 << HCLe_index)
#define	HSps_index	0x04
#define	HSps_flag	(1 << HSps_index)
#define	HUE_index	0x05
#define	HUE_flag	(1 << HUE_index)
#define	CKQAM_index	0x06
#define	CKQAM_flag	(1 << CKQAM_index)
#define	CHCV_index	0x07
#define	CHCV_flag	(1 << CHCV_index)
#define	APER_index	0x08
#define	APER_flag	(1 << APER_index)
#define	COR_index	0x09
#define	COR_flag	(1 << COR_index)
#define	BAND_index	0x0a
#define	BAND_flag	(1 << BAND_index)
#define	AGC_index	0x0b
#define	AGC_flag	(1 << AGC_index)
#define	YDEL_index	0x0c
#define	YDEL_flag	(1 << YDEL_index)
#define	VNOI_index	0x0d
#define	VNOI_flag	(1 << VNOI_index)
#define	PREF_index	0x0e
#define	PREF_flag	(1 << PREF_index)
#define	CTBS_index	0x0f
#define	CTBS_flag	(1 << CTBS_index)
#define	OFTS_index	0x10
#define	OFTS_flag	(1 << OFTS_index)
#define	AUFS_index	0x11
#define	AUFS_flag	(1 << AUFS_index)
#define	HPLL_index	0x12
#define	HPLL_flag	(1 << HPLL_index)
#define	HRFS_index	0x13
#define	HRFS_flag	(1 << HRFS_index)
#define	COLS_index	0x14
#define	COLS_flag	(1 << COLS_index)
#define	DENC_index	0x15
#define	DENC_flag	(1 << DENC_index)
#define	IDEL_index	0x16
#define	IDEL_flag	(1 << IDEL_index)
#define	TEST_index	0x17
#define	TEST_flag	(1 << TEST_index)
#define	PALSEN_index	0x18
#define	PALSEN_flag	(1 << PALSEN_index)
#define	UNUSED0_index	0x19
#define	UNUSED_flag	(1 << UNUSED0_index)
#define	GDC_index	0x1a
#define	GDC_flag	(1 << GDC_index)
#define	DIDEL_index	0x1b
#define	DIDEL_flag	(1 << DIDEL_index)
#define	PSO_index	0x1c
#define	PSO_flag	(1 << PSO_index)
#define	CHPS_index	0x1d
#define	CHPS_flag	(1 << CHPS_index)
#define	FSCO_index	0x1e
#define	FSCO_flag	(1 << FSCO_index)
#define	CHCV_PAL_index	0x1f
#define	CHCV_PAL_flag	(1 << FSCO_index)
#define SET_MAX_index	CHCV_PAL_index

/*
 * This struct is passed down with the GFX_V_SET_PARAMS ioctl.
 * It allows you to set a variety of numeric parameters for
 * the video board.   The first field, valid_mask, should have
 * a '1' bit set in each position corresponding to the
 * parameter to be set.   Parameter values for which the
 * corresponding bit is not set, will be ignored by the driver.
 *
 * NB: When the video board is first init'ed, the numeric
 * parameters are set to typical valid values, and you probably
 * need to be doing some rather special kind of thing to need
 * to set them here.
 */
enum	denc_mode	{NO_DENCMODE = 0, DENC_STANDALONE,
				DENC_GENLOCK, DENC_MASTER};
struct video_params_args {
	int		valid_mask;	/* says which fields contain
						valid values */
	int		HSYbvalue;	/* min 0x00 max 0xff nom 0x30/0x34 */
	int		HSYevalue;	/* min 0x00 max 0xff nom 0x00/60 */
	int		HCLbvalue;	/* min 0x00 max 0xff nom 0xe8/f4 */
	int		HCLevalue;	/* min 0x00 max 0xff nom 0xb6/ce */
	int		HSpsvalue;	/* min 0x00 max 0xff nom 0xf4/0a */
	int		HUEvalue;	/* min 0x00 max 0xff nom 0x00 */
	int		CKQAMvalue;	/* min 0x00 max 0xff nom 0xf8 */
	int		CHCVvalue;	/* min 0x00 max 0xff nom 0x2c */
	int		APERvalue;	/* min 0x00 max 0x03 nom 0x01 */
	int		CORvalue;	/* min 0x00 max 0x03 nom 0x00 */
	int		BANDvalue;	/* min 0x00 max 0x03 nom 0x00 */
	int		AGCvalue;	/* min 0x00 max 0x03 nom 0x00 */
	int		YDELvalue;	/* min 0x00 max 0x07 nom 0x01 */
	int		VNOIvalue;	/* min 0x00 max 0x03 nom 0x00 */
	enum boolean	PREFflag;	/* TRUE/FALSE nom TRUE */
	enum boolean	CTBSflag;	/* TRUE/FALSE nom TRUE */
	enum boolean	OFTSflag;	/* TRUE/FALSE nom TRUE */
	enum boolean	AUFSflag;	/* TRUE/FALSE nom TRUE */
	enum boolean	HPLLflag;	/* TRUE/FALSE nom TRUE */
	enum boolean	HRFSflag;	/* TRUE/FALSE nom TRUE */
	enum boolean	COLSflag;	/* TRUE/FALSE nom TRUE */
	enum denc_mode	DENCMode;
	int		IDELvalue;	/* min 0x00 max 0xff nom 0x51 */
	int		TESTvalue;	/* min 0x00 max 0xffffff      */
	int		PALSENvalue;	/* min 0x00 max 0xff          */
	int		UNUSED0value;	/* min 0x00 max 0xff nom 0x51 */
	int		GDCvalue;	/* min 0x00 max 0xff nom 0x51 */
	int		DIDELvalue;	/* min 0x00 max 0xff nom 0x51 */
	int		PSOvalue;	/* min 0x00 max 0xff nom 0x51 */
	int		CHPSvalue;	/* min 0x00 max 0xff nom 0x51 */
	int		FSCOvalue;	/* min 0x00 max 0xff nom 0x51 */
	int		CHCVPALvalue;	/* min 0x00 max 0xff nom 0x59 */
};

/*
 * The struct is passed down with the GFX_V_GET_MAP or the
 * GFX_V_LOAD_MAP ioctls.
 */
enum map_mode {NO_MAP_MODE = 0,
	MAP_0, MAP_1, MAP_2, MAP_3, MAP_DENC, MAP_CSC, MAP_PIXKEY, MAP_DEBUG = 0x1000};
#define	MAX_COLOR_ENTRIES	256
struct color_entry {
	unsigned short	index;
	int		length;
	unsigned char	*red_map;
	unsigned char	*green_map;
	unsigned char	*blue_map;
};

struct video_load_map_args {
	enum map_mode	map_mode;
	int		entries;	/* map length */
	struct color_entry	entry[1];
};

/*
 * The struct is passed down with the GFX_V_GET_FRAME ioctl,
 * which allows you to grab a YUV frame.
 */
enum framegrab_mode	{NO_FRAMEGRAB_MODE = 0,  FRAMEGRAB_FULL,
				FRAMEGRAB_QUARTER, FRAMEGRAB_PART,
				FRAMEOUT_CANCEL};
struct video_get_frame_args {
	enum framegrab_mode	framegrab_mode;
	caddr_t		buffer;
	caddr_t		lbuffer;	/* for leader.   Ignored if zero */
	int		x_size;
	int		y_size;
	int		x_offset;	/* ignored unless FRAMEGRAB_PART */
	int		y_offset;	/* ignored unless FRAMEGRAB_PART */
	int		*total_size;	/* ingored if NULL */
};

/*
 * The struct is passed down with the GFX_V_FEATURES ioctl,
 * which allows you to find out what features the video board
 * currently supports.
 */
struct video_get_features {
	struct video_features	*features;
};
struct video_features {
	long	features_1;
};

/*
 * The struct is passed down with the GFX_V_DMA ioctl,
 * which allows you to DMA data to and from the video
 * board.
 */
#define	SV1_READ	0x1	/* direction video to memory */
#define	SV1_WRITE	0x2	/* direction memory to video */
enum dma_mode	{DMA_NO_MODE = 0, DMA_RGB_MODE, DMA_YUV_MODE,
	DMA_RGB_TIME_MODE, DMA_YUV_NOFREEZE_MODE,
	DMA_RGB_CIRC_START, DMA_CIRC_END, DMA_YUV_CIRC_START,
	DMA_CIRC_SUSPEND};
#define	DMA_DEBUG		0x00010000
#define	DMA_BLOCKING_MODE	0x00020000
#define	DMA_CIRCULAR_MODE	0x00040000
#define	DMA_BY_FRAMES		0x00080000
#define	DMA_EVEN_FIELDS		0x00100000
#define	DMA_ODD_FIELDS		0x00200000
#define	DMA_ALL_FIELDS		0x00400000
#define DMA_BUFFER_PINNED	0x00800000
#define	DMA_MODE_MASK	0xffff
struct	video_dma_info {
	struct video_dma_info	*next_info;
	int	field_ctr;
	int	hpc_time;
	int	retrace_count;
	int	handshake;
	char	*framebuf;
	char	*blankbuf;
	struct timeval	frame_time[2];
	int	hpc_time_array[4];
};
struct	video_dma_args {
	long		flags;		/* flags to indicate direction. */
	long		xlen;
	long		ylen;
	int		frames;
	char		*buf;		/* Data buffer */
	enum dma_mode	dma_mode;
	int		burst;
	int		delay;
	struct video_dma_info	*info_buffer;
};

/*
 * Used to set standard when switching between sources
 * get and set with GFX_V_SET_FORMAT & GFX_V_GET_FORMAT
 */
#define BROADCAST_flag	0x1
#define VIDEOMODE_flag	0x2
struct	video_type {
	enum	broadcast_std	BroadcastStd;
	enum	video_mode	VideoMode;
};

struct	video_format {
	int			valid_mask;
	struct	video_type	SourceFormat[3];	
};

/*
 * Status directly from hardware, used by GFX_V_GET_STATUS
 */
struct	video_status {
	enum	boolean		Signal;
	enum	boolean		Color;
	enum	broadcast_std	BroadcastStd;	
};

#define VID_XMAX                0x140
#define VID_YMAX                0x1E0

#define VID_XSTART              0xF8
#define VID_XSTOP               (VID_XSTART + VID_XMAX)

#define VID_YSTART              0x5B
#define VID_YSTOP               (VID_YSTART + VID_YMAX)


#define	NTSC_MAX_XSIZE	640
#define	NTSC_MAX_YSIZE	480
#define	PAL_MAX_XSIZE	768
#define	PAL_MAX_YSIZE	576

#define MAX_XSIZE	PAL_MAX_XSIZE
#define MAX_YSIZE	PAL_MAX_YSIZE

#define	SCREEN_MIN_X	2
#define	SCREEN_MIN_Y	0
#define	SCREEN_MAX_X	616
#define	SCREEN_MAX_Y	900

#define VERT_BLANKING   (MAX_XSIZE * 11)
#define MAX_YUV_DATA_SIZE   (VERT_BLANKING + (MAX_XSIZE * (MAX_YSIZE / 2)))

#define YUV_DATA_SIZE(b,x,y)    (b + (x * (y / 2)))
#define YUV_VIS_DATA_SIZE(x,y)    (x * (y / 2))

/* getvid defines */
#define SV1_DISP	0
#define SV1_WIN		1

#endif /* __SV1_VIDEO_H_ */
