
/*
 * User GPIB Include File.  Used to interface to the National Instruments
 * GPIB controller.  It is NOT used by the kernel, and duplicates some
 * of the defines in sys/gpib.h and sys/gpibib.h, but also has some
 * unique defines, etc.
 *
 * Two manuals may be useful if you work with this device:
 *	"National Instruments IEEE-488 Multitasking UNIX Device Driver
 *		User Manual" Part #320062-01
 *		and
 *	"National Instruments GPIB-1014 User Manual" Part #320030-01
 *		Both from Nat'l Instr. at 12109 Technology Blvd., Austin, Texas
 *			(512) 250-9119
 *
 *		The first describes the software interface user programs see,
 *		the second describes the hardware and it's kernel interface.
 */
#ident "ugpib.h $Revision: 1.4 $ $Date: 1990/12/11 17:07:30 $ $Author: jfw $ "

typedef short word_t;     /* two byte object */

/* structure exits one per gpib board in system */
struct board {
        word_t  b_uflags;          /* user flags (see below) */
        struct ibregs * b_addr;    /* I/O port address */
        char    b_pad,b_sad;       /* primary and secondary gpib address */
        char    b_eos;             /* end-of-string character */
        word_t  b_tmo;             /* timeout value (see below) */
        char    b_ppe;             /* parallel poll enable byte */
        char    b_irq;             /* interrupt level */
        char    b_ivec;            /* interrupt vector number */
        char    b_sig;             /* signal to send */
};

/* board user flags (b_uflags) */
#define EOT    	000001  /* assert EOI with last byte of each write */
#define HLD    	000002  /* holdoff handshake at end of each read */
#define EOSM   	000034  /* end-of-string modes */
#define  REOS  	000004  /*  terminate read on eos */
#define  XEOS  	000010  /*  assert EOI with eos byte */
#define  BIN   	000020  /*  eight-bit compare */
#define TDCL   	000040  /* terminate I/O on device clear */
#define TTICK  	000400  /* if set, b_tmo is ticks not scale factor */
#define IST    	001000  /* individual status bit */
#define SRE    	002000  /* assert REN when SC */
#define SC     	004000  /* interface is system controller */
#define TRI    	010000  /* use tri-state timing */
#define NOPOLL	020000 	/* do not perform automatic serial polls */

struct device {
        word_t d_uflags;     /* (see below) */
        char d_bna;       /* access board number */
        char d_pad,d_sad; /* primary and secondary gpib address */
        char d_eos;       /* end-of-string character */
        char d_tmo;       /* timeout value */
        char d_ppe;       /* parallel poll enable byte */
};

/* device user flags (d_uflags) */
/*      EOT     000001  /* assert EOI with last byte of each write */
/*      HLD     000002  /* holdoff handshake at end of each read */
/*      EOSM    000034  /* end-of-string modes */
/*       REOS   000004  /*  terminate read on eos */
/*       XEOS   000010  /*  assert EOI with eos byte */
/*       BIN    000020  /*  eight-bit compare */
/*      TDCL    000040  /* terminate I/O on device clear */

/* GPIB status bit vector */

#define ERR     (1<<15) /* Error detected */
#define TIMO    (1<<14) /* timeout */
#define END     (1<<13) /* EOI or eos detected */
#define SRQI    (1<<12) /* SRQ detected by CIC */
#define SRQ	SRQI
#define RQS     (1<<11) /* Device requires service */
#define ADSC	(1<<10) /* address status change */
#define CMPL    (1<<8)  /* DMA completed (SH/AH synch'd) */
#define LOK     (1<<7)  /* local lockout state */
#define REM     (1<<6)  /* remote state */
#define CIC     (1<<5)  /* controller-in-charge */
#define ATN     (1<<4)  /* attention asserted */
#define TACS    (1<<3)  /* talker active */
#define LACS    (1<<2)  /* listener active */
#define DTAS    (1<<1)  /* device trigger state */
#define DCAS    (1<<0)  /* device clear state */

/* Error messages */

#define EDVR  0  /* System error */
#define ECIC  1  /* Not CIC (or lost CIC during command) */
#define ENOL  2  /* write detected no listeners */
#define EADR  3  /* board not addressed correctly */
#define EARG  4  /* bad argument to function call */
#define ESAC  5  /* function requires board to be SAC */
#define EABO  6  /* asynchronous operation was aborted */
#define ENEB  7  /* non-existent board */
#define EDMA  8  /* DMA hardware error detected */
#define EBTO  9  /* DMA hardware uP bus timeout */
#define EOIP 10  /* new I/O attempted with old I/O in progress  */
#define ECAP 11  /* no capability for intended opeation         */
#define EFSO 12  /* file system operation error                 */
#define EOWN 13  /* shareable board exclusively owned */
#define EBUS 14  /* bus error */
#define ESTB 15  /* serial poll queue overflow */
#define ESRQ 16  /* SRQ line 'stuck' on */

/* Timeout values and meanings                          */
#define TNONE    0   /* Infinite timeout (disabled)     */
#define T10us    1   /* Timeout of 10 us (ideal)        */
#define T30us    2   /* Timeout of 30 us (ideal)        */
#define T100us   3   /* Timeout of 100 us (ideal)       */
#define T300us   4   /* Timeout of 300 us (ideal)       */
#define T1ms     5   /* Timeout of 1 ms (ideal)         */
#define T3ms     6   /* Timeout of 3 ms (ideal)         */
#define T10ms    7   /* Timeout of 10 ms (ideal)        */
#define T30ms    8   /* Timeout of 30 ms (ideal)        */
#define T100ms   9   /* Timeout of 100 ms (ideal)       */
#define T300ms  10   /* Timeout of 300 ms (ideal)       */
#define T1s     11   /* Timeout of 1 s (ideal)          */
#define T3s     12   /* Timeout of 3 s (ideal)          */
#define T10s    13   /* Timeout of 10 s (ideal)         */
#define T30s    14   /* Timeout of 30 s (ideal)         */
#define T100s   15   /* Timeout of 100 s (ideal)        */
#define T300s   16   /* Timeout of 300 s (ideal)        */
#define T1000s  17   /* Timeout of 1000 s (maximum)     */

extern unsigned int ibsta,iberr,ibcnt;

/* ioctl codes */
#define IIOC    ('I'<<8)

/* IBGET/IBSET use the form: ioctl(f,IBGET,&board); */
#define IBGET    (IIOC|0)       /* get board/device structure */
#define IBSET    (IIOC|1)       /* set board/device structure */

#define IBINB    (IIOC|2)       /* special */
#define IBOUTB   (IIOC|3)       /* special */

struct ibinfo {
        char nbrds,      /* number of boards */
             ndevs,      /* number of devcies */
             isdev;     /* non-zero if called with device file descriptor */
};
/* IBINFO uses the form: ioctl(f,IBINFO,&ibinfo); */
#define IBINFO   (IIOC|4)       /* get board and device info */

struct ibarg {
        long     buf;           /* buffer address */
        unsigned n;             /* count or argument */
        unsigned ibsta;         /* return status */
        unsigned iberr;         /* return error code */
        unsigned ibcnt;         /* return count */
};

/* the following codes use the form: ioctl(f,IBRD,&ibarg);  */
#define IBRD    (IIOC|10)
#define IBWRT   (IIOC|11)
#define IBCMD   (IIOC|12)
#define IBXTRC  (IIOC|14)
#define IBRSP   (IIOC|15)
#define IBRPP   (IIOC|16)
#define IBBNA   (IIOC|17)
#define IBWAIT  (IIOC|18)
#define IBONL   (IIOC|19)
#define IBRSV   (IIOC|20)
#define IBGTS   (IIOC|21)
#define IBCAC   (IIOC|22)
#define IBPOKE  (IIOC|23)
#define IBSGNL  (IIOC|24)
#define IBSRE   (IIOC|25)
#define IBTRG   (IIOC|26)
#define IBCLR   (IIOC|27)
#define IBLOC   (IIOC|28)
#define IBPCT   (IIOC|29)
#define IBSIC   (IIOC|30)
#define IBIOAB  (IIOC|31)
#define IBPPC   (IIOC|32)
#define IBRSC   (IIOC|33)
#define IBIST   (IIOC|34)
#define IBEOT   (IIOC|35)
#define IBPAD   (IIOC|36)
#define IBSAD   (IIOC|37)
#define IBDMA   (IIOC|38)
#define IBEOS   (IIOC|39)
#define IBTMO   (IIOC|40)
#define IBWRTF  (IIOC|41)
#define IBRDF   (IIOC|42)
#define IBLLO   (IIOC|43)
