/**************************************************************************
 *									  *
 * 		 Copyright (C) 1992, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/

/*
 * $Revision: 1.177 $
 */

#ifndef _VENICE_H_
#define _VENICE_H_

#include "sys/param.h"
#include "sys/sysmacros.h"

#define VENICE_3WAY_SIZE	0x2000	/* size of 3way hardware to map */
#define VENICE_FIFO_SIZE	0x1000	/* size of fifo hardware to map */
#define VENICE_PRIV_SIZE	0x1000	/* size of priviledged fifo for diags */
#define VENICE_MPG_SIZE		0x1000	/* size of MPG regs for diags */
#define VENICE_RDATA_SIZE	0x2000	/* size of return data area */
#define VENICE_MAP_SIZE		(VENICE_3WAY_SIZE+VENICE_FIFO_SIZE+\
				 VENICE_PRIV_SIZE+VENICE_MPG_SIZE+VENICE_RDATA_SIZE)
#define BASE_ADDR           0x1000
#define FIFO_ADDR           0x3000
#define PRIVATE_FIFO_ADDR   (FIFO_ADDR + VENICE_FIFO_SIZE)
#define REGS_ADDR           (PRIVATE_FIFO_ADDR + VENICE_PRIV_SIZE)

/* configuration limitations */
#define VENICE_FIFO_LENGTH	255	/* #entries in upper fifo */

#define VENICE_FIFO_HIWATER	200	/* hardware checked hiwater */
#define VENICE_FIFO_LOWATER	4	/* hardware checked lowater */
#define VENICE_FIFO_SOFTWATER	140	/* software checked lowater */
#define VENICE_FIFO_SPINCNT	30	/* in terms of usec */

#define VENICE_WID_CNT		40	/* number of display contexts in hw */
#define VENICE_PCX_CNT		16	/* number of pipe contexts in hw */
#define VENICE_PIECELIST_SIZE	14	/* obscured rectangle list size */
#define SPLVENICE		spl5	/* spl to protect against intr */
#define VENICE_MEMALLOC_CHUNKSIZE 25	/* memory allocation chunk size */
#define VENICE_MEMALLOC_HIWATER	40	/* free list high water mark */
#define VENICE_VS2_MAX_HEADS	6	/* heads available on VS2 */

#define VENICE_PIPE0_ADDR	0xb7000000	/* MP address for pipe 0 */


/*
 * Frame buffer configuration info structure declarations...
 *
 * The Venice frame buffer may have different pixel sizes depending upon the
 * number of available frame buffer tiles.  The maximum number of tiles per
 * 8 raster scan lines is 32, and a different number of tiles are available 
 * depending on whether or not we configure for small/medium/large pixels, 
 * and have 1/2/4 RM boards installed in the system.
 *
 * Depending on the format resolution, the actual number of tiles per line
 * may be less than 32.
 *
 * # of RM's	Pixel Size	Tile size	# of Tiles	DG2 max res.
 * **********	**********	*********	**********	************
 *
 * 1 RM Board:	large		8 x  80		 512		 640 x  512
 * 1 RM Board:	medium		8 x  80		1024		1024 x  640
 * 1 RM Board:	small		8 x  80		2048		1280 x 1024
 *
 * 2 RM Boards:	large		8 x 160		 512		1024 x  640
 * 2 RM Boards:	medium		8 x 160		1024		1280 x 1024
 * 2 RM Boards:	small		8 x 160		2048		2048 x 1280
 *
 * 4 RM Boards:	large		8 x 320		 512		1280 x 1024
 * 4 RM Boards:	medium		8 x 320		1024		2048 x 1280
 * 4 RM Boards:	small		8 x 320		2048		2560 x 2048
 */

/* Definitions for venice_vof_file_info_t.flags */

#define VOF_F_BRESENHAM		0x0001		/* special for DG2->VS2 */
#define VOF_F_INTERLACED	0x0002		/* interlaced output */
#define VOF_F_OLD_STEREO	0x0004		/* old-style, stereo format */
#define VOF_F_STEREO		0x0008		/* new-style, stereo format */
#define VOF_F_ZOOMED		0x0010		/* zoomed format */
#define VOF_F_SPLIT_REQ		0x0020		/* split each req into 2 req */
#define VOF_F_DO_GENLOCK	0x0040		/* lock to incoming sync */
#define	VOF_F_TTL_OUTSYNC	0x0080		/* output sync at TTL levels */

/* this structure is copied verbatim out of the vof.u microcode file */

typedef struct {

    /* Size of active area for width and height */
    int vof_width;	/* the size of DG2 memory used */
    int vof_height;	/* must be <= the display surface size */

    int cursor_fudge_x;
    int cursor_fudge_y;

    int flags;		/* see #defines above */

    int xfer_bandwidth;	/* required transfer bandwidth (pixels/sec) */
    int hwalk_length;	/* number of operations available in hblank */
    int vof_framerate;	/* VS2 vof loader will need this too */

    int	monitor_type;	/* Monitor type.  See getmonitor(3G). */
} venice_vof_file_info_t;

typedef struct {

    int active;		/* whether this channel is being used or not */
    int cursor_enable;	/* whether to put a cursor on this channel or not */

    int pan_x;		/* offset between visible output and display surface */
    int pan_y;

    venice_vof_file_info_t vof_file_info;

} venice_vof_head_info_t;

/*
 * Structure returned by ioctl GETBOARDINFO when executed on a VENICE.
 */
struct venice_info {

	/* the device-dependant structure contains two numbers, xpmax
	 * and ypmax. These are the user's desired display surface area
	 * (which may be different from his viewable screen size).
	 */

	struct gfx_info gfx_info; 	/* device independent information */

	/*
	 * VENICE specific information.
	 * Note: If we ever implement a multi-head single-pipe X server/kernel,
	 * some of this must be rearranged.
	 */

	int	ge_rev;			/* GE revision (indicative of count) */
	int	rm_count;		/* number of RM boards in the system */
	int	pixel_depth;		/* small, medium, or large */
	int	tiles_per_line;		/* xpmax / (80 * #RMs) */
	int     baseTileAddr;           /* Starting tile number of frame buffer */
	int     ilOffset;               /* Initial Line Offset (pixel/10 offset within tile) */
	int	pixel_density;		/* 10bitRGBA, 10bitRGB, or 8bitRGB */
	int	hwalk_length;		/* # ops available in hblank */

	/*
	 * For VS2, there are six independent channels; hence the head_info[6].
	 * If not using a VS2, then only entry 0 will be used.
	 */

	int	video_format;		/* identifier for video format */
	int	going_to_vs2;		/* we're feeding a VS2 */

	venice_vof_head_info_t vof_head_info[VENICE_VS2_MAX_HEADS];

};

typedef struct venice_info venice_info_t;

/* 
 * This maps onto pipe of the ge8 board, and is utilized to access the
 * graphics pipe from the host.
 * It must be exactly 8 words long.  It does not matter which word is written
 * to.  The low three bits of the address all map to the same pipe command.
 * The different types are provided only to keep C from mucking with your data
 */
union venice_pipe {
	float f;
	long i;
	unsigned long ul;
	long dummy[2];
};

struct venice_dmavec {
	caddr_t dmav_base;		/* user virtual address of data block */
	int     dmav_len;		/* length of data block in bytes */
};

/* private ioctl numbers */
#define VENICE_BASE			10000
#define VENICE_DMAREAD			(VENICE_BASE + 0)
#define VENICE_DMAWRITE			(VENICE_BASE + 1)
#define VENICE_LOAD_CURSOR		(VENICE_BASE + 2)
#define VENICE_SELECT_CURSOR		(VENICE_BASE + 3)
#define VENICE_SETDISPLAYMODE		(VENICE_BASE + 4)
#define VENICE_START_PICKFEED		(VENICE_BASE + 5)
#define VENICE_FINISH_PICKFEED		(VENICE_BASE + 6)
#define VENICE_LOAD_GAMMA		(VENICE_BASE + 7)
#define VENICE_TEXTUREBIND		(VENICE_BASE + 8)
#define VENICE_DIAG_INTR		(VENICE_BASE + 10)
#define VENICE_DIAG_PHYSMAP		(VENICE_BASE + 11)
#define VENICE_DIAG_INIT		(VENICE_BASE + 12)
#define VENICE_ILBUFFER			(VENICE_BASE + 13)
#define VENICE_SETBOARDINFO		(VENICE_BASE + 14)
#define VENICE_DGSKIP			(VENICE_BASE + 15)
#define VENICE_EMPTYQUERY		(VENICE_BASE + 16)
#define VENICE_VSYNC_SET		(VENICE_BASE + 17) 
#define VENICE_VSYNC_GET		(VENICE_BASE + 18)
#define VENICE_VSYNC_SLEEP		(VENICE_BASE + 19)
#define VENICE_LOAD_IE_CONFIG_LO	(VENICE_BASE + 20)
#define VENICE_LOAD_UNDERLAY		VENICE_LOAD_IE_CONFIG_LO
#define VENICE_SUBTEXLOAD		(VENICE_BASE + 21)
#define VENICE_SET_VIDEO		(VENICE_BASE + 22)
#define VENICE_REINIT_VIDEO		(VENICE_BASE + 23)
#define VENICE_WRITE_DG2_EEPROM		(VENICE_BASE + 24)
#define VENICE_GET_VIDEO		(VENICE_BASE + 25)
#define VENICE_READ_DG2_EEPROM		(VENICE_BASE + 26)

/*
 *   VENICE Ioctl args
 */
struct venice_ilbuffer_args {
   int il; /* are we the il or the gl? */
   int buffer; /* which buffer are we reserving (0 means deallocation ) */
};

struct venice_load_cursor_args {
	int	id;				/* cursor ID */
	int	*glyph;				/* glyph bits (on long boundary) */
	int	size;				/* size of glyph bits in bytes */
};

struct venice_select_cursor_args {
	int	mode;				/* on/off/xhair/glyph */
#define VENICE_CURSOR_OFF	0		/* this turns cursor off */
#define VENICE_CURSOR_XHAIR	1		/* this enables a cursor xhair */
#define VENICE_CURSOR_GLYPH	2		/* this enables a cursor glyph */
	int	valid;				/* parameters valid */
/*
 * for VENICE_CURSOR_GLYPH, valid causes all of the below to be respected.
 * for VENICE_CURSOR_XHAIR, valid causes only color3 to be respected.
 */
	int	id;				/* cursor ID */
	int	hotx;				/* x component of hot spot */
	int	hoty;				/* y component of hot spot */
	int	color1;				/* RGBA value of color 1 */
	int	color2;				/* RGBA value of color 2 */
	int	color3;				/* RGBA value of color 3 */
};

/* defines for cursor mode arg */
#define VENICE_MAX_CURSOR_ID	31		/* we cache 32 64X64 cursors */
#define VENICE_OFF_CURSOR_ID	(VENICE_MAX_CURSOR_ID+1)
#define VENICE_ON_CURSOR_ID	(VENICE_MAX_CURSOR_ID+2)
#define VENICE_XHAIR_CURSOR_ID	(VENICE_MAX_CURSOR_ID+3)

struct venice_setdisplaymode_args {
   	long int wid;				/* window id */
	unsigned long int displaymode;		/* IE DID table value */
};

struct venice_start_pickfeed_args {
        char *buffer;           /* user buffer virtual address */
        long buflen;            /* length of buffer */
};

struct venice_finish_pickfeed_args {
	long wordcount;		/* number of words transferred */
        long overflow;          /* buffer overflow flag */
};

struct venice_dmaread_args {
        char *buffer;           /* user buffer virtual address */
        long buflen;            /* length of buffer */
	long wordcount;		/* number of words transferred */
        long overflow;          /* buffer overflow flag */
	long cpdata[32];	/* buffer for cp cmds/data */
	long cpcmdcnt;		/* how many commands in buffer */
};

struct venice_dmawrite_args {
	struct venice_dmavec *tptr;
	int tlen;
};

struct venice_texturebind_args {
	long	n;
	long	m;
	struct ulamslot *slots;
};

struct venice_diag_intr_args {
	int cmd;			/* what operation to perform */
#define VENICE_DIAG_INTR_CONFIG	0	/* pick up information on how to use */
#define VENICE_DIAG_INTR_CLEAR	1	/* zero any counts specified in mask */
#define VENICE_DIAG_INTR_READ	2	/* read current counts out */
#define VENICE_DIAG_INTR_WAIT	3	/* wait for specified counts to match */
	unsigned long mask;		/* wait mask */
	unsigned long timeout;		/* 100Hz timeout value, 0 is forever */
#define VENICE_NUM_DIAG_CPU_MASKS 8
	unsigned int cpu_masks[VENICE_NUM_DIAG_CPU_MASKS];
	unsigned int vector;
};

#define VENICE_DIAG_INTR_VECTORS_MASK	0x1f

struct venice_diag_physmap_args {
	char *buffer;		/* address of virtual area to translate */
	int buflen;		/* size of virtual area to translate */
	int *table;		/* pointer to a translation table */
	int table_len;		/* number of ints in translation table */
};

struct venice_vsync_sleep_args {
	long	interval;
	long	mod;
};

struct venice_set_video_args {
	int	regaddr;	/* VCBUS address */
	int	mask;
	int	value;
	int	onvlist;
};

/* size of array used for allocations for structures describing textures */
#define NUM_TEXSHADOWS	1

/*
 * State of a slot of graphics texture memory:
 * FREE     - memory does not contain a valid texture map.
 * UNLOADED - memory did contain a valid texture map and will likely 
	      need to again.
 * INACTIVE - memory contains a valid texture map for a valid process that is
 *            not currently running.
 * ACTIVE   - memory contains a valid texture map that may be used at any time.
 * RESERVED- transient state appears during kernel computations.
 */
#define FREE		0
#define UNLOADED	(FREE+1)
#define INACTIVE        (UNLOADED+1)
#define ACTIVE		(INACTIVE+1)
#define RESERVED	(ACTIVE+1)
#define ACCBUFUSE	(RESERVED+1)

/*
 * DG register definitions and formats
 */
#define V_DG_VC_BASE		0x0
#define V_DG_SRAM_BASE		(V_DG_VC_BASE + 0x0)
#define FM_SRAM_SIZE            32768
#define FM_SRAM_DATA_SIZE       32
#define FM_SRAM_ADDR_SIZE       15

#define VDRC_SRAM_SIZE_P0	32768
#define VDRC_SRAM_SIZE		32768*4
#define VDRC_SRAM_DATA_SIZE	8
/*
 * The last 16 bytes of VDRC SRAM contain (offset, length) pairs which are
 * global for all video request records in the SRAM (except for those records
 * which are created to work around the IMP page crossing bug).  There are
 * 8 such pairs so that we can have different line lengths & offsets for each
 * of the 8 (nay, 6) VS2 output formats.
 */
#define VDRC_OFFSET_TABLE_START		(VDRC_SRAM_SIZE - 16)
#define VDRC_OFFSET_TABLE_START_P0	(VDRC_SRAM_SIZE_P0 - 16)

#define DG_EEPROM_BASE_ADDR	(V_DG_VC_BASE + 0x8000)
#define DG_EEPROM_SIZE		8192
#define DG_EEPROM_DATA_SIZE     8 
#define DG_EEPROM_ADDR_SIZE     13
#define DG_CURSOR_SIZE	        512	
#define DG_CURSOR_DATA_SIZE     8 
#define DG_DAC_PRIMRAM_SIZE     1024
#define DG_DAC_ALTRAM_SIZE      256
#define DG_DAC_OVLMEM_SIZE      32
#define DG_RGBDAC_DATA_SIZE     24
#define DG_DAC_DATA_SIZE        8
#define DG_XMAP_DID_SIZE        42      
#define DG_XMAP_DID_DATA_SIZE   19      
#define DG_XMAP_CMAP_SIZE      	32768
#define DG_XMAP_CMAP_ADDR_SIZE  15
#define DG_XMAP_CMAP_DATA_SIZE  30
#define DG_XMAP_DID_BASE	0x0
#define V_DG_XMAP_BASE_ALL	(V_DG_VC_BASE + 0xa000)	  /* 0th xmap base */
#define XMAP_SIZE		0x100
#define V_DG_XMAP_BASE(i)	(V_DG_XMAP_BASE_ALL + XMAP_SIZE * (i)) 
#define ALL			10	/* broadcast address for xmaps */
#define	Q30			0x3B
#define	Q4			0x3C
#define	Q5			0x3D
#define	Q6			0x3E
#define DG_ENC_CLUT_SIZE        3*256
#define DG_ENC_CLUT_DATA_SIZE   8

#define V_DG_CURSOR_BASE	(V_DG_VC_BASE + 0xac00)
#define V_DG_ENCODER_BASE	(V_DG_VC_BASE + 0xae00)
#define V_DG_CLOCK_BASE		(V_DG_VC_BASE + 0xaf00)
#define V_DG_VC_XILINX_BASE	(V_DG_VC_BASE + 0xb000)
#define V_DG_VOF_XILINX_BASE	(V_DG_VC_BASE + 0xb100)
#define V_DG_VIF_XILINX_BASE	(V_DG_VC_BASE + 0xb200)
#define V_DG_VDRC_XILINX_BASE	(V_DG_VC_BASE + 0xb300)
#define V_DG_VFC_XILINX_BASE	(V_DG_VC_BASE + 0xb400)
#define V_DG_RGBDAC_BASE	(V_DG_VC_BASE + 0xb800)
#define V_DG_ALPHADAC_BASE	(V_DG_VC_BASE + 0xbb00)

#define V_DG_INITREG		V_DG_VC_XILINX_BASE + 0x00e0
#define   V_DG_XMAP_RESET	  0x10000000
#define   V_DG_VWALK_ENABLE	  0x20000000
#define   V_DG_HWALK_ENABLE	  0x40000000

#define V_DG_XMAP_ALL		(V_DG_XMAP_BASE(10))      /* broadcast */
#define V_DG_XMAP_DID_BASE(i)	(V_DG_XMAP_BASE(i) + 0x00)
#define V_DG_XMAP_DID(i, DID)	(V_DG_XMAP_DID_BASE(i)+(DID))
#define V_DG_XMAP_ODID_BASE(i)	(V_DG_XMAP_BASE(i) + 0x20)
#define V_DG_XMAP_ODID(i, ODID)	(V_DG_XMAP_ODID_BASE(i)+(ODID))
#define	V_DG_XMAP_PDID(i)	(V_DG_XMAP_BASE(i) + 0x28)
#define	V_DG_XMAP_UDID(i)	(V_DG_XMAP_BASE(i) + 0x29)
#define	V_DG_XMAP_DIDOFF(x)	((x & 0x7fff) << 4)
#define	V_DG_XMAP_DIDLEN(x)	(x & 0xf)
#define	V_DG_XMAP_CMAPR(x)	((x & 0x3ff) << 20)
#define	V_DG_XMAP_CMAPG(x)	((x & 0x3ff) << 10)
#define	V_DG_XMAP_CMAPB(x)	(x & 0x3ff)
#define	V_DG_XMAP_BDID(i)	(V_DG_XMAP_BASE(i) + 0x30) /* rev ID number*/
#define	V_DG_XMAP_SGMUX(i)	(V_DG_XMAP_BASE(i) + 0x31) /* sig mux addr*/
#define	V_DG_XMAP_ISG(i)	(V_DG_XMAP_BASE(i) + 0x32) /* input sig addr*/
#define	V_DG_XMAP_CHIPID(i)	(V_DG_XMAP_BASE(i) + 0x33) /* chip ID	*/
#define	V_DG_XMAP_SGEN(i)	(V_DG_XMAP_BASE(i) + 0x34) /* signature enable*/
#define	V_DG_XMAP_810(i)	(V_DG_XMAP_BASE(i) + 0x35) /* 8 or 10 bits */
#define	V_DG_XMAP_ALPHA(i)	(V_DG_XMAP_BASE(i) + 0x36) /* send alpha */
#define	V_DG_XMAP_DFALPHA(i)	(V_DG_XMAP_BASE(i) + 0x37) /* default alpha */
#define	V_DG_XMAP_CMAPADDR(i)	(V_DG_XMAP_BASE(i) + 0x38) /* colormap addres */
#define	V_DG_XMAP_LSB0(i)	(V_DG_XMAP_BASE(i) + 0x39) /* zero lsbs	*/
#define	V_DG_XMAP_CMAPDATA(i)	(V_DG_XMAP_BASE(i) + 0x3A) /* colormap data */
#define	V_DG_XMAP_Q30(i)	(V_DG_XMAP_BASE(i) + 0x3B) /* bit 3:0, Q mux */
#define	V_DG_XMAP_Q4(i)		(V_DG_XMAP_BASE(i) + 0x3C) /* bit 4 of Q mux */
#define	V_DG_XMAP_Q5(i)		(V_DG_XMAP_BASE(i) + 0x3D) /* bit 5 of Q mux */
#define	V_DG_XMAP_Q6(i)		(V_DG_XMAP_BASE(i) + 0x3E) /* bit 6 of Q mux */
#define V_DG_ENC_CLUT_ADDR	(V_DG_ENCODER_BASE + 0x0)
#define V_DG_ENC_CLUT_DATA	(V_DG_ENCODER_BASE + 0x1)
#define V_DG_ENC_CNTRL_ADDR	(V_DG_ENCODER_BASE + 0x2)
#define V_DG_ENC_CNTRL_DATA	(V_DG_ENCODER_BASE + 0x3)

/* selectable outputs to A/D path (selected by V_DG_CAL_SEL) */
#define RED_CHANNEL_OUT         0x0
#define GREEN_CHANNEL_OUT       0x1
#define BLUE_CHANNEL_OUT        0x2
#define ALPHA_CHANNEL_OUT       0x3
#define SYNC_CHANNEL_OUT        0x4
#define ENCODER_LUM_OUT         0x5
#define ENCODER_COMP_OUT        0x6
#define ONBOARD_REF_VOLT        0x7


/* register assignments across xmaps */
#define V_DG_CAL_SEL            V_DG_XMAP_Q30(0)    /* (W 3) */
#define V_DG_BLUE_DPOT_SEL      V_DG_XMAP_Q4(0)     /* (R/W 1) */
#define V_DG_GREEN_DPOT_SEL     V_DG_XMAP_Q5(0)     /* (R/W 1) */
#define V_DG_RED_DPOT_SEL       V_DG_XMAP_Q6(0)     /* (R/W 1) */

#define V_DG_XILINX_RDBCK_TRIG  V_DG_XMAP_Q30(1)	/* R/W 4 */ 
#define V_DG_VFC_RDBCK_TRIG	V_DG_XMAP_Q4(1)		/* (R/W 1) */
#define V_DG_SYNC_DPOT_SEL      V_DG_XMAP_Q5(1)  	/* (R/W 1) */
#define V_DG_ALPHA_DPOT_SEL     V_DG_XMAP_Q6(1)		/* (R/W 1) */

/* dignam's hack to yank the sync level into genlock range */
#define	V_DG_GENLOCK_LEVEL	V_DG_XMAP_Q30(2)	/* (W 2) */
#define V_DG_VFC_PROG_L     	V_DG_XMAP_Q4(2)		/* (R/W 1) */
#define V_DG_VOF_PROG_L      	V_DG_XMAP_Q5(2) 	/* (R/W 1) */
#define V_DG_VIF_PROG_L      	V_DG_XMAP_Q6(2) 	/* (R/W 1) */

#define V_DG_ENCODER_CNTRL      V_DG_XMAP_Q30(3)	/* (R/W 3) */
/*
 * Note: V_DG_ENCODER_RESET_L is only present on the P2 spin of the DG2.
 *       On the P1 board it is bit 1 of the V_DG_ENCODER_CNTRL register.
 */
#define V_DG_ENCODER_RESET_L    V_DG_XMAP_Q30(3)	/* (R/W 3) */
#define V_DG_VDRC_RDBCK_TRIG	V_DG_XMAP_Q4(3) 	/* (R/W 1) */
#define V_DG_VDRC_PROG_L     	V_DG_XMAP_Q5(3)		/* (R/W 1) */
#define V_DG_DI_CNTRL_PROG_L  	V_DG_XMAP_Q6(3)		/* (R/W 1) */

#define V_DG_SER_BUS		V_DG_XMAP_Q30(4)	/* R/W 4 */ 

#define V_DG_CLK_RESET_L	V_DG_XMAP_Q4(4)		/* (R/W 1) */
#define V_DG_VDRC_RESET_L	V_DG_XMAP_Q5(4)		/* (R/W 1) */
#define V_DG_VFC_RESET_L	V_DG_XMAP_Q6(4)	 	/* (R/W 1) */

#define V_DG_DVI	  	V_DG_XMAP_Q30(5)	/* (R/W 2) */
#define V_DG_SWAP_BUFFER  	V_DG_XMAP_Q4(5)		/* (R/W 1) */
#define V_DG_H_DIV_EN 	  	V_DG_XMAP_Q5(5)	    	/* (R/W 1) */
#define V_DG_DPOT_UD            V_DG_XMAP_Q6(5)		/* (R/W 1) */

#define V_DG_KEY_SEL	  	V_DG_XMAP_Q30(6)	/* (R/W 4) */

#define V_DG_SERIAL_CLK_EN_L   	V_DG_XMAP_Q4(6)		/* (R/W 1) */

#define V_DG_ADC_SEL1	  	V_DG_XMAP_Q5(6)		/* (R/W 1) */
#define V_DG_AGC_OFF	  	V_DG_XMAP_Q6(6) 	/* (R/W 1) */
#define V_DG_ENC_ADCCLK_SEL	V_DG_XMAP_Q6(9)		/* (R/W 1) */

#define V_DG_GENLOCK	 	V_DG_XMAP_Q4(7)		/* (R/W 1) */
#define V_DG_ADC_DISABLE 	V_DG_XMAP_Q5(7)		/* (R/W 1) */
#define V_DG_TTL_SYNC_SEL	V_DG_XMAP_Q6(7)		/* (R/W 1) */

#define V_DG_STEREO_L	 	V_DG_XMAP_Q6(8)		/* (R/W 1) */
#define V_DG_DI_CNTL_RESET_L 	V_DG_XMAP_Q5(8)		/* (R/W 1) */
#define V_DG_SWAP_READY		V_DG_XMAP_Q4(8)		/* (R/W 1) */


#define V_DG_PROG_CLK           (V_DG_XMAP_BASE(0) + 0xf00)  	/* (W 5) */
/*
 * To probe for a P0 board, first look for a P1 board.  If V_DG_BOARD_ID
 * does not return DG_P2_BOARD_PRESENT, assume a P1/P0 board is present.
 */
#define V_DG_BOARD_ID_P0        (V_DG_XMAP_BASE(0) + 0x30) 	/* (R 4) */
#define V_DG_BOARD_ID           (V_DG_XMAP_BASE(8) + 0x30) 	/* (R 4) */
#define DG_P2_BOARD_PRESENT	0x1

#define V_DG_RM_PRESENT_L       (V_DG_XMAP_BASE(1) + 0x30)	/* (R 4) */
#define V_DG_RMT_PRESENT_L      (V_DG_XMAP_BASE(2) + 0x30)	/* (R 4) */
#define V_DG_XILINX_RDBCK_1  	(V_DG_XMAP_BASE(3) + 0x30)	/* (R 4) */
#define V_DG_XILINX_RDBCK_2  	(V_DG_XMAP_BASE(4) + 0x30)	/* (R 4) */
#define V_DG_VIDEO_REG0		(V_DG_XMAP_BASE(5) + 0x30)	/* (R 4) */
#define V_DG_XILINX_DONE_PROG	(V_DG_XMAP_BASE(6) + 0x30)	/* (R 4) */
#define V_DG_MONITOR_TYPE	(V_DG_XMAP_BASE(7) + 0x30)    	/* (R 4) */

/* XILINX resident  */

/*
 * VC Bus Controller register definitions
 */

#define V_DG_INTERRUPT_MASK	(V_DG_VC_XILINX_BASE + 0x00)    /* (W 3) */
#define   V_DG_BLANKSTART_INT_EN	0x20000000
#define V_DG_FIFO_FULL 		(V_DG_VC_XILINX_BASE + 0x20)    /* (W 1) */
#define V_DG_HLIST_LENGTH_LOW 	(V_DG_VC_XILINX_BASE + 0x40)    /* (W 4) */
#define V_DG_HLIST_LENGTH_HIGH 	(V_DG_VC_XILINX_BASE + 0x60)    /* (W 4) */
#define V_DG_HLIST 		(V_DG_VC_XILINX_BASE + 0x80) /* (R/W 32) */
#define V_DG_VLIST		(V_DG_VC_XILINX_BASE + 0xa0) /* (R/W 32) */
#define V_DG_SERIAL_CLOCK	(V_DG_VC_XILINX_BASE + 0xc0) /* (R/W 32) */

/*
 * VDRC register definitions
 */

/*
 * DG_VDRC_MEM_ACCESS, when set, will reset the address counter that points
 * to the VDRC SRAM that contains the IMP addresses.  Subsequent writes to
 * the V_DG_VDRC_DATA register will cause a write to the SRAM and will 
 * automatically increment the address counter.  This bit should be cleared
 * after the memory is loaded, so that writes to the V_DG_IE_WRITE 
 * register (which is used to load IMP registers) will succeed.
 */
#define V_DG_VDRC_CNTRL 	(V_DG_VDRC_XILINX_BASE + 0x0)	/* (W 2) */
#define DG_VDRC_MEM_ACCESS	0x1			/* set cntrl bit 0 */
#define DG_VDRC_DID_NEXTBYTE_EN	0x2			/* set cntrl bit 1 */

#define V_DG_VDRC_MODE	 	(V_DG_VDRC_XILINX_BASE + 0x1)	/* (W 5) */
#define DG_VDRC_PIXSIZE_MASK	0x3			/* bits <1..0> */
#define DG_VDRC_PIXSIZE_26	0x0			/* RGB 8 */
#define DG_VDRC_PIXSIZE_32	0x1			/* RGB 10 */
#define DG_VDRC_PIXSIZE_34	0x2			/* RGBA 8 (can't use) */
#define DG_VDRC_PIXSIZE_42	0x3			/* RGBA 10 */
#define DG_VDRC_NUM_RM_MASK	0xc			/* bits <3..2> */
#define DG_VDRC_ONE_RM		0x0
#define DG_VDRC_TWO_RMS		0x1
#define DG_VDRC_FOUR_RMS	0x2
#define DG_VDRC_VS2_MODE_MASK	0x10
/*
 * Writes to V_DG_VDRC_DATA go to VDRC_SRAM, if bit 0 of the control 
 * register is set.
 */
#define V_DG_VDRC_DATA	 	(V_DG_VDRC_XILINX_BASE + 0x2)	/* (W 8) */
/*
 * Writes to V_DG_VDRC_IE_WRITE will go to the first byte or the second 
 * byte of a 16 bit latch.  When the 'next' byte is written, the data is 
 * flushed and written into the IMP register location using the address & data 
 * encoded into the two bytes.
 *
 * The following programming sequence should be followed:
 *
 * Set:
 * bit 0 of V_DG_VDRC_CNTRL = 0
 * bit 1 of V_DG_VDRC_CNTRL = 0
 *
 * write first byte to V_DG_VDRC_IE_WRITE
 *
 * Set:
 * bit 0 of V_DG_VDRC_CNTRL = 0
 * bit 1 of V_DG_VDRC_CNTRL = 1
 *
 * write next byte to V_DG_VDRC_IE_WRITE
 *
 * The data format of these two bytes is as follows:
 * 
 *    7      6      5      4      3      2      1      0
 * ========================================================
 * |  D0  |  A5  |  A4  |  A3  |  A2  |  A1  |  A0  |  1  |  "first"
 * ========================================================
 * |  X   |  X   |  X   |  D5  |  D4  |  D3  |  D2  |  D1 |  "next"
 * ========================================================
 */
#define V_DG_VDRC_IE_WRITE	(V_DG_VDRC_XILINX_BASE + 0x3)  /* (W 8) */

/*
 * Symbolic constants for IMP register bit fields.
 */
#define V_DG_IE_DID(DID)	   (DID)
#define V_DG_IE_DID_BASE	   (0x0)
#define   V_DG_IE_DID_CIMODE	   0x01	/* DID bit for CI selection */
#define   V_DG_IE_DID_GAMMA	   0x02	/* DID bit for gamma selection */
#define   V_DG_IE_DID_DITHER	   0x04	/* DID bit for dither enable */
#define   V_DG_IE_DID_BUFFER	   0x08	/* DID bit for buffer select */
#define   V_DG_IE_DID_STEREO	   0x10	/* DID bit for stereo buffer select */
#define   V_DG_IE_DID_8BIT	   0x20	/* DID bit for VGX style 8bit out */
#define   V_DG_IE_DID_GANG	   0x800/* DID bit for gang swapuffer */
#define V_DG_IE_ODID_BASE	   (0x20)
#define V_DG_IE_ODID(ODID)	   (V_DG_IE_ODID_BASE+(ODID))
#define   V_DG_IE_ODID_BUFFER	   0x01	/* ODID bit for buffer select */
#define   V_DG_IE_ODID_OPAQUE	   0x02	/* ODID bit for being opaque */
#define   V_DG_IE_ODID_ENABLE	   0x04	/* ODID bit for enable */
#define V_DG_IE_CONFIG_LO	   (0x28)
#define   V_DG_IE_CONFIG_ALPHA	   0x01	/* When set, send alpha */
#define   V_DG_IE_CONFIG_GSTEREO   0x02	/* global stereo mode */
#define   V_DG_IE_CONFIG_SEND8	   0x04	/* When set, send 8 bits;else 10 bits */
#define   V_DG_IE_CONFIG_GUNDERLAY 0x08	/* When set, global underlay enabled */
#define   V_DG_IE_CONFIG_LO_ALL	   0x0f	/* Allows access to all configlo bits */
#define V_DG_IE_CONFIG_HI	   (0x29)
#define   V_DG_IE_CONFIG_SMALLPIX  0x00	/* large pixel */
#define   V_DG_IE_CONFIG_MEDPIX	   0x01	/* jumbo pixel */
#define   V_DG_IE_CONFIG_LARGEPIX  0x02	/* colossal pixel */

#define	V_DG_SMALL_PIXEL_SIZE	80
#define	V_DG_MEDIUM_PIXEL_SIZE	160
#define	V_DG_LARGE_PIXEL_SIZE	320

#define	PIX26_RGB8	V_DG_IE_CONFIG_SEND8
#define	PIX32_RGB10	0
#define	PIX42_RGBA10	V_DG_IE_CONFIG_ALPHA

/*
 * VFC register definitions (bogus)
 */

#define V_DG_VIDEO_XADD_LOW 	(V_DG_VFC_XILINX_BASE + 0x00)	/* (W 8) */
#define V_DG_VIDEO_XADD_HIGH 	(V_DG_VFC_XILINX_BASE + 0x01)   /* (W 8) */
#define V_DG_VIDEO_XADD_SUB10	(V_DG_VFC_XILINX_BASE + 0x02)   /* (W 3) */
#define V_DG_VIDEO_YADD_LOW 	(V_DG_VFC_XILINX_BASE + 0x03)   /* (W 8) */
#define V_DG_VIDEO_YADD_HIGH 	(V_DG_VFC_XILINX_BASE + 0x04)   /* (W 8) */
#define V_DG_VIDEO_CONTROL	(V_DG_VFC_XILINX_BASE + 0x06)	/* (W 8) */


/*	The following 2 must be checked for accuracy.
*/
#define V_DG_VIDEO_YADD_SUB	(V_DG_VFC_XILINX_BASE + 0x05)	/* (W 8 */
#define V_DG_VFC_VIDEO	 	(V_DG_VFC_XILINX_BASE + 0x00)   /* (W 8) */

#define V_DG_GRAB_DATA_REG 	(V_DG_VFC_XILINX_BASE + 0x00)   /* (R 8) */


/*	VFC Register Definition 
 *
 */

/*	VIDEO_CONTROL	*/
#define	V_DG_VIDEO_ENABLE_KEY		0x00 ;	/* Enable Keyeing */
#define V_DG_VIDEO_FORCE_KEY		0x02 ;	/* Force Key	*/
#define V_DG_VIDEO_DECIMATION		0x04 ;	/* Turn Decimation on */
#define	V_DG_VIDEO_NTSC_PAL_MOD		0x08 ;	/* NTSC  = 1, PAL = 0	*/

/*
 * VOF register definitions 
 *
 * The first three VOF registers are written indirectly through the edge 
 * memory address and edge memory data registers.  The address definitions 
 * provided for these three are to be written into the edge memory address
 * register; data values created by oring together the bit definitions 
 * defined below are to be written into the edge memory data register.
 */


/*
 * Address definitions for the directly addressed VOF registers follow.
 */

/*
 * Definitions for V_DG_VOF_CNTRL (start/stop video output formatter)
 */

#define V_DG_VOF_CNTRL		(V_DG_VOF_XILINX_BASE + 0x0)   /* (W 1) */

#define DG_VOF_RUN		0x0	
#define DG_VOF_STOP		0x1	/* assert while loading vof's */
/*
 * DG_VOF_XMAP_MUX_TOGGLE provides 5 pixel positioning of output video by 
 * inverting the xmap mux control signal (data coming out of the xmaps is
 * 10 pixels wide, and multiplexed to get 5 pixels out).
 */
#define DG_VOF_XMAP_MUX_TOGGLE	0x2
#define DG_VOF_BLANK_SCREEN	0x4   /* When set, disables DG2 video out */
#define DG_VOF_DVI_BLANK_L	0x8   /* When low, disables digital video out */
#define DG_VOF_SYNC_MASK	0x30  /* bits <5..4> define sync type */

#define DG_VOF_HDTV_TRI_LEVEL_SYNC_MASK	0x00
#define DG_VOF_4V_SYNC_MASK		0x10
#define DG_VOF_2V_SYNC_MASK		0x20
#define DG_VOF_COMPOSITE_SYNC_MASK	0x30

/*
 * The following bit masks affect bit <6> of the control register, which
 * is used to specify which edge memory address register latch will be 
 * written to when a write cycle to V_DG_VERT_SEQ_COUNTER is encountered.
 *
 * DG_EDGE_LOW_EDGEMEM_ADDR	V_DG_VOF_CNTRL<7..6> = 00 low   8 bits of addr
 * DG_EDGE_HIGH_EDGEMEM_ADDR	V_DG_VOF_CNTRL<7..6> = 01 upper 5 bits of addr
 */

#define DG_EDGE_MEMCTRL_MASK		0x40
#define DG_EDGE_LOW_EDGEMEM_ADDR	0x00
#define DG_EDGE_HIGH_EDGEMEM_ADDR	0x40
#define DG_DSLUT_SELECT                 0x80

/*
 * V_DG_VERT_SEQ_COUNTER is used only when loading VOF's, although it is
 * dual purpose:
 *
 * In VOF loading mode, it serves as the address register for the line type
 * lut memory (which must be used to indirectly address the line length 
 * memory) and the line duration memory.
 *
 * In VOF operational mode, it becomes the vertical sequence counter, and
 * is counted up & reset automatically by the VOF state machine controller.
 */

#define V_DG_VERT_SEQ_COUNTER 	(V_DG_VOF_XILINX_BASE + 0x1)   /* (W 5) */
#define V_DG_LINE_TYPE_LUT  	(V_DG_VOF_XILINX_BASE + 0x2)   /* (W 5) */

/*
 * Each line type definition can have a different 9 bit length, although in
 * practice they will all have the same length.  The length counts are
 * calculated in terms of (pixel counts)/5.  The value that should be
 * written is actually (line length - 2).
 */

#define V_DG_LINE_LENGTH_LOW  	(V_DG_VOF_XILINX_BASE + 0x3)   /* (W 8) */
#define V_DG_LINE_LENGTH_HIGH 	(V_DG_VOF_XILINX_BASE + 0x4)   /* (W 1) */

/*
 * V_DG_LINE_DURATION stores the duration (number of lines) for each 
 * vertical state in the sequence (256 max).  The same line type (e.g. 
 * active video) might need to repeat, so we would repeat the line's 
 * definition in the line type memory for however many more states are 
 * required (up to 256 for each vertical state).
 */

#define V_DG_LINE_DURATION	(V_DG_VOF_XILINX_BASE + 0x5)	/* (W 8) */
#define V_DG_EDGE_MEMORY_ADDR	(V_DG_VOF_XILINX_BASE + 0x6)	/* (W 7) */
#define V_DG_EDGE_MEMORY_DATA	(V_DG_VOF_XILINX_BASE + 0x7)	/* (W 1) */

/*
 * VOF edge memory addresses for video signal edges
 *
 * bits <6..3> denote signal (e.g. csync, cblank); bits <2..0> denote edge
 */

#define DG_VOF_CSYNC_0		0x0
#define DG_VOF_CSYNC_1		0x1
#define DG_VOF_CSYNC_2		0x2
#define DG_VOF_CSYNC_3		0x3
#define DG_VOF_CSYNC_4		0x4
#define DG_VOF_CSYNC_5		0x5
#define DG_VOF_CSYNC_6		0x6

#define DG_VOF_CBLANK_0		0x8
#define DG_VOF_CBLANK_1		0x9
#define DG_VOF_CBLANK_2		0xa

/*
 * The stereo signal will probably be generated by a vlist programmable 
 * register, so we'll probably never use the DG_VOF_STEREO signal.
 */

#define DG_VOF_STEREO		0x10

#define DG_VOF_HCURSOR_0	0x18
#define DG_VOF_HCURSOR_1	0x19
#define DG_VOF_HCURSOR_2	0x1a
#define DG_VOF_HCURSOR_3	0x1b

#define DG_VOF_VCURSOR_0	0x20
#define DG_VOF_VCURSOR_1	0x21

#define DG_VOF_VID_ADDR_REQ_0	0x28
#define DG_VOF_VID_ADDR_REQ_1	0x29

#define DG_VOF_ACTIVE_VIDEO_0	0x30
#define DG_VOF_ACTIVE_VIDEO_1	0x31

#define DG_VOF_VWALK_0		0x38
#define DG_VOF_VWALK_1		0x39

#define DG_VOF_HWALK_0		0x40
#define DG_VOF_HWALK_1		0x41

/*
 * VIF register definitions
 */

#define V_DG_VIF_SYNC_WIDTH 	(V_DG_VIF_XILINX_BASE + 0x0)	/* (W 8) */
#define V_DG_VIF_HPHASE_LOOPGAIN (V_DG_VIF_XILINX_BASE + 0x1)	/* (W 8) */
#define V_DG_VIF_MCOUNTER 	(V_DG_VIF_XILINX_BASE + 0x2)	/* (W 8) */
#define V_DG_VIF_HORZ_LENGTH 	(V_DG_VIF_XILINX_BASE + 0x3)	/* (W 8) */
#define V_DG_VIF_GENSRC_CNTRL 	(V_DG_VIF_XILINX_BASE + 0x4)	/* (W 8) */
/*
 * Bit definitions for V_DG_VIF_GENSRC_CNTRL
 */
#define DG2_SYNC_CLAMP_DISABLE		0x1
#define DG2_VIDEO_LEVEL_SYNC_SOURCE	0x40
#define DG2_NON_INTERLACE_SYNC_SOURCE	0x80

#define V_DG_VIF_BPORCH_CLAMP 	(V_DG_VIF_XILINX_BASE + 0x5)	/* (W 8) */
#define V_DG_VIF_FINE_PHASE_ADJ	(V_DG_VIF_XILINX_BASE + 0x6)	/* (W 8) */
#define V_DG_VIF_MAGIC_HIGH	(V_DG_VIF_XILINX_BASE + 0x7)	/* (W 8) */


/* cursor chip r/w regs */

#define V_DG_CURSOR_ADDRPTR_LO  (V_DG_CURSOR_BASE + 0x00)	/* (R/W 8) */
#define V_DG_CURSOR_ADDRPTR_HI  (V_DG_CURSOR_BASE + 0x01)	/* (R/W 8) */
#define V_DG_CURSOR_RAMDATA     (V_DG_CURSOR_BASE + 0x02)       /* (R/W 8) */
#define V_DG_CURSOR_REGDATA     (V_DG_CURSOR_BASE + 0x03)       /* (R/W 8) */
#define   V_DG_CURSOR_CMD        0x00				/* (R/W 8) */
#define   V_DG_CURSOR_CMD_OFF	 0x08
#define   V_DG_CURSOR_CMD_ON	 0x48
#define   V_DG_CURSOR_CMD_XHAIR	 0x28
#define   V_DG_CURSOR_XLOW        0x1    /* (R/W 8) */
#define   V_DG_CURSOR_XHIGH       0x2    /* (R/W 4) */
#define   V_DG_CURSOR_YLOW        0x3    /* (R/W 8) */
#define   V_DG_CURSOR_YHIGH       0x4    /* (R/W 4) */
#define   V_DG_WINDOW_XLOW        0x5    /* (R/W ) */
#define   V_DG_WINDOW_XHIGH       0x6    /* (R/W ) */
#define   V_DG_WINDOW_YLOW        0x7    /* (R/W ) */
#define   V_DG_WINDOW_YHIGH       0x8    /* (R/W ) */
#define   V_DG_WINDOW_WIDLOW      0x9    /* (R/W ) */
#define   V_DG_WINDOW_WIDHIGH     0xa    /* (R/W ) */
#define   V_DG_WINDOW_HEIGHTLOW   0xb    /* (R/W ) */
#define   V_DG_WINDOW_HEIGHTHIGH  0xc    /* (R/W ) */
/* DAC r/w regs (x4 DAC chips), DAC_0,1,2,3 = red,green,blue,alpha */
#define V_DG_RGBDAC_ADDRPTR_LO 	(V_DG_RGBDAC_BASE + 0x0)
#define V_DG_RGBDAC_ADDRPTR_HI 	(V_DG_RGBDAC_BASE + 0x1)
#define V_DG_RGBDAC_REGDATA     (V_DG_RGBDAC_BASE + 0x2) 
#define V_DG_RGBDAC_ALTDATA     (V_DG_RGBDAC_BASE + 0x2)
#define V_DG_RGBDAC_OVLDATA     (V_DG_RGBDAC_BASE + 0x2)
#define V_DG_RGBDAC_PRIMDATA    (V_DG_RGBDAC_BASE + 0x3)
#define V_DG_RGBDAC_ID	        (V_DG_RGBDAC_BASE + 0x0200)
#define V_DG_RGBDAC_CMD_0		0x0201
#define V_DG_RGBDAC_CMD_1	 	0x0202
#define V_DG_RGBDAC_CMD_2		0x0203
#define V_DG_RGBDAC_PIX_READ_MASK_LO    0x0204
#define V_DG_RGBDAC_PIX_READ_MASK_HI    0x0205
#define V_DG_RGBDAC_PIX_BLINK_MASK_LO   0x0206
#define V_DG_RGBDAC_PIX_BLINK_MASK_HI   0x0207
#define V_DG_RGBDAC_OVLAY_READ_MASK     0x0208
#define V_DG_RGBDAC_OVLAY_BLINK_MASK    0x0209
#define V_DG_RGBDAC_TESTREG        	0x020c

#define V_DG_ALPHADAC_ADDRPTR_LO  (V_DG_ALPHADAC_BASE + 0x0)
#define V_DG_ALPHADAC_ADDRPTR_HI  (V_DG_ALPHADAC_BASE + 0x1)
#define V_DG_ALPHADAC_REGDATA     (V_DG_ALPHADAC_BASE + 0x2) 
#define V_DG_ALPHADAC_ALTDATA     (V_DG_ALPHADAC_BASE + 0x2)
#define V_DG_ALPHADAC_OVLDATA     (V_DG_ALPHADAC_BASE + 0x2)
#define V_DG_ALPHADAC_PRIMDATA    (V_DG_ALPHADAC_BASE + 0x3)
#define V_DG_ALPHADAC_ID	       	  0x0200
#define V_DG_ALPHADAC_CMD_0		  0x0201
#define V_DG_ALPHADAC_CMD_1	 	  0x0202
#define V_DG_ALPHADAC_CMD_2		  0x0203
#define V_DG_ALPHADAC_PIX_READ_MASK_LO    0x0204
#define V_DG_ALPHADAC_PIX_READ_MASK_HI    0x0205
#define V_DG_ALPHADAC_PIX_BLINK_MASK_LO   0x0206
#define V_DG_ALPHADAC_PIX_BLINK_MASK_HI   0x0207
#define V_DG_ALPHADAC_OVLAY_READ_MASK     0x0208
#define V_DG_ALPHADAC_OVLAY_BLINK_MASK    0x0209
#define V_DG_ALPHADAC_TESTREG        	  0x020c

#define TESTREG_ENABLE		0x1	/* mask */
/*
 * Macros to access fields within V_DG registers
 */
#define V_DG_SER_CLK(x)         (x & 0x1)
#define V_DG_SER_DATA(x)        (x & 0x2)

/*
 * Bit masks for V_DG_SER_BUS
 */
#define V_DG_SER_DATA_MASK	0x8

/*
 * Bit masks for V_DG_XILINX_RDBCK_1
 */
#define V_DG_DI_CTRL_RDBK_MASK	0x1
#define V_DG_VIF_RDBK_MASK	0x2
#define V_DG_VOF_RDBK_MASK	0x4
#define V_DG_VFC_RDBK_MASK	0x8

/*
 * Bit masks for V_DG_XILINX_RDBCK_2
NOTE: vcbus_ctrl is wrong.
 */
#define V_DG_VCBUS_CTRL_DONE_MASK	0x1
#define V_DG_VDRC_RDBK_MASK		0x2
#define V_DG_VCBUS_CTRL_RDBK_MASK	0x4
#define V_DG_VDRC_CTRL_DONE_MASK	0x8

/*
 * Bit masks for V_DG_XILINX_DONE_PROG
 */
#define V_DG_VFC_CTRL_DONE_MASK	0x1
#define V_DG_VIF_CTRL_DONE_MASK	0x2
#define V_DG_VOF_CTRL_DONE_MASK	0x4
#define V_DG_DI_CTRL_DONE_MASK	0x8


/*
 * Bit masks for V_DG_XILINX_READBACK_TRIG
 */
#define V_DG_DI_CTRL_RDBK_TRIG_MASK	0x1
#define V_DG_VCBUS_CTRL_RDBK_TRIG_MASK	0x2
#define V_DG_VOF_RDBK_TRIG_MASK		0x4
#define V_DG_VIF_RDBK_TRIG_MASK		0x8

/*
 * Xilinx enumerated types for use with xilinx load & readback routines.
 */
#define DG2_VOF_XILINX		0
#define DG2_VIF_XILINX		1
#define DG2_DI_CNTRL_XILINX	2
#define DG2_VCBUS_CNTRL_XILINX	3
#define DG2_VDRC_XILINX		4
#define DG2_VFC_XILINX		5
#define DG2_DUMMY3000_XILINX	6
#define DG2_DI_CNTRL_TESTCONFIG_XILINX	7
#define DG2_VDRC_XILINX_P0	8


/*
 * DG SRAM allocation (staging areas)
 */

/* memory area reserved for hlist and vlist fifos */
#define STAGE_MEM_SIZE			32768
#define STAGE_HLIST_START		V_DG_SRAM_BASE
#define STAGE_HLIST_SIZE		256
#define STAGE_VLIST_PERM_ENTRY		(STAGE_HLIST_START+STAGE_HLIST_SIZE)
#define STAGE_VLIST_START		(STAGE_VLIST_PERM_ENTRY+1)
#define STAGE_VLIST_SIZE		255
#define STAGE_LISTS_SIZE		(STAGE_VLIST_SIZE+STAGE_HLIST_SIZE+1)
#define STAGE_ROUTINES_SIZE		(STAGE_MEM_SIZE-STAGE_LISTS_SIZE)
#define STAGE_ROUTINES_START		(STAGE_VLIST_START+STAGE_VLIST_SIZE)

/* routines */
#define STAGE_DIAGS			STAGE_ROUTINES_START
#define STAGE_DIAGS_SIZE		16		/* size of sequence */
#define STAGE_XMAP_RESET		(STAGE_DIAGS+STAGE_DIAGS_SIZE)
#define STAGE_XMAP_RESET_SIZE		4		/* size of sequence */

#define STAGE_PERM_STUB			(STAGE_XMAP_RESET+STAGE_XMAP_RESET_SIZE)
#define STAGE_PERM_STUB_SIZE		1

/* sequence of VCBUS operations to turn cursor off */
#define STAGE_CURSOR_OFF		(STAGE_PERM_STUB+STAGE_PERM_STUB_SIZE)
#define STAGE_CURSOR_OFF_SIZE		5		/* size of sequence */

/* sequence of VCBUS operations to turn cursor on (glyph mode) */
#define STAGE_CURSOR_ON			(STAGE_CURSOR_OFF+STAGE_CURSOR_OFF_SIZE)
#define STAGE_CURSOR_ON_SIZE		5		/* size of sequence */

/* sequence of VCBUS operations to turn cursor on (xhair mode) */
#define STAGE_CURSOR_XHAIR		(STAGE_CURSOR_ON+STAGE_CURSOR_ON_SIZE)
#define STAGE_CURSOR_XHAIR_SIZE		5		/* size of sequence */

#define STAGE_CURSOR_POS		(STAGE_CURSOR_XHAIR+STAGE_CURSOR_XHAIR_SIZE)
#define STAGE_CURSOR_POS_SIZE		8		/* size of sequence */
#define STAGE_CURSOR_POS_PATCH_LOC	3		/* offset to patch */
#define STAGE_CURSOR_POS_PATCH		(STAGE_CURSOR_POS+STAGE_CURSOR_POS_PATCH_LOC)

/* sequence of VCBUS operations to set cursor colors (and prepare for glyph) */
#define STAGE_CURSOR_COLOR		(STAGE_CURSOR_POS+STAGE_CURSOR_POS_SIZE)
#define STAGE_CURSOR_COLOR_SIZE		18
#define STAGE_CURSOR_COLOR_PATCH_RGB_LOC 5
#define STAGE_CURSOR_COLOR_PATCH_A_LOC	13
#define STAGE_CURSOR_COLOR_PATCH_RGB	(STAGE_CURSOR_COLOR+STAGE_CURSOR_COLOR_PATCH_RGB_LOC)
#define STAGE_CURSOR_COLOR_PATCH_A	(STAGE_CURSOR_COLOR+STAGE_CURSOR_COLOR_PATCH_A_LOC)

/* array of sequences of VCBUS operations to load a cursor glyph + colors */
#define STAGE_CURSOR_GLYPH_BASE		(STAGE_CURSOR_COLOR+STAGE_CURSOR_COLOR_SIZE)
#define STAGE_CURSOR_GLYPH_SIZE		560		/* size of sequence */
#define STAGE_CURSOR_GLYPH(ID)		(STAGE_CURSOR_GLYPH_BASE+(STAGE_CURSOR_GLYPH_SIZE*(ID)))
#define STAGE_CURSOR_CHUNK_SIZE		70		/* size of sub-sequence */
#define STAGE_CURSOR_CHUNK(ID,CHUNK)	(STAGE_CURSOR_GLYPH(ID)+(STAGE_CURSOR_CHUNK_SIZE*(CHUNK)))
#define STAGE_CURSOR_LOC		5
#define STAGE_CURSOR_PATCH(ID,CHUNK)	(STAGE_CURSOR_CHUNK(ID,CHUNK)+STAGE_CURSOR_LOC)

#define STAGE_CURSOR_CHUNK_ZERO_BASE	(STAGE_CURSOR_GLYPH(VENICE_MAX_CURSOR_ID+1))
#define STAGE_CURSOR_CHUNK_ZERO(CHUNK)	(STAGE_CURSOR_CHUNK_ZERO_BASE+(STAGE_CURSOR_CHUNK_SIZE*(CHUNK)))

#define STAGE_DISPMODE_BASE		(STAGE_CURSOR_CHUNK_ZERO(9))
#define STAGE_DISPMODE_SIZE		9		/* size of sequence */
#define STAGE_DISPMODE_PATCH_LOC1	3		/* offset to patch */
#define STAGE_DISPMODE_PATCH_LOC2	7		/* offset to patch */
#define STAGE_DISPMODE(WID)		(STAGE_DISPMODE_BASE+(STAGE_DISPMODE_SIZE*(WID)))
#define STAGE_DISPMODE_PATCH1(WID)	(STAGE_DISPMODE(WID)+STAGE_DISPMODE_PATCH_LOC1)
#define STAGE_DISPMODE_PATCH2(WID)	(STAGE_DISPMODE(WID)+STAGE_DISPMODE_PATCH_LOC2)
/*
 * DAC clock must be reset on the VLIST each time video is disabled/enabled 
 * (e.g. after setmon loads a new video format).
 */
#define STAGE_DAC_CLK_RESET		STAGE_DISPMODE((VENICE_WID_CNT+2))
#define STAGE_DAC_CLK_RESET_SIZE	5

#define STAGE_NOP			(STAGE_DAC_CLK_RESET+STAGE_DAC_CLK_RESET_SIZE)
#define STAGE_NOP_SIZE			1

/*
 * Gamma is loaded in 16 separate chunks (each VLIST function chunk is
 * 70 bytes in size, for a total of 1120 bytes).
 */
#define STAGE_GAMMA_BASE		(STAGE_NOP+STAGE_NOP_SIZE)
#define STAGE_GAMMA_SIZE		1120
#define STAGE_GAMMA_CHUNK_SIZE		70
#define STAGE_GAMMA_CHUNK(CHUNK)	(STAGE_GAMMA_BASE+(STAGE_GAMMA_CHUNK_SIZE*(CHUNK)))
#define STAGE_GAMMA_LOC			5
#define STAGE_GAMMA_PATCH(CHUNK)	(STAGE_GAMMA_CHUNK(CHUNK)+STAGE_GAMMA_LOC)

/*
 * Stereo must be flipped every retrace on the permanent VLIST; we'll use the
 * 'WAIT' instruction to inhibit further VLIST processing until the next 
 * retrace, so that the swap won't complete until both (left, right) buffers 
 * have had a chance to get displayed.
 *
 * We have to patch the contents of the stereo VLIST subroutine each time the
 * IE_CONFIG_LO register changes (due to the VENICE_SET_IE_CONFIG_LO ioctl),
 * so we define the appropriate offsets & patch subroutines here.
 */
#define STAGE_STEREO			(STAGE_GAMMA_BASE+STAGE_GAMMA_SIZE)
#define STAGE_STEREO_SIZE		26		/* size of sequence */
#define STAGE_STEREO_PATCH_LOC1		7		/* offset to patch */
#define STAGE_STEREO_PATCH_LOC2		11		/* offset to patch */
#define STAGE_STEREO_PATCH_LOC3		20		/* offset to patch */
#define STAGE_STEREO_PATCH_LOC4		24		/* offset to patch */
#define STAGE_STEREO_PATCH1		(STAGE_STEREO+STAGE_STEREO_PATCH_LOC1)
#define STAGE_STEREO_PATCH2		(STAGE_STEREO+STAGE_STEREO_PATCH_LOC2)
#define STAGE_STEREO_PATCH3		(STAGE_STEREO+STAGE_STEREO_PATCH_LOC3)
#define STAGE_STEREO_PATCH4		(STAGE_STEREO+STAGE_STEREO_PATCH_LOC4)

/*
 *
 */
#define STAGE_SET_VOF_CNTRL		(STAGE_STEREO+STAGE_STEREO_SIZE)
#define STAGE_SET_VOF_CNTRL_SIZE	3
#define	STAGE_SET_VOF_CNTRL_PATCH	(STAGE_SET_VOF_CNTRL+1)

/* vc routine to clear ready right after swap */
#define STAGE_CLEAR_READY		(STAGE_SET_VOF_CNTRL + STAGE_SET_VOF_CNTRL_SIZE)
#define STAGE_CLEAR_READY_SIZE		3


/* tags for building VCBUS display-lists in SRAM (hlist, vlist, staging area) */
#define STAGE_TAG_DATA(DATA)       	(DATA)
#define STAGE_TAG_ADDR(ADDR)       	((ADDR)|0x40000000)
#define STAGE_TAG_CALL(ADDR)       	((ADDR)|0x80000000)
#define STAGE_TAG_RETURN           	0xc0000000
#define STAGE_TAG_WAIT           	0xe0000000

/*
 * VENICE MP slave interface offsets and descriptions
 */
#define VENICE_BOARD_ID		(0x0000>>2)	/* board ID offset	    */
#define   VENICE_BOARD_ID_VALUE	  0x000740ee	  /* board ID value	    */
#define VENICE_MPG_ID		(0x0004>>2)	/* chip ID offset	    */
#ifdef MPG1_B
#define   VENICE_MPG_ID_VALUE	  0x00000001	  /* MPG1 ID value	    */
#else
#define   VENICE_MPG_ID_VALUE	  0x00000000	  /* MPG1 ID value	    */
#endif
#define VENICE_RESET		(0x0008>>2)	/* reset register offset    */
#define VENICE_SLAVE_LE		(0x000c>>2)	/* little endian	    */
#define VENICE_KERNEL_LE	(0x0010>>2)	/* scatter-gather endian    */
#define VENICE_IDMA_CONTROL	(0x0100>>2)	/* how to stop an IDMA	    */
#define   VENICE_IDMA_CPCMD	      0x0ff	  /* forced CP cmd with intr*/
#define   VENICE_IDMA_UNSTOPPABLE     0x400	  /* is it OK to intr       */
#define   VENICE_IDMA_PASSTHRU	      0x800	  /* enter passthru mode    */
#define VENICE_IDMA_SEQID	(0x0104>>2)	/* IDMA sequence ID 	    */
#define   VENICE_IDMA_SEQID_MASK      0xffff	  /* only 16 bits valid	    */
#define VENICE_ODMA_COUNT	(0x0200>>2)	/* ODMA word count	    */
#define   VENICE_ODMA_INTR_ENABLE   0x800000	  /* Interrupt enable	    */
#define VENICE_VIDEO0_INTR	(0x0300>>2)	/* configure video intr 0   */
/* due to MPG design oversights, the following (VIDEO1) should not be used */
#define VENICE_VIDEO1_INTR	(0x0304>>2)	/* configure video intr 1   */
/* due to MPG design oversights, the previous (VIDEO1) should not be used */
#define VENICE_FIFO_HI_INTR	(0x0308>>2)	/* configure hi water intr  */
#define VENICE_FIFO_LO_INTR	(0x030c>>2)	/* configure lo water intr  */
#define VENICE_FIFO_OVER_INTR	(0x0310>>2)	/* configure OVERFLOW intr  */
#define VENICE_GEF_INTR		(0x0318>>2)	/* configure GEF intr	    */
#define VENICE_GEF_INTR_CLEAR	(0x031c>>2)	/* clear GEF intr (protocol)*/
#define VENICE_ODMA_INTR	(0x0320>>2)	/* configure ODMA intr	    */
#define VENICE_FIFO_HI		(0x0500>>2)	/* configure hi water mark  */
#define VENICE_FIFO_LO		(0x0504>>2)	/* configure lo water mark  */
#define VENICE_FIFO_READ	(0x0508>>2)	/* fifo read address	    */
#define VENICE_FIFO_WRITE	(0x050c>>2)	/* fifo write address	    */
#define VENICE_WORDS_IN_FIFO	(0x0510>>2)	/* how full is the fifo?    */
#define VENICE_3WAY_COUNT	(0x0600>>2)	/* 3way count for cxt switch*/
#define VENICE_VCWRITE_ADDR	(0x0700>>2)	/* vcbus write address	    */
#define VENICE_VCWRITE_DATA	(0x0704>>2)	/* vcbus write data	    */
#define VENICE_VCWRITE_FIFO	(0x0708>>2)	/* vcbus write fifo data    */
#define VENICE_VCREAD_DATA	(0x0018>>2)	/* vcbus read data	    */
#define VENICE_VCREAD_STATUS	(0x001c>>2)	/* vcbus status		    */
#define   VENICE_VC_READ_VALID	    0x00000001	  /* read data valid	    */
#define   VENICE_VC_FIFO_NOTFULL    0x00000002	  /* VC fifo not full	    */
#define VENICE_VC_STATUS	VENICE_VCREAD_STATUS	/* XXX OBSOLETE NAME */
#define VENICE_VCREAD_ADDR	(0x0720>>2)	/* vcbus read address	    */
#define VENICE_VCREAD_TRIGGER	(0x0724>>2)	/* vcbus read trigger	    */
#define VENICE_UBUS_ADDR0	(0x0740>>2)	/* low 8 bits of EEPROM addr*/
#define VENICE_UBUS_ADDR1	(0x0744>>2)	/* bits 8-15 of EEPROM addr */
#define VENICE_UBUS_ADDR2	(0x0748>>2)	/* jtag ctrl & upper EEPROM */
#define   VENICE_ADDR2_EEPROM_MASK    0x03	  /* bits 16-17 EEPROM addr */
#define VENICE_UBUS_CONTROL	(0x0748>>2)	/* jtag ctrl & upper EEPROM */
#define   VENICE_CTL_JSEL_RM0	     0x00	  /* select RM 0 chain	    */
#define   VENICE_CTL_JSEL_RM1	     0x04	  /* select RM 1 chain	    */
#define   VENICE_CTL_JSEL_RM2	     0x08	  /* select RM 2 chain	    */
#define   VENICE_CTL_JSEL_RM3	     0x0c	  /* select RM 3 chain	    */
#define   VENICE_CTL_JSEL_DG	     0x10	  /* select DG chain	    */
#define   VENICE_CTL_JSEL_GE	     0x30	  /* select GE chain	    */
#define   VENICE_CTL_JTAG_SHIFT1     0x80	  /* shift 1 bit not 8	    */
#define   VENICE_CTL_JTAG_TESTMODE   0x40	  /* test mode		    */
#define VENICE_UBUS_EEPROM_DATA	(0x074C>>2)	/* EEPROM read/write data   */
#define   VENICE_EEPROM_SIZE	    (1<<18)	  /* size of EEPROM	    */
#define VENICE_UBUS_JTAG_DATA	(0x0750>>2)	/* jtag shift data          */
#define VENICE_UBUS_JTAG_TMS	(0x0754>>2)	/* jtag TMS data            */
#define VENICE_UBUS_STATUS	(0x0758>>2)	/* board rev & jtag status  */
#define   VENICE_SR_JTAG_GE_TMS	     0x10	  /* ge tms bit             */
#define   VENICE_SR_JTAG_TB_TMS	     0x20	  /* tbus tms bit           */
#define   VENICE_SR_JTAG_TDO	     0x30	  /* ctrler TDO bit         */
#define   VENICE_SR_JTAG_READY	     0x80	  /* controller ready       */
#define   VENICE_SR_BD_ID_POS		0
#define   VENICE_SR_JTAG_GE_TMS_POS	4
#define   VENICE_SR_JTAG_TB_TMS_POS	5
#define   VENICE_SR_JTAG_TDO_POS	6
#define   VENICE_SR_JTAG_READY_POS	7
#define   VENICE_SR_BD_ID_MASK	     0x0f	  /* board revision         */
#define     VENICE_ID_8GES	      0xf	     /* set by jumpers	    */
#define     VENICE_ID_4GES	      0xe	     /* to determine GE	    */
#define     VENICE_ID_2GES	      0xc	     /* count at init time  */
#define     VENICE_ID_1GE	      0x1
#define VENICE_RETRACE_SEQID	(0x0800>>2)	/* retrace seqid for vlist  */
#define VENICE_FIFO		(0x2000>>2)	/* input fifo; user mapped  */

/* interrupt configuration registers from above all use these bit defs */
#define VENICE_INTR_VECTOR_POS	    0		/* interrupt status	    */
#define VENICE_INTR_VECTOR_MASK	    0x000000ff
#define VENICE_INTR_DST_POS	    8		/* interrupt destination    */
#define VENICE_INTR_DST_MASK	    0x00000f00
#define VENICE_INTR_ENABLE_POS	    12		/* enable interrupt	    */
#define VENICE_INTR_ENABLE_MASK	    0x00001000

/* macro to generate interrupt configuration registers */
#define VENICE_INTR_CONFIG(vector,dst,enable)\
	(((vector) & VENICE_INTR_VECTOR_MASK) |\
	(((dst) << VENICE_INTR_DST_POS) & VENICE_INTR_DST_MASK) |\
	(((enable) << VENICE_INTR_ENABLE_POS) & VENICE_INTR_ENABLE_MASK))

/* with idma configuration being a shifted exception */
#define VENICE_IDMA_COUNT_POS		0
#define VENICE_IDMA_COUNT_MASK		0x0000ffff
#define VENICE_IDMA_INTR_VECTOR_POS	16
#define VENICE_IDMA_INTR_VECTOR_MASK	0x00ff0000
#define VENICE_IDMA_INTR_DST_POS	24
#define VENICE_IDMA_INTR_DST_MASK	0x0f000000
#define VENICE_IDMA_INTR_ENABLE_POS	31
#define VENICE_IDMA_INTR_ENABLE_MASK	0x80000000

#define VENICE_IDMA_INTR_CONFIG(count,vector,dst,enable)\
	(((count) & VENICE_IDMA_COUNT_MASK) |\
	(((vector) << VENICE_IDMA_INTR_VECTOR_POS) & VENICE_IDMA_INTR_VECTOR_MASK) |\
	(((dst) << VENICE_IDMA_INTR_DST_POS) & VENICE_IDMA_INTR_DST_MASK) |\
	(((enable) << VENICE_IDMA_INTR_ENABLE_POS) & VENICE_IDMA_INTR_ENABLE_MASK))

#define VENICE_FIFO_VECTOR_NUM		0
#define VENICE_IDMA_VECTOR_NUM		1
#define VENICE_ODMA_VECTOR_NUM		2
#define VENICE_VIDEO_VECTOR_NUM		3
#define VENICE_GEF_VECTOR_NUM		4
#define VENICE_NUM_INTR_VECTORS		5


#ifdef _KERNEL
/*
 * interrupt definitions for VENICE:
 * VENICE MP supports 5 independent interrupt vectors.  we map the 7 interrupt
 * sources to the 5 vectors.  we can do this since we can make assumptions about
 * the relationship of the FIFO related sources (hi, lo, over).
 */
#define VENICE_INTR_VECTOR(NUM, PIPE)	((PIPE) ? (NUM)+4 : (NUM)+9)
#define VENICE_HIWATER_VECTOR(PIPE)	((VENICE_INTR_VECTOR(VENICE_FIFO_VECTOR_NUM, PIPE) << 4) | 0x0)
#define VENICE_LOWATER_VECTOR(PIPE)	((VENICE_INTR_VECTOR(VENICE_FIFO_VECTOR_NUM, PIPE) << 4)| 0x1)
#define VENICE_OVERWATER_VECTOR(PIPE)	((VENICE_INTR_VECTOR(VENICE_FIFO_VECTOR_NUM, PIPE) << 4)| 0x2)
#define VENICE_IDMA_VECTOR(PIPE)	(VENICE_INTR_VECTOR(VENICE_IDMA_VECTOR_NUM, PIPE) << 4)
#define VENICE_ODMA_VECTOR(PIPE)	(VENICE_INTR_VECTOR(VENICE_ODMA_VECTOR_NUM, PIPE) << 4)
#define VENICE_ALTVIDEO_VECTOR(PIPE)	((VENICE_INTR_VECTOR(VENICE_VIDEO_VECTOR_NUM, PIPE) << 4) | 0x00)
#define VENICE_VIDEO_VECTOR(PIPE)	((VENICE_INTR_VECTOR(VENICE_VIDEO_VECTOR_NUM, PIPE) << 4) | 0x01)
#define VENICE_GEF_VECTOR(PIPE)		(VENICE_INTR_VECTOR(VENICE_GEF_VECTOR_NUM, PIPE) << 4)
#define  VENICE_GEF_INTR_ALIVE		0	/* the GE is alive/dead... */
#define  VENICE_GEF_INTR_PRINTF		1	/* printf from GE */
#define  VENICE_GEF_INTR_DEACT		2	/* deactivation complete */
#define  VENICE_GEF_INTR_HEAVY		3	/* heavyweight op complete */
#define  VENICE_GEF_INTR_DMA		4	/* dma completion */
#define  VENICE_GEF_INTR_PICKFEED	5	/* pickfeed completion */
#define  VENICE_GEF_INTR_VCSTAGE	6	/* staging area modification complete */
#define  VENICE_GEF_INTR_CURSOR_SELECT	7	/* fully selected cursor */

/* timeout IDs and values */
#define VENICE_TIMEOUT				(15*HZ)
#define VENICE_FIFO_TIMEOUT_TOKEN		0
#define VENICE_FIFO_TIMEOUT_VALUE		(15*HZ)
#define VENICE_VIDEO_TIMEOUT_TOKEN		1
#define VENICE_VIDEO_TIMEOUT_VALUE		VENICE_TIMEOUT
#define VENICE_DEACT_TIMEOUT_TOKEN		2
#define VENICE_DEACT_TIMEOUT_VALUE		VENICE_TIMEOUT
#define VENICE_IDMA_TIMEOUT_TOKEN		3
#define VENICE_IDMA_TIMEOUT_VALUE		VENICE_TIMEOUT
#define VENICE_HEAVY_TIMEOUT_TOKEN		4
#define VENICE_HEAVY_TIMEOUT_VALUE		VENICE_TIMEOUT
#define VENICE_DMAWRITE_TIMEOUT_TOKEN		5
#define VENICE_DMAWRITE_TIMEOUT_VALUE		VENICE_TIMEOUT
#define VENICE_PICKFEED_TIMEOUT_TOKEN		7
#define VENICE_PICKFEED_TIMEOUT_VALUE		VENICE_TIMEOUT
#define VENICE_VCSTAGE_TIMEOUT_TOKEN		8
#define VENICE_VCSTAGE_TIMEOUT_VALUE		VENICE_TIMEOUT
#define VENICE_CURSOR_SELECT_TIMEOUT_TOKEN	9
#define VENICE_CURSOR_SELECT_TIMEOUT_VALUE	VENICE_TIMEOUT
#define VENICE_DIAG_INTR_TIMEOUT_TOKEN		10

struct venice_subtexload_args {
   struct ulamslot slot;
};

/*
 * the following structure is utilized to reference real
 * hardware registers.
 */
struct venice_hw_reg {
	unsigned int reg;
};

/*
 * MPG input DMA expects the following structure.
 */
struct venice_idmavec {
	caddr_t dmav_base;		/* physical address of data block */
	int     dmav_len;		/* length of data block in words */
};

/*
 * MPG output DMA expects the following structure.
 */
struct venice_odmavec {
	caddr_t dmav_base;		/* physical address of data block */
};

/*
 * Structure to keep track of what textures are loaded in
 * the graphics texture memory.  It is also used for graphics
 * texture memory allocation.
 */
typedef struct texshadow {
	long  addr;		/* physical address in hw texture memory */
	long  state;		/* state of segment */
	long  reverse;		/* had to reverse banks from what GL expected */
	long  size;		/* size of this segment in 16 bit quads */ 

	unsigned long* vaddr;   /* User addr of the texture map */
	long  npixels;		/* Size of user texture map in 32 bit words */

	/* Next fields are needed for raster order download */
	long  download_style;
	long  width;
	long  height;
	long  texel_size; /* XXX should be a function of dst_texel_format */
	long src_texel_format;
	long dst_texel_format;

	long xsrc;
	long ysrc;

	/* for the packer */
	long src_width;
	long src_height;
	long src_eol_offset;

	caddr_t uaddr;		/* Uaddr of the corresponding ulamslot struct */
	struct gfx_gfx *gfxp;	/* gfxp struct of owner if ACTIVE/INACTIVE */
	struct rrm_rnode *rnp;	/* Pointer to rendering node structure */
	struct texshadow *next;	/* pointer to next segment */
	struct texshadow *prev;	/* pointer to prev segment */
} texshadow_t;

/*
 * Structure to shadow contents of write-only DG2 registers accessible only 
 * from the backend of VENICE graphics.
 */
typedef struct {
	/*
	 * ieconfiglo is a shadow copy of the ie config register at addr 40, 
	 * and contains the global stereo, global underlay, and pixel density 
	 * (send alpha, send 10 bit component) control bits.  This gets set by 
	 * the VENICE_LOAD_IE_CONFIG_LO ioctl.  If the global stereo flag is 
	 * set, a permanent VLIST subroutine is enabled which will flip the 
	 * state of this bit every vertical retrace interval; interrupts will 
	 * occur half as often when this is happening.
	 */
	int	ieconfiglo;		/* shadow of ie config reg 40 */
	int	initreg;		/* V_DG_INITREG: HWALK, VWALK enables */
	int	interruptmask;		/* V_DG_INTERRUPT_MASK */
	int	vofctrlreg;		/* V_DG_VOF_CNTRL: screen blank, etc */
	int	rgbdac_cmd0;		/* RGB DAC's command register 0 */
	int	rgbdac_cmd1;		/* RGB DAC's command register 1 */
	/*
	 *
	 *VFC Video control register.
	 *
	 */

	int	videoctrlreg;		/*	V_DG_VIDEO_CONTROL */
	/*
	 * Note: you must reset the DAC clocks on the VLIST whenever you 
	 * twiddle bits in rgbdac_cmd2.
	 */
	int	rgbdac_cmd2;		/* RGB DAC's command register 2 */
	int	rgbdac_pixreadlo;	/* RGB DAC's pixel read mask lo */
	int	rgbdac_pixreadhi;	/* RGB DAC's pixel read mask hi */
	int	rgbdac_pixblinklo;	/* RGB DAC's pixel blink mask lo */
	int	rgbdac_pixblinkhi;	/* RGB DAC's pixel blink mask hi */
	int	rgbdac_overreadlo;	/* RGB DAC's overlay read mask lo */
	int	rgbdac_overreadhi;	/* RGB DAC's overlay read mask hi */
	int	rgbdac_overblinklo;	/* RGB DAC's overlay blink mask lo */
	int	rgbdac_overblinkhi;	/* RGB DAC's overlay blink mask hi */
	int	alphadac_cmd0;		/* ALPHA DAC's command register 0 */
	int	alphadac_cmd1;		/* ALPHA DAC's command register 1 */
	/*
	 * Note: you must reset the DAC clocks on the VLIST whenever you 
	 * twiddle bits in alphadac_cmd2.
	 */
	int	alphadac_cmd2;		/* ALPHA DAC's command register 2 */
	int	alphadac_pixreadlo;	/* ALPHA DAC's pixel read mask lo */
	int	alphadac_pixreadhi;	/* ALPHA DAC's pixel read mask hi */
	int	alphadac_pixblinklo;	/* ALPHA DAC's pixel blink mask lo */
	int	alphadac_pixblinkhi;	/* ALPHA DAC's pixel blink mask hi */
	int	alphadac_overreadlo;	/* ALPHA DAC's overlay read mask lo */
	int	alphadac_overreadhi;	/* ALPHA DAC's overlay read mask hi */
	int	alphadac_overblinklo;	/* ALPHA DAC's overlay blink mask lo */
	int	alphadac_overblinkhi;	/* ALPHA DAC's overlay blink mask hi */
	long	glvideoreg	;	/* setvideo status register shadow */
} dg2shadow_t;

struct venice_map {
	caddr_t	kvaddr;
	caddr_t	physaddr;
	struct venice_map *next;
};

struct venice_kcomm {
	long data[0x400-1-btoc(VENICE_RDATA_SIZE)];
	struct venice_odmavec kmap[1];
	struct venice_odmavec rmap[btoc(VENICE_RDATA_SIZE)];
};

struct venice_pickfeed {
	char	*buf;	/* pointer to pickfeed buffer */
	int	len;	/* length of pickfeed buffer */
	int	rem;	/* pickfeed remainder cnt */
	int	over;	/* pickfeed overflow flag */
	struct venice_odmavec *map;	/* mapping information */
	sema_t	sema;	/* pickfeed semaphore */
	int	timid;	/* pickfeed timeout id */
	int	busy;	/* busy flag */
};

/*
 * each instance of a VENICE graphics head requires one of the following
 * structures to control the head.
 */
struct venice_data {
	struct gfx_data		gfx_data;	/* device independent data */
	struct venice_info	info;		/* venice specific info */
	int			slot;		/* bus slot number */
	int			pipenum;	/* graphics pipe number */
	int			alive;
	int			mpg_idma;	/* indicates that mpg is idma capable */
	struct venice_rnode	*gang_swapper;

	sema_t			idma_sema;	/* idma semaphore */
	int			idma_timid;	/* idma timeout id */

	/* tables managed by device independent layer */
	struct rrm_rnode *loadedpcxs[VENICE_PCX_CNT];
	unsigned long loadedmodes[VENICE_WID_CNT];

	/* processor board specific stuff */
	char			*threeway;	/* physaddr of 3way hardware */

	/* general purpose spinlock */
	lock_t		spinlock;		/* for MP protection */
	lock_t		fifo_spinlock;		/* for MP protection */

	/* one way communication channel */
	struct venice_kcomm	*kcomm;		/* GE to kernel comm area */
	char			print_buf[400];	/* printf storage */

	/* vsync related data structures */
	int cur_vsync_cnt;		/* Current vsync count as given by the pipe */
	struct venice_rnode *vsync_sleep_list;

	/* fifo management parameters */
	int			fifo_timid;	/* fifo timeout id */

	/* interrupt vectors for all interrupt sources */
	int			hiwater_intr_vector;
	int			lowater_intr_vector;
	int			overwater_intr_vector;
	int			idma_intr_vector;
	int			odma_intr_vector;
	int			altvideo_intr_vector;
	int			video_intr_vector;
	int			gef_intr_vector;

	/* memory management stuff for pcx areas */
	struct venice_map	*mem_freelist;	/* locked mem freelist */
	int			mem_freecnt;	/* size of mem freelist */
	int			mem_hiwater;	/* settable hi water mark */
	int			mem_chunksize;	/* allocation granularity */
	struct venice_map	*mem_limbo;	/* queued to be freed */

	/* the retrace list keeps track of requests to video hardware */
	struct rrm_rnode	*retrace_list_new;/* newest end of list */
	struct rrm_rnode	*retrace_list_old;/* oldest end of list */
	long			retrace_seqid;	  /* unique id generator */

	/* the wait list keeps track of time with respect to video hardware */
	struct rrm_rnode	*wait_list;	/* wait list */
	long			swaptime;	/* current time (in retraces) */

	/* cursor tracking and cursor management state */
	int		cposx;		/* x cursor position */
	int		cposy;		/* y cursor position */
	int		newcpos;	/* cursor position update flag */
	int		cid;		/* cursor id on the vlist */
	int		csize[VENICE_MAX_CURSOR_ID+1];	/* size of cursor */
	int		oldcsize;	/* size of last cursor selected */
	int		newcid;		/* cursor id for the vlist */
	int		cfudgex;	/* cursor fudge factor */
	int		cfudgey;	/* cursor fudge factor */
	sema_t		cursor_select_sema; /* waition for cursor to get to chips */
	int		cursor_select_timid;/* cursor select timeout id */
	struct venice_select_cursor_args cursor_attrib;
	int		newgamma;	/* gamma table needs changing */

	/* shadow of write-only DG2 registers */
	dg2shadow_t	dg2shadow;

	/* Texture manager data structures below: */
	sema_t		texsema;	/* semaphore to serialize tex */
	long num_tex_seg; /* Number of texture segments in the system	*/
	texshadow_t	texshadow[NUM_TEXSHADOWS];
	texshadow_t*	freeptr;	/* for texture memory allocation */
	texshadow_t*	texseg[2];	/* list of structures in segment. */
					/* indexed by segment number. */
	struct venice_idmavec *texture_maps;

	/* ilbuffer state */
	long ilbuffer_use_cnt;		/* count of GL programs using ILbuffer */
	long ilbuffer_used_by_il;	/* is the IL actually using the ILbuffer? */

	caddr_t rdata;			/* standalone rdata area */
	struct venice_pickfeed pickfeed;/* standalone pickfeed info */

	struct venice_odmavec *map;	/* big shared mapper for huge odma */

	/* diagnostic support structures */
	sema_t		diag_sema;	/* diagnostic sema for intr iface */
	int		diag_timid;	/* timeout ID for intr iface */
	struct venice_diag_intr_args diag_intr_args;
	int		diag_handle_mask;

	/* pointers to VENICE hardware registers */
	volatile struct venice_hw_reg	*baseaddr;	/* base address */
	volatile struct venice_hw_reg	*mpgid;		/* mpg id register */
	volatile struct venice_hw_reg	*reset;		/* reset register */
	volatile struct venice_hw_reg	*sendian;	/* slave endian */
	volatile struct venice_hw_reg	*kendian;	/* kernel endian */
	volatile struct venice_hw_reg	*idmactrl;	/* IDMA control */
	volatile struct venice_hw_reg	*idmaseq;	/* IDMA seqid */
	volatile struct venice_hw_reg	*videointr;	/* video intr config */
	volatile struct venice_hw_reg	*altvideointr;	/* alternate video intr config */
	volatile struct venice_hw_reg	*hiintr;	/* hi intr config */
	volatile struct venice_hw_reg	*lointr;	/* lo intr config */
	volatile struct venice_hw_reg	*overintr;	/* overflow intr conf */
	volatile struct venice_hw_reg	*gefintr;	/* gef intr config */
	volatile struct venice_hw_reg	*gefclear;	/* gef intr clear */
	volatile struct venice_hw_reg	*odmaintr;	/* ODMA intr config */
	volatile struct venice_hw_reg	*fifohi;	/* hi water config */
	volatile struct venice_hw_reg	*fifolo;	/* lo water config */
	volatile struct venice_hw_reg	*fiforaddr;	/* fifo read address */
	volatile struct venice_hw_reg	*fifowaddr;	/* fifo write address */
	volatile struct venice_hw_reg	*fifocnt;	/* words in fifo */
	volatile struct venice_hw_reg	*threewaycnt;	/* 3way count */
	volatile struct venice_hw_reg	*vcwaddr;	/* vc write addr */
	volatile struct venice_hw_reg	*vcwdata;	/* vc write data */
	volatile struct venice_hw_reg	*vcraddr;	/* vc read addr */
	volatile struct venice_hw_reg	*vcrdata;	/* vc read data */
	volatile struct venice_hw_reg	*vcrtrigger;	/* vc read trigger */
	volatile struct venice_hw_reg	*vcrstatus;	/* vc read status */
	volatile struct venice_hw_reg	*gefsel;	/* selected gef */
	volatile struct venice_hw_reg	*gerev;		/* ge revision level */
	volatile struct venice_hw_reg	*seqid;		/* retrace seqid */
	volatile struct venice_hw_reg	*pipebase;	/* input pipe */
};

/*
 * note that this structure describes context information
 * as the VENICE GE writes it to host memory.  this is a
 * READ-ONLY structure as far as we are concerned.  any changes
 * to this structure should be tied to changes in GE ucode.
 */
struct venice_kpcx {
	int	heavy_cnt;	/* how many words in heavyweight context */
	int	idma_cnt;	/* how many words to idma upon activation */
	int	fifo_cnt;	/* how many words to restore to fifo */
	int	cp_pc;		/* cp saved pc... very important */
	int	cp_restart;	/* cp restart flag... very important */
	int	idma_mpaddr;	/* part of interrupted idma state */
	int	idma_mpcnt;	/* part of interrupted idma state */
	int	idma_sgaddr;	/* part of interrupted idma state */
	int	idma_sgcnt;	/* part of interrupted idma state */
	int	fifo_data[800];	/* saved fifo */
	/* 
	 * fifo allows room for (256 + 100 + 44) * 2
	 * 256 for VENICE upper FIFO size
	 * 100 for write gatherer + EBUS pipelining
	 * 44 for slop/paranoia
	 * *2 for VENICE passthru mode
	 */
};
#define VENICE_KPCX_SIZE btoc(sizeof(struct venice_kpcx))
#define VENICE_LIGHT_PCX_SIZE	16	/* max size of light cxt in pages */
#define VENICE_HEAVY_PCX_SIZE	16	/* max size of heavy cxt in pages */
#define VENICE_MAX_TEXTURE_SIZE	512	/* maz size of texture in pages */

/*
 * this structure must be allocated on page aligned boundaries.
 *
 * do not allow its size to exceed 4K since we don't want to
 * deal with the possibility of discontiguous page boundaries.
 */
struct venice_kmaps {
	/* NOTE! NOTE! keep all of the idmavecs first to preserve double alignment */
	struct venice_idmavec vcstage_in[2];
	struct venice_idmavec light_in[VENICE_LIGHT_PCX_SIZE];
	struct venice_idmavec heavy_in[VENICE_HEAVY_PCX_SIZE];

	struct venice_odmavec kpcx_out[VENICE_KPCX_SIZE];
	struct venice_odmavec light_out[VENICE_LIGHT_PCX_SIZE];
	struct venice_odmavec heavy_out[VENICE_HEAVY_PCX_SIZE];
	struct venice_odmavec rdata_out[btoc(VENICE_RDATA_SIZE)];
};

/*
 * Device DEPENDENT rendering node structure
 *
 * the following structure comprises the overall context of a
 * graphics context (as far as the kernel is concerned).  this
 * includes the IDMA state and FIFO context as well as all the
 * mapping information for a given context.
 */
struct venice_rnode {
	/* maps for various pieces of the context (allocated page aligned) */
	struct venice_kmaps	*kmaps;

	/* kernel portion of VENICE context (written by GE, cachealigned) */
	struct venice_kpcx	*kpcx;

	long			threewaycnt;	/* saved 3way count */
#ifdef IP17
	unsigned long 		ip17p3way_cntcmdreg; /* only 16 bits are valid */
	unsigned long		ip17p3way_ABCDregs[4];
#endif
	texshadow_t*		texshadow[2];	/* ptrs to tex shadow lists */
	struct RRM_ValidateClip validateclip;	/* clip info for activate */
	struct RRM_PieceList	piece[VENICE_PIECELIST_SIZE];
	long			dispmode;	/* deferred display mode */

	long			ilbuffer_use;	/* 1 if using ilbuffer */

	/* round trips thru pipe from kernel land use the following */
	sema_t			misc_sema;	/* misc sema for round trips */
	int			misc_timid;	/* misc timeout for round trips */

	sema_t			vcstage_sema;	/* waiting for cursor to get to sram */
	int			vcstage_timid;	/* cursor load timeout id */

	/* vsync related data structures */
	sema_t			vsync_sema;
	struct venice_rnode 	*vsync_sleep_next;/* Points to next guy sleeping on a vsync */
	int			vsync_interval;	/* Number of vertical retraces before a wakeup */
	int			vsync_mod;	/* */

	/* lightweight context storage and control */
	struct venice_map	*light_map;	/* light pcx map list */
	sema_t			deact_sema;	/* deactivate sema */
	int			deact_timid;	/* deactivate timeout id */

	/* heavyweight context storage and control */
	struct venice_map	*heavy_map;	/* heavy pcx map list */
	sema_t			heavy_sema;	/* heavy context sema */

	caddr_t			rdata;		/* per context rdata area */
	struct venice_pickfeed	pickfeed;	/* per context pickfeed info */

	struct rrm_rnode	*retrace_next;	/* next guy on list */
	void			(*retrace_func)();/* completion function */
	int			retrace_seqid;	/* seqid to figure completions */
	int			retrace_timid;	/* timeout id for retrace */

	struct rrm_rnode	*wait_next;	/* next guy on wait list */
	void			(*wait_func)();	/* wait completion function */
};

#endif /* _KERNEL */

#define LU_CP_LOAD              0x00000001

#define LU_GE_BOOTSTRAP         0x00000002
#define LU_GE_LOAD_PB           0x00000004
#define LU_GE_LOAD_DRAM_TEXT    0x00000008
#define LU_GE_LOAD_DRAM_DATA    0x00000010
#define LU_GE_RUN               0x00000020
#define LU_GE_BROADCAST         0x00000040
#define LU_GE_SWAPB             0x00000080

#define LU_COMPARE              0x10000000
#define LU_FIFO_FLOW_CNTL       0x20000000

#endif /* _VENICE_H_ */
