/**************************************************************************
 *									  *
 * 		 Copyright (C) 1988, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/
#ident "$Header: /bvd/att/usr/src/uts/mips/sys/RCS/vigra_kern.h,v 1.1 1992/06/17 17:01:02 keng Exp $"

/*
**  vigra_kern.h - Hollywood DSP device driver internal structs.
**  (programmer visible vigra structs are in vigra.h).
*/

/* The following struct describes what the vigra driver knows about a host ring
   buffer.  */

typedef struct hrb_s {
    ulong *vheader;	 /* Virtual address of the header part of the hrb */
    caddr_t pheader;	 /* Physical address of the header part of the hrb */
    void *vdata;	 /* Virtual address of the data part of the hrb */
    caddr_t pdata;	 /* Physical address of the data part of the hrb */
    caddr_t vaddr;	 /* Virtual address of the data in the user's address
			    space */
    int nlocs;		 /* Number of locations in the data part of the hrb */
    int nbytes;		 /* Number of bytes in the hrb, including the header */
    int minordev;	 /* Minor device number associated with this hrb. */
    void (*hrb_isr)();	 /* Pointer to the interrupt service routine. */
    struct hrb_s *next;  /* Pointer to the next free or allocated hrb struct */
    char type;		 /* Type of hrb (TOMIPS, FROMMIPS, FOLDED, etc.) */
    struct pollqueue pq; /* Poll queue (for select, poll) */
    char state;		 /* State of the hrb. */
    char rbnum;		 /* Ring Buffer Number */
    char dspnotified;	 /* True if the DSP has been notified of the hrb */
    int maplen;		 /* Length of the mmaped region */
}hrb_t;

/* The following defines indicate the state of a host ring buffer.  */

#define HRB_WAITING 1

/* The following flags controll the behavior of kernel writes to the dsp */
/*	-- needed by streams drivers to prevent sleeping */

#define HDSPSLEEPOK 0
#define HDSPNOSLEEP 1

/* The following macros are convenient ways to query the type of a host ring
   buffer. */

#define HRB_ISFOLDED(x)		(((x)&HRB_FOLDED)!=0)
#define HRB_ISFROMMIPS(x)	(((x)&HRB_FROMMIPS)!=0)
#define HRB_ISEXTEND(x)		(((x)&HRB_EXTEND)!=0)
#define HRB_ISUSERSPACE(x)	(((x)&HRB_USERSPACE)!=0)

#define HRB_ISUNFOLDED(x)	(((x)&HRB_FOLDED)==0)
#define HRB_ISTOMIPS(x)		(((x)&HRB_FROMMIPS)==0)
#define HRB_ISNOEXTEND(x)	(((x)&HRB_EXTEND)==0)
#define HRB_ISKERNELSPACE(x)	(((x)&HRB_USERSPACE)==0)

/* limits */
#define HDSPMEMORYHI	65536	/* In DSP Words. */
#define HDSPMEMORYLOW	32768	/* In DSP Words. */
#define MAXHRB 30

/* The following struct describes a Hollywood DSP on a HPC1 controller.  */

typedef struct vigra_s {
	volatile unsigned int   state;
	volatile unsigned long *dmawdcount;
	volatile unsigned long *gioaddl;
	volatile unsigned long *gioaddm;
	volatile unsigned long *pbusadd;
	volatile unsigned long *dmactl;
	volatile unsigned long *handtx;
	volatile unsigned long *handrx;
	volatile unsigned long *cintstat;
	volatile unsigned long *cintmask;
	volatile unsigned long *miscsr;
	volatile unsigned long *burstctl;
	volatile unsigned long *memory;
	hrb_t *cmdin;	/* Commands from the DSP */
	hrb_t *cmdout;  /* Commands to the DSP */
	int input_source;
	int left_input_atten;
	int right_input_atten;
	int input_rate;
	int output_rate;
	int left_speaker_gain;
	int right_speaker_gain;
	int inputrbcount;	/* Count of input (to mips) ring buffers */
	int outputrbcount;	/* Count of output (from mips) ring buffers */
} vigra_t;

/* The following defines indicate the state of the vigra */

#define HDSP_INITIALIZING 1

/* The following procedures place and remove data to/from host ring buffers */

void hrb_dsptomipsfolded_isr(hrb_t*);
void hrb_dsptomipsunfolded_isr(hrb_t*);
void hrb_mipstodspfolded_isr(hrb_t*);
void hrb_mipstodspunfolded_isr(hrb_t*);

void hrb_user_dsptomipsfolded_isr(hrb_t*);
void hrb_user_dsptomipsunfolded_isr(hrb_t*);
void hrb_user_mipstodspfolded_isr(hrb_t*);
void hrb_user_mipstodspunfolded_isr(hrb_t*);

/* Hollywood DSP commands. */

#define DSPCMD_CREATEHRB		1
#define DSPCMD_ACQUIRE_SAMPLERB		4
#define DSPCMD_FREE_SAMPLERB		5
#define DSPCMD_CHANGE_SAMPLE_RB		6
#define DSPCMD_CHANGE_AUDIO_PARMS	7

/* Client locks state */

#define HDSP_CLIENT_UNOPENED	(-2)
#define HDSP_CLIENT_OPENED	(-1)

/* Priority that the DSP blocks interrupts at. */

#define splvigra()	splhi()

/* HPC1 CINTMASK register bits */

#define HDSP_CINTMASK_DMA       1
#define HDSP_CINTMASK_HANDTX    2
#define HDSP_CINTMASK_HANDRX    4

/* Configuration constants */

#define CMDBUFSIZE   300
#define REPLYBUFSIZE 20
#define MAXHDSPCLIENTS	12
#define HDSPSPRI (PZERO-1)
