/**************************************************************************
 *									  *
 * 		 Copyright (C) 1989, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/

#ifndef __TAR_H__
#define __TAR_H__
#ident "$Revision: 1.8 $"

#include <sys/types.h>

#define	TBLOCK	512	/* size of a tape block */

#define	FCHR	0020000
#define	FBLK	0060000
#define	FIFO	0010000

/* even for SVR3 must keep NAMSPACE 100 and not MAXPATHLEN because TAR
 * tape block size written in stone (TBLOCK)
 */
#define	NAMSPACE	100	/* total storage, including terminating NULL */
#define NAMSIZ		(NAMSPACE-1)	/* max # of 'usable' chars */

#define	PREFIXSPACE	156		/* POSIX: prefix space including NULL */
#define	PREFIXSIZ	(PREFIXSPACE-1)

union hblock {
	char dummy[TBLOCK];
	struct header {
		char name[NAMSPACE];
		char mode[8];
		char uid[8];
		char gid[8];
		char size[12];
		char mtime[12];
		char chksum[8];
		char linkflag;
		char linkname[NAMSPACE];
		char rdev[12];
	} dbuf;
	struct pheader {				/* POSIX */
		char name[NAMSPACE];			/* POSIX */
		char mode[8];				/* POSIX */
		char uid[8];				/* POSIX */
		char gid[8];				/* POSIX */
		char size[12];				/* POSIX */
		char mtime[12];				/* POSIX */
		char chksum[8];				/* POSIX */
		char typeflag;				/* POSIX */
		char linkname[NAMSPACE];		/* POSIX */
		char magic[6];				/* POSIX */
		char version[2];			/* POSIX */
		char uname[32];				/* POSIX */
		char gname[32];				/* POSIX */
		char devmajor[8];			/* POSIX */
		char devminor[8];			/* POSIX */
		char prefix[PREFIXSPACE];		/* POSIX */
	} pdbuf;					/* POSIX */
};
/* #define linkflag typeflag */

struct linkbuf {
	struct	linkbuf *nextp;
	ino_t	inum;
	dev_t	devnum;
	short	count;
	char	pathname[1];	/* actually alloced larger */
};

#define TMAGIC		"ustar"	/* ustar and a null */
#define TMAGLEN		6
#define TVERSION	"00"	/* 00 and no null */
#define TVERSLEN	2

/* values used in typeflag field */
#define REGTYPE		'0'	/* regular file	*/
#define AREGTYPE	'\0'	/* regular file	*/
#define LNKTYPE		'1'	/* link */
#define SYMTYPE		'2'	/* Reserved */
#define CHRTYPE		'3'	/* Character special */
#define BLKTYPE		'4'	/* Block Special */
#define DIRTYPE		'5'	/* Directory */
#define FIFOTYPE	'6'	/* FIFO special */
#define CONTTYPE	'7'	/* Reserved */

/* Bits used in the mode field - values in octal */
#define	TSUID	04000		/* set UID on execution */
#define	TSGID	02000		/* set GID on execution */
#define	TSVTX	01000		/* reserved */
#define	TUREAD	00400		/* read by owner */
#define	TUWRITE	00200		/* write by owner */
#define	TUEXEC	00100		/* execute/search by owner */
#define	TGREAD	00040		/* read by group */
#define	TGWRITE	00020		/* write by group */
#define	TGEXEC	00010		/* exec/search by group */
#define	TOREAD	00004		/* read by other */
#define	TOWRITE	00002		/* write by other */
#define	TOEXEC	00001		/* execute/search by other */

#endif /* __TAR_H__ */
