#ifndef __TERMIOS_H__
#define __TERMIOS_H__
#ifdef __cplusplus
extern "C" {
#endif
 /*************************************************************************
 #									  *
 # 		 Copyright (C) 1989, Silicon Graphics, Inc.		  *
 #									  *
 #  These coded instructions, statements, and computer programs  contain  *
 #  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 #  are protected by Federal copyright law.  They  may  not be disclosed  *
 #  to  third  parties  or copied or duplicated in any form, in whole or  *
 #  in part, without the prior written consent of Silicon Graphics, Inc.  *
 #									  *
 #************************************************************************/

#ident	"$Revision: 1.7 $"

#include <sys/types.h>
#include <sys/termio.h>

#define VSUSP	VSWTCH

#define TCSANOW		0	/* change term attributes NOW */	
#define TCSADRAIN	1	/* change after current output is written */
#define TCSAFLUSH	2	/* as DRAIN, but discard pending input */

/* For tcflush sys call, designate which queue to flush */
#define TCIFLUSH	0
#define TCOFLUSH	1
#define TCIOFLUSH	2

/* For tcflow sys call, following defines specify on/off, and input/output */
#define TCOOFF		0
#define TCOON		1
#define TCIOFF		2
#define TCION		3

typedef uint	speed_t;

#define TO_STOP	LOBLK	/* send SIGTTOU for background output */

/*
 * POSIX termios control structure.
 * Note that this structure is identical with "struct termio"
 * defined in <sys/termio.h> - should there be need to change
 * one or the other, the POSIX compatibility functions
 * tcgetattr and tcsetattr (at least) will have to be
 * changed.
 */
struct termios {
	tcflag_t	c_iflag;	/* input modes */
	tcflag_t	c_oflag;	/* output modes */
	tcflag_t	c_cflag;	/* control modes */
	tcflag_t	c_lflag;	/* line discipline modes */
	char		c_line;		/* line disc. selector */
	cc_t		c_cc[NCCS];	/* control chars */
};

/* Two extra POSIX lflags -- other lflags are in <sys/termio.h> */
#define IEXTEN	IIEXTEN		/* !!!!!NOT IMPLEMENTED YET!!!!! */
#define TOSTOP	ITOSTOP

extern speed_t cfgetospeed(struct termios *);
extern int cfsetospeed(struct termios *, speed_t);
extern speed_t cfgetispeed(struct termios *);
extern int cfsetispeed(struct termios *, speed_t);
extern int tcgetattr(int, struct termios *);
extern int tcsetattr(int, int, struct termios *);
extern int tcsendbreak(int, int);
extern int tcdrain(int);
extern int tcflush(int, int);
extern int tcflow(int, int);

#ifdef __cplusplus
}
#endif
#endif /* !__TERMIOS_H__ */
